import mpi.*;

public class PiExamplePt2pt {
  public PiExamplePt2pt() {} 

  public PiExamplePt2pt(String[] args) throws Exception {
    int intervals[] = new int[1];
    int rank, tasks, i = 0;  
    final double PI25DT = 3.141592653589793238462643; 
    double mypi[] = new double[1];
    double pi[] = new double[1];
    double h, sum, x;

    MPI.Init(args);
    rank = MPI.COMM_WORLD.getRank();
    tasks = MPI.COMM_WORLD.getSize();
    
    if (args[0] == null) {
      intervals[0] = 10;
    } else {
      intervals[0] = Integer.parseInt(args[0]);
    }

    h = 1.0 / (double) intervals[0];
    sum = 0.0;
    for (i = rank + 1; i <= intervals[0]; i += tasks) {
      x = h * ((double)i - 0.5);
      sum += 4.0 / (1.0 + x*x);
    }
    mypi[0] = h * sum;

    if (rank != 0) {
      MPI.COMM_WORLD.send(mypi, 1, MPI.DOUBLE, 0, rank);
    }

    if (rank == 0) {
      double finalPi = mypi[0];
      for (i = 1; i <= tasks - 1; i++) {
        MPI.COMM_WORLD.recv(pi, 1, MPI.DOUBLE, i, i); 
        finalPi += pi[0];
      }

      System.out.println();
      System.out.printf("pi is approximately %.16f, Error is %.16f\n", finalPi, 
              Math.abs(finalPi - PI25DT));
    }
    MPI.Finalize();
  } 

  public static void main (String[] args) { 
    try {
      PiExamplePt2pt c = new PiExamplePt2pt(args);
    } catch (Exception e) {
    }
  }
}
