/* Copyright (c) 2001-2022, The Ohio State University. All rights
 * reserved.
 *
 * This file is part of the MVAPICH2-J software package developed by the
 * team members of The Ohio State University's Network-Based Computing
 * Laboratory (NBCL), headed by Professor Dhabaleswar K. (DK) Panda.
 *
 * For detailed copyright and licensing information, please refer to the
 * copyright file COPYRIGHT in the top level MVAPICH2-J directory.
 */

/*
 The MIT License

 Copyright (c) 2013 - 2014
 1. SEECS, National University of Sciences and Technology, Pakistan (2013 - 2014)
 2. Bibrak Qamar  (2013 - 2014)

 Permission is hereby granted, free of charge, to any person obtaining
 a copy of this software and associated documentation files (the
 "Software"), to deal in the Software without restriction, including
 without limitation the rights to use, copy, modify, merge, publish,
 distribute, sublicense, and/or sell copies of the Software, and to
 permit persons to whom the Software is furnished to do so, subject to
 the following conditions:

 The above copyright notice and this permission notice shall be included
 in all copies or substantial portions of the Software.

 THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN
 NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
 DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR
 OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR
 THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */

/*
 * File         : mpi_Comm.c
 * Author       : Bibrak Qamar
 * Created      : Tue Sept  17 11:15:11 PKT 2013
 * Revision     : $Revision: 1.1 $
 * Updated      : $Date: 2014/03/11 11:30:00 $
 */

#include "mpi.h"
#include "mpi_Comm.h"
#include <stdlib.h>
#include <string.h>
#include <inttypes.h>
#include "mpi_Commons.h"

static JavaVM *jvm;

jclass CL_mpjbuf_NIOBuffer;
jclass CL_mpi_Comm;
jclass CL_mpi_Status;
jclass CL_mpjbuf_Buffer;
jclass CL_mpi_Request;
jclass CL_mpi_NativeRequest;

jfieldID FID_mpi_Status_source;
jfieldID FID_mpi_Status_tag;
jfieldID FID_mpi_Status_count;
jfieldID FID_mpi_Status_index;
jfieldID FID_mpi_Status_numEls;
jfieldID FID_mpi_Status_handle;
jfieldID FID_mpi_Status_isBufferComms;
jmethodID MID_mpi_Status_constructor;

jfieldID FID_mpi_Comm_CommhandleID;

jfieldID FID_mpjbuf_Buffer_staticBuffer;
jfieldID FID_mpjbuf_NIOBuffer_buffer;
jfieldID FID_mpjbuf_Buffer_size;
jfieldID FID_mpjbuf_Buffer_capacity;

jfieldID FID_mpi_NativeRequest_requestHandle;

jfieldID FID_mpi_Request_handle;
jfieldID FID_mpi_Request_isBufferComms;
jmethodID MID_mpi_Request_complete;

/* 
 * JNI caching
 * JNI_OnLoad()
 */
jint JNI_OnLoad(JavaVM *vm, void *reserved)
{

    JNIEnv *env;
    jvm = vm; //the static

    if (JNI_OK != (*vm)->GetEnv(vm, (void **) &env, JNI_VERSION_1_8)) {
        exit(1);
    }

    /***** caching classes and flieds *****/
    
    /* src/java/mpjbuf/Buffer */
    CL_mpjbuf_Buffer = (*env)->NewGlobalRef(
            env, (*env)->FindClass(env, "mpjbuf/Buffer"));

    FID_mpjbuf_Buffer_size = (*env)->GetFieldID(env,
            CL_mpjbuf_Buffer, "size", "I");
    
    FID_mpjbuf_Buffer_capacity =
        (*env)->GetFieldID(env, CL_mpjbuf_Buffer, "capacity", "I");
   
    FID_mpjbuf_Buffer_staticBuffer = (*env)->GetFieldID(env, 
            CL_mpjbuf_Buffer, "staticBuffer", "Lmpjbuf/RawBuffer;");
    /**/

   
    /* src/java/mpjbuf/NIOBuffer */
    CL_mpjbuf_NIOBuffer = (*env)->NewGlobalRef(env,
            (*env)->FindClass(env, "mpjbuf/NIOBuffer"));

    FID_mpjbuf_NIOBuffer_buffer = (*env)->GetFieldID(
            env, CL_mpjbuf_NIOBuffer, "buffer", "Ljava/nio/ByteBuffer;");
    /**/

    
    /* src/java/mpi/Status.java */
    CL_mpi_Status = (*env)->NewGlobalRef(
            env, (*env)->FindClass(env, "mpi/Status"));

    FID_mpi_Status_source = (*env)->GetFieldID(
            env, CL_mpi_Status, "source", "I");
    
    FID_mpi_Status_tag = (*env)->GetFieldID(
            env, CL_mpi_Status, "tag", "I");
    
    FID_mpi_Status_count = (*env)->GetFieldID(
            env, CL_mpi_Status, "count", "I");
    
    FID_mpi_Status_numEls = (*env)->GetFieldID(
            env, CL_mpi_Status, "numEls", "I");
    
    FID_mpi_Status_handle = (*env)->GetFieldID(
            env, CL_mpi_Status, "handle", "J");
    
    FID_mpi_Status_isBufferComms =(*env)->GetFieldID(
            env, CL_mpi_Status, "isBufferComms", "Z");

    FID_mpi_Status_index = (*env)->GetFieldID(
            env, CL_mpi_Status, "index", "I");

    MID_mpi_Status_constructor = (*env)->GetMethodID(
            env, CL_mpi_Status, "<init>", "(Z)V");
    /**/ 
    
    /* src/java/mpi/NativeRequest.java */
    CL_mpi_NativeRequest = (*env)->NewGlobalRef(env,
            (*env)->FindClass(env, "mpi/NativeRequest"));
    
    FID_mpi_NativeRequest_requestHandle = (*env)->GetFieldID(env,
            CL_mpi_NativeRequest, "requestHandle", "J"); 
    /**/
    
    /* src/java/mpi/Request.java */
    CL_mpi_Request = (*env)->NewGlobalRef(env, (*env)->FindClass(env, "mpi/Request"));

    FID_mpi_Request_handle = (*env)->GetFieldID(env, 
            CL_mpi_Request, "handle", "J");

    FID_mpi_Request_isBufferComms = (*env)->GetFieldID(env,
            CL_mpi_Request, "isBufferComms", "Z");

    MID_mpi_Request_complete = (*env)->GetMethodID(env,
            CL_mpi_Request, "complete", "(Lmpi/Status;)V");
    /**/ 
    
    /* src/java/mpi/Comm.java */
    CL_mpi_Comm =  (*env)->NewGlobalRef(
            env, (*env)->FindClass(env, "mpi/Comm"));

    FID_mpi_Comm_CommhandleID =
        (*env)->GetFieldID(env, CL_mpi_Comm, "handle", "J");
    /**/

    if (FID_mpjbuf_Buffer_staticBuffer && FID_mpjbuf_Buffer_size &&
        FID_mpjbuf_NIOBuffer_buffer && FID_mpjbuf_Buffer_capacity) {
        return JNI_VERSION_1_8;
    } else {
        fprintf(stderr, "\n Fatal error getting FIDs");
        exit(3);
    }
}

/*
 * Class:     mpi_Comm
 * Method:    nativeByteBufferSend
 * Signature: (JLjava/nio/ByteBuffer;III)V
 */
JNIEXPORT void JNICALL Java_mpi_Comm_nativeByteBufferSend(JNIEnv *env,
							  jobject thisObject,
							  jlong comm,
							  jobject buf,
                                                          jint dest,
                                                          jint count,
                                                          jint tag)
{
    MPI_Comm mpi_comm = (MPI_Comm)(intptr_t)comm;
    char *buffer_address = (char *)(*env)->GetDirectBufferAddress(
            env,(jobject)buf);
    
    MPI_Send(buffer_address, count, MPI_BYTE, dest, tag, mpi_comm);
}

/*
 * Class:     mpi_Comm
 * Method:    nativeByteBufferSendRecv
 * Signature: (JLjava/nio/ByteBuffer;IIIILjava/nio/ByteBuffer;IIIILmpi/Status;)V
 */
JNIEXPORT void JNICALL Java_mpi_Comm_nativeByteBufferSendRecv(JNIEnv *env,
                                                              jobject thisObj,
                                                              jlong comm,
                                                              jobject sendbuf,
                                                              jint sendcount,
                                                              jint dest,
                                                              jint sendtag,
                                                              jobject recvbuf,
                                                              jint recvcount,
                                                              jint source,
                                                              jint recvtag,
                                                              jobject status)
{
    MPI_Comm mpi_comm = (MPI_Comm)(intptr_t)comm;
    MPI_Status *mpi_status =
        (MPI_Status *)((*env)->GetLongField(
                    env, status, FID_mpi_Status_handle));

    char *send_buffer_address = (char *)(*env)->GetDirectBufferAddress(
            env,(jobject)sendbuf);
    char *recv_buffer_address = (char *)(*env)->GetDirectBufferAddress(
            env,(jobject)recvbuf);

    
    MPI_Sendrecv(send_buffer_address, sendcount, MPI_BYTE,
                 dest, sendtag, recv_buffer_address, recvcount, MPI_BYTE,
                 source, recvtag, mpi_comm, mpi_status);
}

/*
 * Class:     mpi_Comm
 * Method:    nativeByteBufferSendRecvReplace
 * Signature: (JLjava/nio/ByteBuffer;IIIIIILmpi/Status;)V
 */
JNIEXPORT void JNICALL Java_mpi_Comm_nativeByteBufferSendRecvReplace(JNIEnv *env,
                                                                     jobject thisObj,
                                                                     jlong comm,
                                                                     jobject sendbuf,
                                                                     jint sendcount,
                                                                     jint dest,
                                                                     jint sendtag,
                                                                     jint source,
                                                                     jint recvtag,
                                                                     jobject status)
{
    MPI_Comm mpi_comm = (MPI_Comm)(intptr_t)comm;
    MPI_Status *mpi_status =
        (MPI_Status *)((*env)->GetLongField(
                    env, status, FID_mpi_Status_handle));

    char *buffer_address = (char *)(*env)->GetDirectBufferAddress(
            env,(jobject)sendbuf);

    MPI_Sendrecv_replace(buffer_address, sendcount, MPI_BYTE,
                         dest, sendtag, source, recvtag,
                         mpi_comm, mpi_status);
}



/*
 * Class:     mpi_Comm
 * Method:    nativeByteBufferBsend
 * Signature: (JLjava/nio/ByteBuffer;III)V
 */
JNIEXPORT void JNICALL Java_mpi_Comm_nativeByteBufferBsend(JNIEnv *env,
							   jobject thisObject,
							   jlong comm,
							   jobject buf,
                                                           jint dest,
                                                           jint count,
                                                           jint tag)
{
    MPI_Comm mpi_comm = (MPI_Comm)(intptr_t)comm;
    char *buffer_address = (char *)(*env)->GetDirectBufferAddress(
            env,(jobject)buf);
    MPI_Bsend(buffer_address, count, MPI_BYTE, dest, tag, mpi_comm);
}

/*
 * Class:     mpi_Comm
 * Method:    nativeByteBufferRsend
 * Signature: (JLjava/nio/ByteBuffer;III)V
 */
JNIEXPORT void JNICALL Java_mpi_Comm_nativeByteBufferRsend(JNIEnv *env,
							   jobject thisObject,
							   jlong comm,
							   jobject buf,
                                                           jint dest,
                                                           jint count,
                                                           jint tag)
{
    MPI_Comm mpi_comm = (MPI_Comm)(intptr_t)comm;
    char *buffer_address = (char *)(*env)->GetDirectBufferAddress(
            env,(jobject)buf);

    MPI_Rsend(buffer_address, count, MPI_BYTE, dest, tag, mpi_comm);
}



/*
 * Class:     mpi_Comm
 * Method:    nativeByteBufferSsend
 * Signature: (JLjava/nio/ByteBuffer;III)V
 */
JNIEXPORT void JNICALL Java_mpi_Comm_nativeByteBufferSsend(JNIEnv  *env,
							   jobject thisObject,
							   jlong   comm,
							   jobject buf,
							   jint    dest,
							   jint    count,
							   jint    tag)
{
    MPI_Comm mpi_comm = (MPI_Comm)(intptr_t)comm;
    char *buffer_address =
        (char *)(*env)->GetDirectBufferAddress(env,(jobject)buf);

    MPI_Ssend(buffer_address, count, MPI_BYTE, dest, tag, mpi_comm);
}

/*
 * Class:     mpi_Comm
 * Method:    nativeArraysRecv
 * Signature: (JLmpjbuf/Buffer;IIILmpjdev/Status;)V
 */
JNIEXPORT void JNICALL Java_mpi_Comm_nativeArraysRecv(JNIEnv  *env,
                                                      jobject thisObject,
                                                      jlong   comm,
                                                      jobject buf,
                                                      jint    size,
                                                      jint    src,
                                                      jint    tag,
                                                      jobject status)
{
    MPI_Comm mpi_comm = (MPI_Comm)(intptr_t)comm;

    jobject staticBuffer;
    jobject directBuffer;
    MPI_Status *mpi_status =
        (MPI_Status *)((*env)->GetLongField(
                    env, status, FID_mpi_Status_handle));

    char *buffer_address = NULL;
    int dbuf_len = 0;

    staticBuffer = (*env)->GetObjectField(
            env, buf, FID_mpjbuf_Buffer_staticBuffer);
    directBuffer = (jbyteArray)(*env)->GetObjectField(env, staticBuffer,
        FID_mpjbuf_NIOBuffer_buffer);
    buffer_address =
        (char *)(*env)->GetDirectBufferAddress(env, directBuffer);

    MPI_Recv(buffer_address, size, MPI_BYTE, src, tag, comm, mpi_status);
    
    int elements;
    MPI_Get_count(mpi_status, MPI_BYTE, &elements);
    
    (*env)->SetIntField(env, buf, FID_mpjbuf_Buffer_size, elements);
    
    (*env)->SetIntField(
            env, status, FID_mpi_Status_source, mpi_status->MPI_SOURCE);
    (*env)->SetIntField(
            env, status, FID_mpi_Status_tag, mpi_status->MPI_TAG);
}

/*
 * Class:     mpi_Comm
 * Method:    nativeArraysIrecv
 * Signature: (JLmpjbuf/Buffer;IILmpjdev/Status;Lmpjdev/natmpjdev/NativeRecvRequest;)V
 */
JNIEXPORT void JNICALL Java_mpi_Comm_nativeArraysIrecv(JNIEnv  *env,
						       jobject thisObject,
						       jlong   comm,
                                                       jobject buf,
                                                       jint    src,
                                                       jint    tag,
                                                       jobject status,
                                                       jobject request)
{
    MPI_Comm mpi_comm = (MPI_Comm)(intptr_t)comm;
    MPI_Request mpi_request;

    jobject staticBuffer;
    jbyteArray directBuffer;

    int elements;
    char *buffer_address = NULL;

    staticBuffer = (*env)->GetObjectField(env ,buf,
        FID_mpjbuf_Buffer_staticBuffer);
    directBuffer = (jbyteArray)(*env)->GetObjectField(env, staticBuffer,
        FID_mpjbuf_NIOBuffer_buffer);
    buffer_address = (char *)(*env)->GetDirectBufferAddress(env,
        (jobject)directBuffer);
    
    jlong capacity = (*env)->GetDirectBufferCapacity(
        env,(jobject) directBuffer);
    
    MPI_Irecv(buffer_address, capacity, MPI_BYTE, src, tag,
        mpi_comm, &mpi_request);

    (*env)->SetLongField(env, request,
        FID_mpi_Request_handle, (jlong)mpi_request);
}


/*
 * Class:     mpi_Comm
 * Method:    nativeByteBufferIsend
 * Signature: (JLjava/nio/ByteBuffer;IIILmpi/Request;)V
 */
JNIEXPORT void JNICALL Java_mpi_Comm_nativeByteBufferIsend(JNIEnv *env,
							   jobject thisObject,
							   jlong comm,
							   jobject buf,
							   jint dest,
							   jint count,
							   jint tag,
							   jobject request)
{
    MPI_Request mpi_request;
    MPI_Comm mpi_comm = (MPI_Comm)(intptr_t)comm;
    char *buffer_address =
        (char *)(*env)->GetDirectBufferAddress(env,(jobject)buf);

    MPI_Isend(buffer_address, count, MPI_BYTE, dest, tag, mpi_comm,
        &mpi_request);

    (*env)->SetLongField(env, request,
            FID_mpi_NativeRequest_requestHandle, (jlong)mpi_request);
    (*env)->SetLongField(env, request,
            FID_mpi_Request_handle,
            (jlong)mpi_request);
}

/*
 * Class:     mpi_Comm
 * Method:    nativeByteBufferIbsend
 * Signature: (JLjava/nio/ByteBuffer;IIILmpi/Request;)V
 */
JNIEXPORT void JNICALL Java_mpi_Comm_nativeByteBufferIbsend(JNIEnv *env,
							    jobject thisObject,
							    jlong comm,
							    jobject buf,
							    jint dest,
							    jint count,
							    jint tag,
							    jobject request)
{
    MPI_Request mpi_request;
    MPI_Comm mpi_comm = (MPI_Comm)(intptr_t)comm;

    char *buffer_address =
        (char *)(*env)->GetDirectBufferAddress(env,(jobject)buf);

    MPI_Ibsend(buffer_address, count, MPI_BYTE, dest, tag, mpi_comm,
           &mpi_request);

    (*env)->SetLongField(env, request,
            FID_mpi_NativeRequest_requestHandle, (jlong)mpi_request);
    (*env)->SetLongField(env, request,
            FID_mpi_Request_handle,
            (jlong)mpi_request);
}



/*
 * Class:     mpi_Comm
 * Method:    nativeByteBufferIrsend
 * Signature: (JLjava/nio/ByteBuffer;IIILmpi/Request;)V
 */
JNIEXPORT void JNICALL Java_mpi_Comm_nativeByteBufferIrsend(JNIEnv *env,
							    jobject thisObject,
							    jlong comm,
							    jobject buf,
							    jint dest,
							    jint count,
							    jint tag,
							    jobject request)
{
    MPI_Request mpi_request;
    MPI_Comm mpi_comm = (MPI_Comm)(intptr_t)comm;
    char *buffer_address =
        (char *)(*env)->GetDirectBufferAddress(env,(jobject)buf);
  
    MPI_Irsend(buffer_address, count, MPI_BYTE, dest, tag, mpi_comm,
        &mpi_request);
  
    (*env)->SetLongField(env, request,
            FID_mpi_NativeRequest_requestHandle, (jlong)mpi_request);
    (*env)->SetLongField(env, request,
            FID_mpi_Request_handle,
            (jlong)mpi_request);
}



/*
 * Class:     mpi_Comm
 * Method:    nativeByteBufferIssend
 * Signature: (JLjava/nio/ByteBuffer;IIILmpi/Request;)V
 */
JNIEXPORT void JNICALL Java_mpi_Comm_nativeByteBufferIssend(JNIEnv *env,
                                                           jobject thisObject,
                                                           jlong comm,
                                                           jobject buf,
                                                           jint dest,
                                                           jint count,
                                                           jint tag,
                                                           jobject request)
{
    MPI_Request mpi_request;
    MPI_Comm mpi_comm = (MPI_Comm)(intptr_t)comm;
    char *buffer_address =
        (char *)(*env)->GetDirectBufferAddress(env,(jobject)buf);
        
    MPI_Issend(buffer_address, count, MPI_BYTE, dest, tag, mpi_comm,
            &mpi_request);
    
    (*env)->SetLongField(env, request, FID_mpi_NativeRequest_requestHandle,
            (jlong)mpi_request);
    (*env)->SetLongField(env, request,
            FID_mpi_Request_handle, (jlong)mpi_request);
}

/*
 * Class:     mpi_Comm
 * Method:    nativeByteBufferRecv
 * Signature: (JLjava/nio/ByteBuffer;IILmpi/Status;)V
 */
JNIEXPORT void JNICALL Java_mpi_Comm_nativeByteBufferRecv(JNIEnv *env,
							  jobject thisObject,
							  jlong comm,
							  jobject buf,
							  jint src,
							  jint tag,
							  jobject status)
{
    MPI_Comm mpi_comm = (MPI_Comm)(intptr_t)comm;
    MPI_Status *mpi_status =
        (MPI_Status *)((*env)->GetLongField(env, status, FID_mpi_Status_handle));
    
    char *buffer_address =
        (char *)(*env)->GetDirectBufferAddress(env,(jobject)buf);
    
    jlong capacity = (*env)->GetDirectBufferCapacity(env,(jobject) buf);

    MPI_Recv(buffer_address, capacity, MPI_BYTE, src, tag, mpi_comm, mpi_status);
}

/*
 * Class:     mpi_Comm
 * Method:    nativeByteBufferIrecv
 * Signature: (JLjava/nio/ByteBuffer;IILmpi/Request;)V
 */
JNIEXPORT void JNICALL Java_mpi_Comm_nativeByteBufferIrecv(JNIEnv *env,
							   jobject thisObject,
						           jlong comm,
							   jobject buf,
							   jint src,
							   jint tag,
						           jobject request)
{
    MPI_Comm mpi_comm = (MPI_Comm)(intptr_t)comm;
    MPI_Request mpi_request;
    MPI_Status mpi_status;

    char *buffer_address =
        (char *)(*env)->GetDirectBufferAddress(env,(jobject)buf);
    
    jlong capacity = (*env)->GetDirectBufferCapacity(env,(jobject) buf);
    
    MPI_Irecv(buffer_address, capacity, MPI_BYTE, src, tag,
        mpi_comm, &mpi_request);
    
    (*env)->SetLongField(env, request, FID_mpi_NativeRequest_requestHandle,
        (jlong)mpi_request);
    
    (*env)->SetLongField(env, request,
        FID_mpi_Request_handle, (jlong)mpi_request);
}

/*
 * Class:     mpi_Comm
 * Method:    nativeArraysSend
 * Signature: (JLmpjbuf/Buffer;IIII)V
 */
JNIEXPORT void JNICALL Java_mpi_Comm_nativeArraysSend(JNIEnv *env,
						      jobject thisObject,
                                                      jlong comm,
                                                      jobject buf,
                                                      jint dest,
                                                      jint tag,
                                                      jint sbufLen)
{
    MPI_Comm mpi_comm = (MPI_Comm)(intptr_t)comm;

    jobject staticBuffer;
    jobject directBuffer;
    char * buffer_address = NULL;

    staticBuffer =
        (*env)->GetObjectField(env, buf, FID_mpjbuf_Buffer_staticBuffer);
    directBuffer =
        (*env)->GetObjectField(env, staticBuffer, FID_mpjbuf_NIOBuffer_buffer);
    buffer_address = (char *)(*env)->GetDirectBufferAddress(env, directBuffer);

    char encoding = 0;
    buffer_address[0] = encoding;

    MPI_Send(buffer_address, sbufLen, MPI_BYTE, dest, tag, mpi_comm);
}

/*
 * Class:     mpi_Comm
 * Method:    nativeArraysSendRecv
 * Signature: (JLmpjbuf/Buffer;IIILmpjbuf/Buffer;IIILmpi/Status;)Lmpi/Status;
 */
JNIEXPORT jobject JNICALL Java_mpi_Comm_nativeArraysSendRecv(JNIEnv *env,
                                                             jobject thisObject,
                                                             jlong comm,
                                                             jobject sendbuf,
                                                             jint sendcount,
                                                             jint dest,
                                                             jint sendtag,
                                                             jobject recvbuf,
                                                             jint recvcount,
                                                             jint source,
                                                             jint recvtag,
                                                             jobject status) 
{
    MPI_Comm mpi_comm = (MPI_Comm)(intptr_t)comm;
    MPI_Status *mpi_status =
        (MPI_Status *)((*env)->GetLongField(
                    env, status, FID_mpi_Status_handle));

    jobject sendStaticBuffer;
    jobject sendDirectBuffer;
    char * send_buffer_address = NULL;

    jobject recvStaticBuffer;
    jobject recvDirectBuffer;
    char * recv_buffer_address = NULL;

    sendStaticBuffer =
        (*env)->GetObjectField(env, sendbuf, FID_mpjbuf_Buffer_staticBuffer);
    sendDirectBuffer =
        (*env)->GetObjectField(env, sendStaticBuffer, FID_mpjbuf_NIOBuffer_buffer);
    send_buffer_address = (char *)(*env)->GetDirectBufferAddress(env, sendDirectBuffer);

    recvStaticBuffer =
        (*env)->GetObjectField(env, recvbuf, FID_mpjbuf_Buffer_staticBuffer);
    recvDirectBuffer =
        (*env)->GetObjectField(env, recvStaticBuffer, FID_mpjbuf_NIOBuffer_buffer);
    recv_buffer_address = (char *)(*env)->GetDirectBufferAddress(env, recvDirectBuffer);

    char encoding = 0;
    send_buffer_address[0] = encoding;
    recv_buffer_address[0] = encoding;

    MPI_Sendrecv(send_buffer_address, sendcount, MPI_BYTE,
                 dest, sendtag, recv_buffer_address, recvcount, MPI_BYTE,
                 source, recvtag, mpi_comm, mpi_status);

   int elements;
   MPI_Get_count(mpi_status, MPI_BYTE, &elements);
   
   (*env)->SetIntField(env, recvbuf, FID_mpjbuf_Buffer_size, elements);
   
   (*env)->SetIntField(
           env, status, FID_mpi_Status_source, mpi_status->MPI_SOURCE);
   (*env)->SetIntField(
           env, status, FID_mpi_Status_tag, mpi_status->MPI_TAG);
}

/*
 * Class:     mpi_Comm
 * Method:    nativeArraysSendRecvReplace
 * Signature: (JLmpjbuf/Buffer;IIIIILmpi/Status;)Lmpi/Status;
 */
JNIEXPORT jobject JNICALL Java_mpi_Comm_nativeArraysSendRecvReplace(JNIEnv *env,
                                                                    jobject thisObject,
                                                                    jlong comm,
                                                                    jobject sendbuf,
                                                                    jint count,
                                                                    jint dest,
                                                                    jint sendtag,
                                                                    jint source,
                                                                    jint recvtag,
                                                                    jobject status)
{
    MPI_Comm mpi_comm = (MPI_Comm)(intptr_t)comm;
    MPI_Status *mpi_status =
        (MPI_Status *)((*env)->GetLongField(
                    env, status, FID_mpi_Status_handle));

    jobject sendStaticBuffer;
    jobject sendDirectBuffer;
    char * send_buffer_address = NULL;

    sendStaticBuffer =
        (*env)->GetObjectField(env, sendbuf, FID_mpjbuf_Buffer_staticBuffer);
    sendDirectBuffer =
        (*env)->GetObjectField(env, sendStaticBuffer, FID_mpjbuf_NIOBuffer_buffer);
    send_buffer_address = (char *)(*env)->GetDirectBufferAddress(env, sendDirectBuffer);

    //char encoding = 0;
    //send_buffer_address[0] = encoding;

    MPI_Sendrecv_replace(send_buffer_address, count, MPI_BYTE,
                         dest, sendtag, source, recvtag,
                         mpi_comm, mpi_status);

   int elements;
   MPI_Get_count(mpi_status, MPI_BYTE, &elements);
   
   (*env)->SetIntField(env, sendbuf, FID_mpjbuf_Buffer_size, elements);
   
   (*env)->SetIntField(
           env, status, FID_mpi_Status_source, mpi_status->MPI_SOURCE);
   (*env)->SetIntField(
           env, status, FID_mpi_Status_tag, mpi_status->MPI_TAG);
}



/*
 * Class:     mpi_Comm
 * Method:    nativeArraysBsend
 * Signature: (JLmpjbuf/Buffer;IIII)V
 */
JNIEXPORT void JNICALL Java_mpi_Comm_nativeArraysBsend(JNIEnv *env,
						       jobject thisObject,
                                                       jlong comm,
                                                       jobject buf,
                                                       jint dest,
                                                       jint tag,
                                                       jint sbufLen)
{
    MPI_Comm mpi_comm = (MPI_Comm)(intptr_t)comm;

    jobject staticBuffer;
    jobject directBuffer;
    char * buffer_address = NULL;

    staticBuffer =
        (*env)->GetObjectField(env, buf, FID_mpjbuf_Buffer_staticBuffer);
    directBuffer =
        (*env)->GetObjectField(env, staticBuffer, FID_mpjbuf_NIOBuffer_buffer);
    buffer_address = (char *)(*env)->GetDirectBufferAddress(env, directBuffer);

    char encoding = 0;
    buffer_address[0] = encoding;
    MPI_Bsend(buffer_address, sbufLen, MPI_BYTE, dest, tag, mpi_comm);
}

/*
 * Class:     mpi_Comm
 * Method:    nativeArraysRsend
 * Signature: (JLmpjbuf/Buffer;IIII)V
 */
JNIEXPORT void JNICALL Java_mpi_Comm_nativeArraysRsend(JNIEnv *env,
						       jobject thisObject,
                                                       jlong comm,
                                                       jobject buf,
                                                       jint dest,
                                                       jint tag,
                                                       jint sbufLen)
{
    MPI_Comm mpi_comm = (MPI_Comm)(intptr_t)comm;

    jobject staticBuffer;
    jobject directBuffer;
    char * buffer_address = NULL;

    staticBuffer =
        (*env)->GetObjectField(env, buf, FID_mpjbuf_Buffer_staticBuffer);
    directBuffer =
        (*env)->GetObjectField(env, staticBuffer, FID_mpjbuf_NIOBuffer_buffer);
    buffer_address = (char *)(*env)->GetDirectBufferAddress(env, directBuffer);

    char encoding = 0;
    buffer_address[0] = encoding;

    MPI_Rsend(buffer_address, sbufLen, MPI_BYTE, dest, tag, mpi_comm);
}

/*
 * Class:     mpi_Comm
 * Method:    nativeArraysIsend
 * Signature: (JLmpjbuf/Buffer;IIIILmpjdev/natmpjdev/NativeSendRequest;)V
 */
JNIEXPORT void JNICALL Java_mpi_Comm_nativeArraysIsend(JNIEnv  *env,
						       jobject thisObject,
						       jlong   comm,
						       jobject buf,
                                                       jint    dest,
                                                       jint    tag,
                                                       jint    sbufLen,
                                                       jobject request)
{
    MPI_Comm mpi_comm = (MPI_Comm)(intptr_t)comm;
    MPI_Request mpi_request;
    
    jobject staticBuffer;
    jbyteArray directBuffer;
    
    char *buffer_address = NULL;

    staticBuffer =
        (*env)->GetObjectField(env, buf, FID_mpjbuf_Buffer_staticBuffer);
    directBuffer =
        (*env)->GetObjectField(env, staticBuffer, FID_mpjbuf_NIOBuffer_buffer);
    buffer_address =
        (char *)(*env)->GetDirectBufferAddress(env, (jobject)directBuffer);

    char encoding = 0;
    buffer_address[0] = encoding;

    MPI_Isend(buffer_address, sbufLen, MPI_BYTE, dest, tag,
        mpi_comm, &mpi_request);

    (*env)->SetLongField(env, request,
        FID_mpi_Request_handle, (jlong)mpi_request);
}

/*
 * Class:     mpi_Comm
 * Method:    nativeArraysIbsend
 * Signature: (JLmpjbuf/Buffer;IIIILmpjdev/natmpjdev/NativeSendRequest;)V
 */
JNIEXPORT void JNICALL Java_mpi_Comm_nativeArraysIbsend(JNIEnv  *env,
						        jobject thisObject,
						        jlong   comm,
						        jobject buf,
                                                        jint    dest,
                                                        jint    tag,
                                                        jint    sbufLen,
                                                        jobject request)
{
    MPI_Comm mpi_comm = (MPI_Comm)(intptr_t)comm;
    MPI_Request mpi_request;
    jobject staticBuffer;
    jbyteArray directBuffer;
    char *buffer_address = NULL;
    jboolean isCopy = JNI_TRUE;

    staticBuffer =
        (*env)->GetObjectField(env, buf, FID_mpjbuf_Buffer_staticBuffer);
    directBuffer =
        (*env)->GetObjectField(env, staticBuffer, FID_mpjbuf_NIOBuffer_buffer);
    buffer_address =
        (char *)(*env)->GetDirectBufferAddress(env, (jobject)directBuffer);
    char encoding = 0;
    buffer_address[0] = encoding;
    MPI_Ibsend(buffer_address, sbufLen, MPI_BYTE, dest, tag,
        mpi_comm, &mpi_request);

    (*env)->SetLongField(env, request,
        FID_mpi_Request_handle, (jlong)mpi_request);
}

/*
 * Class:     mpi_Comm
 * Method:    nativeArraysIrsend
 * Signature: (JLmpjbuf/Buffer;IIIILmpjdev/natmpjdev/NativeSendRequest;)V
 */
JNIEXPORT void JNICALL Java_mpi_Comm_nativeArraysIrsend(JNIEnv  *env,
						        jobject thisObject,
						        jlong   comm,
						        jobject buf,
                                                        jint    dest,
                                                        jint    tag,
                                                        jint    sbufLen,
                                                        jobject request)
{
    MPI_Comm mpi_comm = (MPI_Comm)(intptr_t)comm;
    MPI_Request mpi_request;
    
    jobject staticBuffer;
    jbyteArray directBuffer;
    char *buffer_address = NULL;

    staticBuffer =
        (*env)->GetObjectField(env, buf, FID_mpjbuf_Buffer_staticBuffer);
    directBuffer =
        (*env)->GetObjectField(env, staticBuffer, FID_mpjbuf_NIOBuffer_buffer);
    buffer_address =
        (char *)(*env)->GetDirectBufferAddress(env, (jobject)directBuffer);
    
    char encoding = 0;
    buffer_address[0] = encoding;
    
    MPI_Irsend(buffer_address, sbufLen, MPI_BYTE, dest, tag,
        mpi_comm, &mpi_request);

    (*env)->SetLongField(env, request,
        FID_mpi_Request_handle, (jlong)mpi_request);
}

/*
 * Class:     mpi_Comm
 * Method:    nativeArraysSsend
 * Signature: (JLmpjbuf/Buffer;IIII)V
 */
JNIEXPORT void JNICALL Java_mpi_Comm_nativeArraysSsend(JNIEnv  *env,
                                                       jobject thisObject,
                                                       jlong   comm,
                                                       jobject buf,
                                                       jint    dest,
                                                       jint    tag,
                                                       jint    sbufLen)
{
    MPI_Comm mpi_comm = (MPI_Comm)(intptr_t)comm;

    jobject staticBuffer;
    jbyteArray directBuffer;
    char * buffer_address = NULL;

    staticBuffer = (*env)->GetObjectField(env, buf,
        FID_mpjbuf_Buffer_staticBuffer);
    directBuffer = (*env)->GetObjectField(env, staticBuffer,
        FID_mpjbuf_NIOBuffer_buffer);
    buffer_address = (char *)(*env)->GetDirectBufferAddress(env,
        (jobject)directBuffer);
    
    jlong capacity =
        (*env)->GetDirectBufferCapacity(env, (jobject) directBuffer);
    
    char encoding = 0;
    buffer_address[0] = encoding;
    
    MPI_Ssend(buffer_address, sbufLen, MPI_BYTE, dest, tag, mpi_comm);
}

/*
 * Class:     mpi_Comm
 * Method:    nativeArraysIssend
 * Signature: (JLmpjbuf/Buffer;IIIILmpjdev/natmpjdev/NativeSendRequest;)V
 */
JNIEXPORT void JNICALL Java_mpi_Comm_nativeArraysIssend(JNIEnv  *env,
                                                        jobject thisObject,
                                                        jlong   comm,
                                                        jobject buf,
                                                        jint    dest,
                                                        jint    tag,
                                                        jint    sbufLen,
                                                        jobject request)
{
    MPI_Comm mpi_comm = (MPI_Comm)(intptr_t)comm;
    MPI_Request mpi_request;

    jobject staticBuffer;
    jbyteArray directBuffer;
    char * buffer_address = NULL;

    staticBuffer = (*env)->GetObjectField(env,
        buf, FID_mpjbuf_Buffer_staticBuffer);
    directBuffer = (*env)->GetObjectField(env,
        staticBuffer, FID_mpjbuf_NIOBuffer_buffer);
    buffer_address = (char *)(*env)->GetDirectBufferAddress(env,
        (jobject)directBuffer);
    
    char encoding = 0;
    buffer_address[0] = encoding;
   
    MPI_Issend(buffer_address, sbufLen, MPI_BYTE, dest, tag,
        mpi_comm, &mpi_request);

    (*env)->SetLongField(env, request,
        FID_mpi_Request_handle, (jlong)mpi_request);
}

/*
 * Class:     mpi_Comm
 * Method:    getTagUB
 * Signature: ()I
 */
JNIEXPORT jint JNICALL Java_mpi_Comm_getTagUB(JNIEnv *env,
                                             jclass thisClass)
{
    int  *maxTag;
    int error, flag;
    MPI_Comm_get_attr(MPI_COMM_WORLD, MPI_TAG_UB, &maxTag, &flag);
    return *maxTag;
}

/*
 * Class:     mpi_Comm
 * Method:    dup
 * Signature: (J)J
 */
JNIEXPORT jlong JNICALL Java_mpi_Comm_dup(JNIEnv  *env,
                                          jobject thisObject,
                                          jlong   comm)
{
    MPI_Comm newComm;
    int errCode = MPI_Comm_dup((MPI_Comm) (intptr_t) comm, &newComm);
    return (long) newComm;
}

/*
 * Class:     mpi_Comm
 * Method:    rank
 * Signature: (J)I
 */
JNIEXPORT jint JNICALL Java_mpi_Comm_rank(JNIEnv  *env,
                                          jobject thisObject,
                                          jlong   comm)
{
    int rank;
    MPI_Comm mpi_comm = (MPI_Comm) (intptr_t) comm;
    MPI_Comm_rank(mpi_comm, &rank);
    return rank;
}

/*
 * Class:     mpi_Comm
 * Method:    size
 * Signature: (J)I
 */
JNIEXPORT jint JNICALL Java_mpi_Comm_size(JNIEnv  *env,
                                          jobject thisObject,
                                          jlong   comm)
{
    int size;
    MPI_Comm mpi_comm = (MPI_Comm) (intptr_t) comm;
    MPI_Comm_size(mpi_comm, &size);
    return size;
}

/*
 * Class:     mpi_Comm
 * Method:    create
 * Signature: (J[I)J
 */
JNIEXPORT jlong JNICALL Java_mpi_Comm_create(JNIEnv    *env,
                                             jobject   thisObject,
                                             jlong     comm,
                                             jintArray ids)
{
    MPI_Comm newComm;
    MPI_Group existingGroup, newGroup;

    int size;
    jint *pids;
    int errCode;

    MPI_Comm existingComm = (MPI_Comm) (intptr_t) comm;
    MPI_Comm_group(existingComm, &existingGroup);
    size = (*env)->GetArrayLength(env, ids);
    pids = (*env)->GetIntArrayElements(env, ids, 0); // 0 or JNI_COPY?
    errCode = MPI_Group_incl(existingGroup, size, (int*) pids, &newGroup);
    (*env)->ReleaseIntArrayElements(env, ids, pids, JNI_ABORT);
    errCode = MPI_Comm_create(existingComm, newGroup, &newComm);

    if (MPI_COMM_NULL == newComm) {
        return 0;
    } else {
        return (jlong) newComm;
    }
}

/*
 * Class:     mpi_Comm
 * Method:    nativeCreateIntercomm
 * Signature: (JJIII)J
 */
JNIEXPORT jlong JNICALL Java_mpi_Comm_nativeCreateIntercomm(JNIEnv *env,
   							    jobject thisObject,
							    jlong peerComm_,
							    jlong localComm_,
    							    jint localleader,
 							    jint remoteleader,
							    jint tag)
{
    int errCode;
    MPI_Comm peerComm = (MPI_Comm) (intptr_t) peerComm_;
    MPI_Comm localComm = (MPI_Comm) (intptr_t) localComm_;
    MPI_Comm newIntercomm;
    errCode = MPI_Intercomm_create(localComm, localleader, peerComm,
            remoteleader, tag, &newIntercomm);

    if (!(MPI_SUCCESS == errCode)) {
        printf("Nativ: Comm - nativeIntercommCreate returned not MPI_SUCCESS\n");
    }

    return (jlong) newIntercomm;
}

/*
 * Class:     mpi_Comm
 * Method:    free
 * Signature: (J)V
 */
JNIEXPORT void JNICALL Java_mpi_Comm_free(JNIEnv *env,
					  jobject thisObject,
					  jlong comm)
{
    MPI_Comm existingComm = (MPI_Comm) (intptr_t) comm;
    int errCode = MPI_Comm_free(&existingComm);
    (*env)->SetLongField(env, thisObject, FID_mpi_Comm_CommhandleID,
        (jlong)MPI_COMM_NULL);
}

/*
 * Class:     mpi_Comm
 * Method:    Test_inter
 * Signature: ()Z
 */
JNIEXPORT jboolean JNICALL Java_mpi_Comm_Test_1inter(JNIEnv  *env,
                                                     jobject jthis)
{
    int flag;
    MPI_Comm_test_inter((MPI_Comm)((*env)->GetLongField(env, jthis,
        FID_mpi_Comm_CommhandleID)), &flag);

    if (0 == flag) {
        return JNI_FALSE;
    } else {
        return JNI_TRUE;
    }
}

/*
 * Class:     mpi_Comm
 * Method:    group
 * Signature: ()J
 *
 * This function returns the default group of the CommhandleID
 */
JNIEXPORT jlong JNICALL Java_mpi_Comm_group(JNIEnv *env,
                                            jobject thisObject)
{
    MPI_Group group;
    MPI_Comm existingComm = (MPI_Comm)(*env)->GetLongField(env, thisObject,
        FID_mpi_Comm_CommhandleID);
    MPI_Comm_group(existingComm, &group);
    return (jlong) group;
}

/*
 * Class:     mpi_Comm
 * Method:    GetComm
 * Signature: (I)V
 */
JNIEXPORT void JNICALL Java_mpi_Comm_GetComm(JNIEnv *env,
					     jobject thisObject,
                                             jint type)
{
    switch (type) {
        case 0: //MPI_COMM_NULL
            (*env)->SetLongField(env,
                                 thisObject,
                                 FID_mpi_Comm_CommhandleID,
                                 (jlong)MPI_COMM_NULL);
            break;
        case 1: //MPI_COMM_SELF
            (*env)->SetLongField(env,
                                 thisObject,
                                 FID_mpi_Comm_CommhandleID,
                                 (jlong)MPI_COMM_SELF);
            break;
        case 2: //MPI_COMM_WORLD
            (*env)->SetLongField(env,
                                 thisObject,
                                 FID_mpi_Comm_CommhandleID,
                                 (jlong)MPI_COMM_WORLD);
            break;
        default:
            break;
    }
}

/*
 * Class:     mpi_Comm
 * Method:    nativeProbe
 * Signature: (JIILmpjdev/Status;)V
 */
JNIEXPORT void JNICALL Java_mpi_Comm_nativeProbe(JNIEnv *env,
                                                 jobject thisObject,
                                                 jlong comm,
                                                 jint src,
                                                 jint tag,
                                                 jobject status)
{
    MPI_Comm mpi_comm = (MPI_Comm)(intptr_t) comm;
    MPI_Status *mpi_status =(MPI_Status *)((*env)->GetLongField(
                env, status, FID_mpi_Status_handle));
    int elements;

    MPI_Probe(src, tag, mpi_comm, mpi_status);

    MPI_Get_count(mpi_status, MPI_BYTE, &elements);

    (*env)->SetIntField(env, status, FID_mpi_Status_source,
            mpi_status->MPI_SOURCE);
    (*env)->SetIntField(env, status, FID_mpi_Status_tag,
            mpi_status->MPI_TAG);
}

/*
 * Class:     mpi_Comm
 * Method:    nativeIprobe
 * Signature: (JIILmpjdev/Status;)V
 */
JNIEXPORT void JNICALL Java_mpi_Comm_nativeIprobe(JNIEnv *env,
                                                  jobject thisObject,
                                                  jlong comm,
                                                  jint src,
                                                  jint tag,
                                                  jobject status)
{
    MPI_Comm mpi_comm = (MPI_Comm)(intptr_t) comm;
    MPI_Status *mpi_status = (MPI_Status *)((*env)->GetLongField(
                env, status, FID_mpi_Status_handle));

    int elements;
    int flag;

    MPI_Iprobe(src, tag, mpi_comm, &flag, mpi_status);

    if (0 != flag) {
        MPI_Get_count(mpi_status, MPI_BYTE, &elements);
        (*env)->SetIntField(env, status, FID_mpi_Status_source,
            mpi_status->MPI_SOURCE);
        (*env)->SetIntField(env, status, FID_mpi_Status_tag,
            mpi_status->MPI_TAG);
    }
}

/*
 * Class:     mpi_Comm
 * Method:    compare
 * Signature: (Lmpi/Comm;Lmpi/Comm;)I
 */
JNIEXPORT jint JNICALL Java_mpi_Comm_compare(JNIEnv  *env,
                                             jclass  jthis,
                                             jobject comm1,
                                             jobject comm2)
{
    int result;
    MPI_Comm_compare(
     (MPI_Comm)((*env)->GetLongField(
             env, comm1, FID_mpi_Comm_CommhandleID)),
     (MPI_Comm)((*env)->GetLongField(
             env, comm2, FID_mpi_Comm_CommhandleID)),
     &result);
    return result;
}

/*
 * Class:     mpi_Comm
 * Method:    free
 * Signature: ()V
 */
JNIEXPORT void JNICALL Java_mpi_Comm_free__(JNIEnv *env, jobject jthis)
{
    MPI_Comm comm = (MPI_Comm)((*env)->GetLongField(env, jthis,
        FID_mpi_Comm_CommhandleID));
    MPI_Comm_free(&comm);
}

/*
 * Class:     mpi_Comm
 * Method:    nativeSpawn
 * Signature: (JLjava/lang/String;[Ljava/lang/String;ILmpi/Info;I[I)J
 */
JNIEXPORT jlong JNICALL Java_mpi_Comm_nativeSpawn(JNIEnv *env,
                                                  jobject jthis,
                                                  jlong comm,
                                                  jstring cmd,
                                                  jobjectArray args,
                                                  jint maxprocs,
                                                  jlong info,
                                                  jint root,
                                                  jintArray errCodes)
{
    MPI_Comm intercomm;
    MPI_Comm mpi_comm = (MPI_Comm)(intptr_t) comm;
    const char *command = (*env)->GetStringUTFChars(env, cmd, 0);

    int len = (*env)->GetArrayLength(env, args);
    char **sargs = (char **) calloc(len + 1, sizeof(char *));
    int i;

    for (i = 0; i < len; i++) {
        jstring jc = (jstring)(*env)->GetObjectArrayElement(env, args, i);
        const char* jc_str = (*env)->GetStringUTFChars(env, jc, 0);
        int slen = strlen(jc_str) + 1;
        sargs[i] = (char *) calloc(slen, sizeof(char));
        strcpy(sargs[i], jc_str);
        (*env)->DeleteLocalRef(env, jc);
    }

    MPI_Comm_spawn((char*)command, sargs, maxprocs, (MPI_Info)info,
            root, mpi_comm, &intercomm, MPI_ERRCODES_IGNORE);
    
    for (i = 0; i < len; i++) { 
        jstring jc = (jstring)(*env)->GetObjectArrayElement(env, args, i);
        (*env)->ReleaseStringUTFChars(env, jc, sargs[i]);
        (*env)->DeleteLocalRef(env, jc);
        free(sargs[i]);
    }

    free(sargs);

    (*env)->ReleaseStringUTFChars(env, cmd, command);
    
    return (jlong)intercomm;
}

/*
 * Class:     mpi_Comm
 * Method:    nativeSpawnMultiple
 * Signature: (J[Ljava/lang/String;[[Ljava/lang/String;[I[JI[I)J
 */
JNIEXPORT jlong JNICALL Java_mpi_Comm_nativeSpawnMultiple(JNIEnv *env,
                                                          jobject jthis,
                                                          jlong comm,
                                                          jobjectArray jcmds,
                                                          jobjectArray argsArr,
                                                          jintArray jmaxprocsArr,
                                                          jlongArray infoArr,
                                                          jint root,
                                                          jintArray errcodes)
{
    MPI_Comm intercomm;

    int cmds_arr_len = (*env)->GetArrayLength(env, jcmds);
    char **cmds = calloc(cmds_arr_len, sizeof(char*));
    int i = 0;

    for (i = 0; i < cmds_arr_len; i++) {
        jstring cmd = (*env)->GetObjectArrayElement(env, jcmds, i);
        cmds[i] = (char*)(*env)->GetStringUTFChars(env, cmd, 0);
        (*env)->DeleteLocalRef(env, cmd);
    }

    char ***args_arr = MPI_ARGVS_NULL;
    int args_arr_len = (*env)->GetArrayLength(env, argsArr);
    args_arr = calloc(args_arr_len, sizeof(char**));

    for (i = 0; i < args_arr_len; i++) {
        jobjectArray jargs = (*env)->GetObjectArrayElement(env, argsArr, i);
        int jargs_len = (*env)->GetArrayLength(env, jargs);
        args_arr[i] = calloc(jargs_len + 1, sizeof(char*));

        int j = 0;
        for (j = 0; j < jargs_len; j++) {
            jstring argStr = (*env)->GetObjectArrayElement(env, jargs, j);
            char* tmp = (char*)(*env)->GetStringUTFChars(env, argStr, 0);
            args_arr[i][j] = tmp;
            (*env)->DeleteLocalRef(env, argStr);
        }

        (*env)->DeleteLocalRef(env, jargs);
    }

    jint *jmpa = (*env)->GetIntArrayElements(env, jmaxprocsArr, 0);

    jlong *jInfoArr = (*env)->GetLongArrayElements(env, infoArr, 0);
    int info_arr_length = (*env)->GetArrayLength(env, infoArr);

    MPI_Info *info_arr = calloc(info_arr_length, sizeof(MPI_Info));
    for (i = 0; i < info_arr_length; i++) {
        info_arr[i] = (MPI_Info)jInfoArr[i];
    }

    MPI_Comm_spawn_multiple(cmds_arr_len, cmds, args_arr, (int*)jmpa, info_arr,
        root, (MPI_Comm)comm, &intercomm, MPI_ERRCODES_IGNORE);

    for (i = 0; i < args_arr_len; i++) {
        jobjectArray jargs = (*env)->GetObjectArrayElement(env, argsArr, i);
        
        int j = 0;
        int jargs_len = (*env)->GetArrayLength(env, jargs);
        
        for (j = 0; j < jargs_len; j++) {
          jstring argStr = (*env)->GetObjectArrayElement(env, jargs, j);
          (*env)->ReleaseStringUTFChars(env, argStr, args_arr[i][j]);
          (*env)->DeleteLocalRef(env, argStr);
        }

        (*env)->DeleteLocalRef(env, jargs);
        free(args_arr[i]);
    }

    free(args_arr);

    for (i = 0; i < cmds_arr_len; i++) {
        jstring cmd = (*env)->GetObjectArrayElement(env, jcmds, i);
        (*env)->ReleaseStringUTFChars(env, cmd, cmds[i]);
        (*env)->DeleteLocalRef(env, cmd);
    }

    free(cmds);
    free(info_arr);
    (*env)->ReleaseLongArrayElements(env, infoArr, jInfoArr, JNI_ABORT);
    (*env)->ReleaseIntArrayElements(env, jmaxprocsArr, jmpa, JNI_ABORT);

    return(jlong)intercomm;
}
