/* Copyright (c) 2001-2022, The Ohio State University. All rights
 * reserved.
 *
 * This file is part of the MVAPICH2-J software package developed by the
 * team members of The Ohio State University's Network-Based Computing
 * Laboratory (NBCL), headed by Professor Dhabaleswar K. (DK) Panda.
 *
 * For detailed copyright and licensing information, please refer to the
 * copyright file COPYRIGHT in the top level MVAPICH2-J directory.
 */

#include "mpi_Info.h"
#include "mpi.h"

/*
 * Class:     mpi_Info
 * Method:    nativeCreate
 * Signature: ()J
 */
JNIEXPORT jlong JNICALL Java_mpi_Info_nativeInfoCreate(JNIEnv *env,
                                                       jobject thisObject)
{
    MPI_Info info;
    MPI_Info_create(&info);
    return (jlong)info;
}

/*
 * Class:     mpi_Info
 * Method:    nativeSet
 * Signature: (JLjava/lang/String;Ljava/lang/String;)V
 */
JNIEXPORT void JNICALL Java_mpi_Info_nativeSet(JNIEnv *env,
                                               jobject thisObject,
                                               jlong comm,
                                               jstring key,
                                               jstring val)
{
    const char *ikey = (*env)->GetStringUTFChars(env, key, NULL);
    const char *ival = (*env)->GetStringUTFChars(env, val, NULL);

    MPI_Info_set((MPI_Info)comm, (char*) ikey, (char*) ival);
    (*env)->ReleaseStringUTFChars(env, key, ikey);
    (*env)->ReleaseStringUTFChars(env, val, ival);
}

/*
 * Class:     mpi_Info
 * Method:    nativeGetNull
 * Signature: ()J
 */
JNIEXPORT jlong JNICALL Java_mpi_Info_nativeGetNull(JNIEnv *env,
                                                    jclass thisClass)
{
    return (jlong)MPI_INFO_NULL;
}
