/* Copyright (c) 2001-2022, The Ohio State University. All rights
 * reserved.
 *
 * This file is part of the MVAPICH2-J software package developed by the
 * team members of The Ohio State University's Network-Based Computing
 * Laboratory (NBCL), headed by Professor Dhabaleswar K. (DK) Panda.
 *
 * For detailed copyright and licensing information, please refer to the
 * copyright file COPYRIGHT in the top level MVAPICH2-J directory.
 */

/*
 The MIT License

 Copyright (c) 2013 - 2014
 1. SEECS, National University of Sciences and Technology, Pakistan (2013 - 2014)
 2. Bibrak Qamar  (2013 - 2014)

 Permission is hereby granted, free of charge, to any person obtaining
 a copy of this software and associated documentation files (the
 "Software"), to deal in the Software without restriction, including
 without limitation the rights to use, copy, modify, merge, publish,
 distribute, sublicense, and/or sell copies of the Software, and to
 permit persons to whom the Software is furnished to do so, subject to
 the following conditions:

 The above copyright notice and this permission notice shall be included
 in all copies or substantial portions of the Software.

 THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN
 NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
 DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR
 OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR
 THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */

 /*
 * File         : mpi_Intercomm.c
 * Author       : Bibrak Qamar
 * Created      : Tue Sept  17 11:15:11 PKT 2013
 * Revision     : $Revision: 1.1 $
 * Updated      : $Date: 2014/03/11 11:30:00 $
 *
 */

#include "mpi.h"
#include "mpi_Intercomm.h"
#include <inttypes.h>
#include "mpi_Commons.h"

/*
 * Class:     mpi_Intercomm
 * Method:    nativeMerge
 * Signature: (Z)Lmpi/Intracomm;
 */
JNIEXPORT jlong JNICALL Java_mpi_Intercomm_nativeMerge(JNIEnv *env,
                                                       jobject jthis,
                                                       jboolean high)
{
    MPI_Comm newintracomm;
    MPI_Intercomm_merge((MPI_Comm)((*env)->GetLongField(env, jthis,
        FID_mpi_Comm_CommhandleID)), high, &newintracomm);
    return (jlong)newintracomm;
}


/*
 * Class:     mpi_Intercomm
 * Method:    Remote_size
 * Signature: ()I
 */
JNIEXPORT jint JNICALL Java_mpi_Intercomm_Remote_1size(JNIEnv  *env,
						       jobject jthis)
{
    int size;
    MPI_Comm_remote_size((MPI_Comm)((*env)->GetLongField(env, jthis,
        FID_mpi_Comm_CommhandleID)), &size);
    return size;
}

/*
 * Class:     mpi_Intercomm
 * Method:    remote_group
 * Signature: ()J
 */
JNIEXPORT jlong JNICALL Java_mpi_Intercomm_remote_1group(JNIEnv  *env,
						         jobject jthis)
{
    MPI_Group group;
    MPI_Comm_remote_group((MPI_Comm)((*env)->GetLongField(env, jthis,
        FID_mpi_Comm_CommhandleID)), &group);
    return (jlong)group;
}

/*
 * Class:     mpi_Intercomm
 * Method:    nativeGetParent
 * Signature: ()J
 */
JNIEXPORT jlong JNICALL Java_mpi_Intercomm_nativeGetParent(JNIEnv *env,
                                                           jclass class)
{
    MPI_Comm parent;
    int rc = MPI_Comm_get_parent(&parent);
    return (jlong) parent;
}
