/* Copyright (c) 2001-2022, The Ohio State University. All rights
 * reserved.
 *
 * This file is part of the MVAPICH2-J software package developed by the
 * team members of The Ohio State University's Network-Based Computing
 * Laboratory (NBCL), headed by Professor Dhabaleswar K. (DK) Panda.
 *
 * For detailed copyright and licensing information, please refer to the
 * copyright file COPYRIGHT in the top level MVAPICH2-J directory.
 */

#include "mpi_MPI.h"
#include <inttypes.h>
#include "mpi_Commons.h"
#include "mpi.h"
#include <stdlib.h>
#include <string.h>

/*
 * Class:     mpi_Status
 * Method:    alloc
 * Signature: ()V
 */
JNIEXPORT void JNICALL Java_mpi_Status_alloc(JNIEnv *env,
                                             jobject jthis)
{
    MPI_Status *status = (MPI_Status*) malloc(sizeof(MPI_Status));
    (*env)->SetLongField(env, jthis, FID_mpi_Status_handle, (jlong)status);
}

/*
 * Class:     mpi_Status
 * Method:    dealloc
 * Signature: ()V
 */
JNIEXPORT void JNICALL Java_mpi_Status_dealloc(JNIEnv *env, jobject jthis)
{
    MPI_Status *status =
        (MPI_Status *)((*env)->GetLongField(env, jthis,
                    FID_mpi_Status_handle));
    free(status);
}

/*
 * Class:     mpi_Status
 * Method:    get_count
 * Signature: (Lmpi/Datatype;)I
 */
JNIEXPORT jint JNICALL Java_mpi_Status_get_1count(JNIEnv *env,
						  jobject jthis,
						  jint type)
{
    int count;
    MPI_Datatype datatype = (MPI_Datatype)type;
    MPI_Status *stat = (MPI_Status*)((*env)->GetLongField(env, jthis,
                FID_mpi_Status_handle));
    MPI_Get_count(stat, datatype, &count);
    return count;
}

/*
 * Class:     mpi_Status
 * Method:    get_elements
 * Signature: (Lmpi/Datatype;)I
 */
JNIEXPORT jint JNICALL Java_mpi_Status_get_1elements(JNIEnv  *env,
						     jobject jthis,
					             jint type)
{
    int elements;
    MPI_Datatype datatype = (MPI_Datatype)type;
    MPI_Status *stat =
        (MPI_Status*)((*env)->GetLongField(env, jthis,
                    FID_mpi_Status_handle));
    MPI_Get_elements(stat, datatype, &elements);
    return elements;
}

/*
 * Class:     mpi_Status
 * Method:    nativeGetSource
 * Signature: ()I
 */
JNIEXPORT jint JNICALL Java_mpi_Status_nativeGetSource(JNIEnv  *env,
                                                       jobject jthis)
{
    MPI_Status *stat = (MPI_Status*)((*env)->GetLongField(env, jthis,
                FID_mpi_Status_handle));
    return stat->MPI_SOURCE;
}

/*
 * Class:     mpi_Status
 * Method:    nativeGetTag
 * Signature: ()I
 */
JNIEXPORT jint JNICALL Java_mpi_Status_nativeGetTag(JNIEnv  *env,
                                                    jobject jthis)
{
    MPI_Status *stat = (MPI_Status*)((*env)->GetLongField(env, jthis,
                FID_mpi_Status_handle));
    return stat->MPI_TAG;
}

/*
 * Class:     mpi_Status
 * Method:    nativeGetIndex
 * Signature: ()I
 */
JNIEXPORT jint JNICALL Java_mpi_Status_nativeGetIndex(JNIEnv  *env,
                                                      jobject jthis)
{
    MPI_Status *stat = (MPI_Status*)((*env)->GetLongField(env, jthis,
                FID_mpi_Status_handle));
    return 0;
}
