/* Copyright (c) 2001-2022, The Ohio State University. All rights
 * reserved.
 *
 * This file is part of the MVAPICH2-J software package developed by the
 * team members of The Ohio State University's Network-Based Computing
 * Laboratory (NBCL), headed by Professor Dhabaleswar K. (DK) Panda.
 *
 * For detailed copyright and licensing information, please refer to the
 * copyright file COPYRIGHT in the top level MVAPICH2-J directory.
 */

/*
The MIT License

Copyright (c) 2005 - 2008
  1. Distributed Systems Group, University of Portsmouth (2005)
  2. Community Grids Laboratory, Indiana University (2005)
  3. Aamir Shafi (2005 - 2008)
  4. Bryan Carpenter (2005 - 2008)
  5. Mark Baker (2005 - 2008)

Permission is hereby granted, free of charge, to any person obtaining
a copy of this software and associated documentation files (the
"Software"), to deal in the Software without restriction, including
without limitation the rights to use, copy, modify, merge, publish,
distribute, sublicense, and/or sell copies of the Software, and to
permit persons to whom the Software is furnished to do so, subject to
the following conditions:

The above copyright notice and this permission notice shall be included
in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN
NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR
OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR
THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*/
/*
 * File         : Op.java
 * Author       : Aamir Shafi, Bryan Carpenter
 * Created      : Fri Sep 10 12:22:15 BST 2004
 * Revision     : $Revision: 1.91 $
 * Updated      : $Date: 2014/03/11 13:26:15 PKT $
 */

package mpi;

/**
 * This class represents {@code MPI_Op}.
 */
public class Op {
  // TODO: Document this and later make this private and provide a
  //	getter function
  public int opCode = -1; // MPI.UNDEFINED
  OpWorker worker = null;
  User_function funct = null;

  Op() {}

  /**
   * Bind a user-defined global reduction operation to an {@code Op} object.
   *
   * @param function user-defined function.
   * @param commute {@code true} if commutative, {@code false} otherwise.
   * 
   *
   * <p>Java binding of the MPI operation {@code MPI_OP_CREATE}.
   *
   * @throws MPIException Points out that an MPI exception has occured.
   */
  public Op(User_function function, boolean commute) throws MPIException {
    funct = function;
  }

  /**
   * Bind a user-defined global reduction operation to an {@code Op} object.
   *
   * @param function user-defined function.
   * @param commute {@code true} if commutative, {@code false} otherwise.
   * @param opCode_ op code.
   *
   * <p>Java binding of the MPI operation {@code MPI_OP_CREATE}.
   *
   * @throws MPIException Points out that an MPI exception has occured.
   */
  public Op(User_function function, boolean commute, int opCode_) throws MPIException {
    funct = function;
    this.opCode = opCode_;
  }

  /* this method should be something like abstract ...
    but its not possible coz these methods like free() and finalize and
    and constructor to create User_fuctions ..is needed here ..
  */
  void perform(Object buf1, int offset, int count) throws MPIException {}

  void createInitialBuffer(Object buf, int offset, int count) throws MPIException {}

  void getResultant(Object buf, int offset, int count) throws MPIException {}

  public void free() throws MPIException {}
  // TODO: document this new constructor

  public void finalize() throws MPIException {}

  void getWorker() throws MPIException {}
}
