/* Copyright (c) 2001-2022, The Ohio State University. All rights
 * reserved.
 *
 * This file is part of the MVAPICH2-J software package developed by the
 * team members of The Ohio State University's Network-Based Computing
 * Laboratory (NBCL), headed by Professor Dhabaleswar K. (DK) Panda.
 *
 * For detailed copyright and licensing information, please refer to the
 * copyright file COPYRIGHT in the top level MVAPICH2-J directory.
 */

/*
The MIT License

Copyright (c) 2005 - 2008
  1. Distributed Systems Group, University of Portsmouth (2005)
  2. Community Grids Laboratory, Indiana University (2005)
  3. Aamir Shafi (2005 - 2008)
  4. Bryan Carpenter (2005 - 2008)
  5. Mark Baker (2005 - 2008)

Permission is hereby granted, free of charge, to any person obtaining
a copy of this software and associated documentation files (the
"Software"), to deal in the Software without restriction, including
without limitation the rights to use, copy, modify, merge, publish,
distribute, sublicense, and/or sell copies of the Software, and to
permit persons to whom the Software is furnished to do so, subject to
the following conditions:

The above copyright notice and this permission notice shall be included
in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN
NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR
OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR
THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*/
/*
 * File         : Prequest.java
 * Author       : Aamir Shafi, Bryan Carpenter
 * Created      : Fri Sep 10 12:22:15 BST 2004
 * Revision     : $Revision: 1.10 $
 * Updated      : $Date: 2005/11/27 19:37:22 $
 */

package mpi;

/**
 * Persistent request object.
 * <p style="color:red"> Not Implemented in the current release </p>.
 */
public class Prequest extends Request {

  Object buf = null;

  int count = 0, dest = 0, tag = 0, op = 0;

  Datatype datatype = null;
  mpi.Comm comm = null;

  Request request = null;

  Prequest(
      Object buf,
      int count,
      Datatype datatype,
      int dest,
      int tag,
      int op,
      mpi.Comm comm) {
    this.buf = buf;
    this.count = count;
    this.datatype = datatype;
    this.dest = dest;
    this.tag = tag;
    this.op = op;
    this.comm = comm;
  }

  /**
   * Activate a list of communication requests.
   *
   * @param array_of_request array of requests.
   *
   * <p>Java binding of the MPI operation {@code MPI_STARTALL}.
   *
   * @throws MPIException Points out that an MPI exception has occured.
   */
  public static void startAll(Prequest[] array_of_request) throws MPIException {

    for (int i = 0; i < array_of_request.length; i++) {
      if (array_of_request[i] != null) array_of_request[i].start();
    }
  }

  /**
   * Activate a persistent communication request.
   *  
   * <p>Java binding of the MPI operation
   * {@code MPI_START}. The communication is completed by using the request in one of the
   * {@code wait} or {@code test} operations. On successful completion the request becomes
   * inactive again. It can be reactivated by a further call to {@code Start}.
   *
   * @throws MPIException Points out that an MPI exception has occured.
   */
  public void start() throws MPIException {

    switch (op) {
      case MPI.OP_SEND:
        request = comm.arraysIsend(buf, count, datatype, dest, tag, this);
        break;

      case MPI.OP_BSEND:
        request = comm.arraysIsend(buf, count, datatype, dest, tag, this);
        break;

      case MPI.OP_RSEND:
        request = comm.arraysIsend(buf, count, datatype, dest, tag, this);
        break;

      case MPI.OP_SSEND:
        request = comm.arraysIsend(buf, count, datatype, dest, tag, this);
        break;

      case MPI.OP_RECV:
        request = comm.arraysIrecv(buf, count, datatype, dest, tag, this);
        break;

      default:
        throw new MPIException("Invalid persistent request operation");
    }
  }
}
