/* Copyright (c) 2001-2022, The Ohio State University. All rights
 * reserved.
 *
 * This file is part of the MVAPICH2-J software package developed by the
 * team members of The Ohio State University's Network-Based Computing
 * Laboratory (NBCL), headed by Professor Dhabaleswar K. (DK) Panda.
 *
 * For detailed copyright and licensing information, please refer to the
 * copyright file COPYRIGHT in the top level MVAPICH2-J directory.
 */

package mpi.ccl;

/*
MPI-Java version :
Sang Lim (slim@npac.syr.edu)
Northeast Parallel Architectures Center at Syracuse University
12/1/98
*/

import mpi.*;

public class AllgathervWithArrays {
  public AllgathervWithArrays() {}

  public AllgathervWithArrays(String[] args) throws Exception {

    final int MAXLEN = 10;

    int root, i, j, k;
    int myself, tasks, stride = 15;

    MPI.Init(args);
    myself = MPI.COMM_WORLD.getRank();
    tasks = MPI.COMM_WORLD.getSize();

    if (tasks > 8) {
      if (myself == 0) System.out.println("AllgathervWithArrays must run with less than 8 tasks");

      MPI.Finalize();
      return;
    }

    int outInt[] = new int[MAXLEN];
    double outDouble[] = new double[MAXLEN];
    long outLong[] = new long[MAXLEN];
    float outFloat[] = new float[MAXLEN];
    short outShort[] = new short[MAXLEN];
    char outChar[] = new char[MAXLEN * MPI.CHAR.getByteSize()];
    boolean outBoolean[] = new boolean[MAXLEN * MPI.BOOLEAN.getByteSize()];
    byte outByte[] = new byte[MAXLEN];

    int inInt[] = new int[MAXLEN * stride * tasks];
    double inDouble[] = new double[MAXLEN * stride * tasks];
    long inLong[] = new long[MAXLEN * stride * tasks];
    float inFloat[] = new float[MAXLEN * stride * tasks];
    short inShort[] = new short[MAXLEN * stride * tasks];
    char inChar[] = new char[MAXLEN * stride * tasks * MPI.CHAR.getByteSize()];
    boolean inBoolean[] = new boolean[MAXLEN * stride * tasks * MPI.BOOLEAN.getByteSize()];
    byte inByte[] = new byte[MAXLEN * stride * tasks];

    int dis[] = new int[MAXLEN];
    int rcount[] = new int[MAXLEN];
    int ans[] = {
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1,
      1, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1,
      1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1,
      1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
    };
    boolean ansBool[] = {
      true, true, true, true, true, true, true, true, true, true, false, false, false, false, false, true, true, true, true, true, false, false, false, false, false, false, false, false, false, false, true,
      true, true, true, true, false, false, false, false, false, false, false, false, false, false, true, true, true, true, true, false, false, false, false, false, false, false, false, false, false, true, true,
      true, true, true, false, false, false, false, false, false, false, false, false, false, true, true, true, true, true, false, false, false, false, false, false, false, false, false, false, true, true, true,
      true, true, false, false, false, false, false, false, false, false, false, false, true, true, true, true, true, false, false, false, false, false, false, false, false, false, false
    };
    char ansChar[] = {
      'a', 'a', 'a', 'a', 'a', 'a', 'a', 'a', 'a', 'a', '\0', '\0', '\0', '\0', '\0', 'a', 'a', 'a', 'a', 'a', '\0', '\0', '\0', '\0', '\0', '\0', '\0', '\0', '\0', '\0', 'a',
      'a', 'a', 'a', 'a', '\0', '\0', '\0', '\0', '\0', '\0', '\0', '\0', '\0', '\0', 'a', 'a', 'a', 'a', 'a', '\0', '\0', '\0', '\0', '\0', '\0', '\0', '\0', '\0', '\0', 'a', 'a',
      'a', 'a', 'a', '\0', '\0', '\0', '\0', '\0', '\0', '\0', '\0', '\0', '\0', 'a', 'a', 'a', 'a', 'a', '\0', '\0', '\0', '\0', '\0', '\0', '\0', '\0', '\0', '\0', 'a', 'a', 'a',
      'a', 'a', '\0', '\0', '\0', '\0', '\0', '\0', '\0', '\0', '\0', '\0', 'a', 'a', 'a', 'a', 'a', '\0', '\0', '\0', '\0', '\0', '\0', '\0', '\0', '\0', '\0'
    };

    for (i = 0; i < MAXLEN; i++) {
      dis[i] = i * stride;
      rcount[i] = 5;
    
      outInt[i] = 1;
      outDouble[i] = (double)1;
      outBoolean[i] = true;
      outFloat[i] = (float)1;
      outShort[i] = (short)1;
      outLong[i] = (long)1;
      outByte[i] = (byte)1;
      outChar[i] = 'a';
    }

    rcount[0] = 10;

    for (i = 0; i < MAXLEN * tasks * stride; i++) {
      inInt[i] = 0;
      inDouble[i] = (double)0;
      inBoolean[i] = false;
      inFloat[i] = (float)0;
      inShort[i] = (short)0;
      inLong[i] = (long)0;
      inByte[i] = (byte)0;
      inChar[i] = '\0';
    }

    if (myself == 0) {
      MPI.COMM_WORLD.allGatherv(outInt, 10, MPI.INT, inInt, rcount, dis, MPI.INT);
      MPI.COMM_WORLD.allGatherv(outDouble, 10, MPI.DOUBLE, inDouble, rcount, dis, MPI.DOUBLE);
      MPI.COMM_WORLD.allGatherv(outFloat, 10, MPI.FLOAT, inFloat, rcount, dis, MPI.FLOAT);
      MPI.COMM_WORLD.allGatherv(outShort, 10, MPI.SHORT, inShort, rcount, dis, MPI.SHORT);
      MPI.COMM_WORLD.allGatherv(outLong, 10, MPI.LONG, inLong, rcount, dis, MPI.LONG);
      MPI.COMM_WORLD.allGatherv(outByte, 10, MPI.BYTE, inByte, rcount, dis, MPI.BYTE);
      MPI.COMM_WORLD.allGatherv(outChar, 10, MPI.CHAR, inChar, rcount, dis, MPI.CHAR);
      MPI.COMM_WORLD.allGatherv(outBoolean, 10, MPI.BOOLEAN, inBoolean, rcount, dis, MPI.BOOLEAN);
    } else {
      MPI.COMM_WORLD.allGatherv(outInt, 5, MPI.INT, inInt, rcount, dis, MPI.INT);
      MPI.COMM_WORLD.allGatherv(outDouble, 5, MPI.DOUBLE, inDouble, rcount, dis, MPI.DOUBLE);
      MPI.COMM_WORLD.allGatherv(outFloat, 5, MPI.FLOAT, inFloat, rcount, dis, MPI.FLOAT);
      MPI.COMM_WORLD.allGatherv(outShort, 5, MPI.SHORT, inShort, rcount, dis, MPI.SHORT);
      MPI.COMM_WORLD.allGatherv(outLong, 5, MPI.LONG, inLong, rcount, dis, MPI.LONG);
      MPI.COMM_WORLD.allGatherv(outByte, 5, MPI.BYTE, inByte, rcount, dis, MPI.BYTE);
      MPI.COMM_WORLD.allGatherv(outChar, 5, MPI.CHAR, inChar, rcount, dis, MPI.CHAR);
      MPI.COMM_WORLD.allGatherv(outBoolean, 5, MPI.BOOLEAN, inBoolean, rcount, dis, MPI.BOOLEAN);
    }

    for (i = 0; i < tasks * stride; i++) {
      int valInt = inInt[i];
      double valDouble = inDouble[i];
      long valLong = inLong[i];
      float valFloat = inFloat[i];
      short valShort = inShort[i];
      byte valByte = inByte[i];
      char valChar = inChar[i];
      boolean valBool = inBoolean[i];

      if (ans[i] != valInt) {
        System.out.println("INT: received data : " + valInt + " at [" + i + "] should be : " + ans[i]);
      }
      if ((double)ans[i] != valDouble) {
        System.out.println("DBL: received data : " + valDouble + " at [" + i + "] should be : " + (double)ans[i]);
      }
      if ((long)ans[i] != valLong) {
        System.out.println("LONG: received data : " + valLong + " at [" + i + "] should be : " + (long)ans[i]);
      }
      if ((short)ans[i] != valShort) {
        System.out.println("SHORT: received data : " + valShort + " at [" + i + "] should be : " + (short)ans[i]);
      }
      if ((float)ans[i] != valFloat) {
        System.out.println("FLOAT: received data : " + valFloat + " at [" + i + "] should be : " + (float)ans[i]);
      }
      if ((byte)ans[i] != valByte) {
        System.out.println("BYTE: received data : " + valByte + " at [" + i + "] should be : " + (byte)ans[i]);
      }
      if (ansChar[i] != valChar) {
        System.out.println("CHAR: received data : " + valChar + " at [" + i + "] should be : " + ansChar[i]);
      }  
      if (ansBool[i] != valBool) {
        System.out.println("BOOL: received data : " + valBool + " at [" + i + "] should be : " + ansBool[i]);
      }
    }

    MPI.COMM_WORLD.barrier();

    if (myself == 0) System.out.println("AllgathervWithArrays TEST COMPLETE");

    MPI.Finalize();
  }

  public static void main(String[] args) throws Exception {
    try {
      AllgathervWithArrays c = new AllgathervWithArrays(args);
    } catch (Exception e) {
      e.printStackTrace();
    }
  }
}
