/* Copyright (c) 2001-2022, The Ohio State University. All rights
 * reserved.
 *
 * This file is part of the MVAPICH2-J software package developed by the
 * team members of The Ohio State University's Network-Based Computing
 * Laboratory (NBCL), headed by Professor Dhabaleswar K. (DK) Panda.
 *
 * For detailed copyright and licensing information, please refer to the
 * copyright file COPYRIGHT in the top level MVAPICH2-J directory.
 */

package mpi.ccl;

/*
MPI-Java version :
Sang Lim (slim@npac.syr.edu)
Northeast Parallel Architectures Center at Syracuse University
12/1/98
*/

import mpi.*;

import java.nio.ByteBuffer;

public class AllgathervWithByteBuffer {
  public AllgathervWithByteBuffer() {}

  public AllgathervWithByteBuffer(String[] args) throws Exception {

    final int MAXLEN = 10;

    int root, i, j, k;
    int myself, tasks, stride = 15;

    MPI.Init(args);
    myself = MPI.COMM_WORLD.getRank();
    tasks = MPI.COMM_WORLD.getSize();

    if (tasks > 8) {
      if (myself == 0)
        System.out.println("AllgathervWithByteBuffer must run with less than 8 tasks");

      MPI.Finalize();
      return;
    }

    ByteBuffer outByte = ByteBuffer.allocateDirect(MAXLEN * MPI.BYTE.getByteSize());

    ByteBuffer inByte = ByteBuffer.allocateDirect(MAXLEN * stride * tasks * MPI.BYTE.getByteSize());

    int dis[] = new int[MAXLEN];
    int rcount[] = new int[MAXLEN];
    int ans[] = {
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1,
      1, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1,
      1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1,
      1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
    };

    for (i = 0; i < MAXLEN; i++) {
      dis[i] = i * stride;
      rcount[i] = 5;
      outByte.put((byte)1);
    }

    rcount[0] = 10;

    for (i = 0; i < MAXLEN * tasks * stride; i++) {
      inByte.put((byte)0);
    }

    if (myself == 0) {
      MPI.COMM_WORLD.allGatherv(outByte, 10, MPI.BYTE, inByte, rcount, dis, MPI.BYTE);
    } else {
      MPI.COMM_WORLD.allGatherv(outByte, 5, MPI.BYTE, inByte, rcount, dis, MPI.BYTE);
    }

    inByte.flip();

    outByte.flip();

    for (i = 0; i < tasks * stride; i++) {
      byte valByte = inByte.get();

      if ((byte)ans[i] != valByte) {
        System.out.println("received data : " + valByte + " at [" + i + "] should be : " + (byte)ans[i]);
      }
    }

    MPI.COMM_WORLD.barrier();

    if (myself == 0) System.out.println("AllgathervWithByteBuffer TEST COMPLETE");

    MPI.Finalize();
  }

  public static void main(String[] args) throws Exception {
    try {
      AllgathervWithByteBuffer c = new AllgathervWithByteBuffer(args);
    } catch (Exception e) {
      e.printStackTrace();
    }
  }
}
