/* Copyright (c) 2001-2022, The Ohio State University. All rights
 * reserved.
 *
 * This file is part of the MVAPICH2-J software package developed by the
 * team members of The Ohio State University's Network-Based Computing
 * Laboratory (NBCL), headed by Professor Dhabaleswar K. (DK) Panda.
 *
 * For detailed copyright and licensing information, please refer to the
 * copyright file COPYRIGHT in the top level MVAPICH2-J directory.
 */

package mpi.ccl;

/****************************************************************************
 *
 * MESSAGE PASSING INTERFACE TEST CASE SUITE
 *
 * Copyright IBM Corp. 1995
 *
 * IBM Corp. hereby grants a non-exclusive license to use, copy, modify, and
 * distribute this software for any purpose and without fee provided that the
 * above copyright notice and the following paragraphs appear in all copies.
 *
 * IBM Corp. makes no representation that the test cases comprising this
 * suite are correct or are an accurate representation of any standard.
 *
 * In no event shall IBM be liable to any party for direct, indirect, special
 * incidental, or consequential damage arising out of the use of this software
 * even if IBM Corp. has been advised of the possibility of such damage.
 *
 * IBM CORP. SPECIFICALLY DISCLAIMS ANY WARRANTIES INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE.  THE SOFTWARE PROVIDED HEREUNDER IS ON AN "AS IS" BASIS AND IBM
 * CORP. HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
 * ENHANCEMENTS, OR MODIFICATIONS.
 ****************************************************************************
 *
 * These test cases reflect an interpretation of the MPI Standard.  They are
 * are, in most cases, unit tests of specific MPI behaviors.  If a user of any
 * test case from this set believes that the MPI Standard requires behavior
 * different than that implied by the test case we would appreciate feedback.
 *
 * Comments may be sent to:
 * Richard Treumann
 * treumann@kgn.ibm.com
 ****************************************************************************
 *
 * MPI-Java version :
 * Sung-Hoon Ko(shko@npac.syr.edu)
 * Northeast Parallel Architectures Center at Syracuse University
 * 03/22/98
 ****************************************************************************/

import mpi.*;

import java.nio.*;

public class AlltoAllWithArrays {
  public AlltoAllWithArrays() {}

  public AlltoAllWithArrays(String[] args) throws Exception {

    final int MAXLEN = 10000;

    int i, j, k;
    int myself, tasks;

    MPI.Init(args);
    myself = MPI.COMM_WORLD.getRank();
    tasks = MPI.COMM_WORLD.getSize();

    int outInt[] = new int[MAXLEN * tasks];
    double outDouble[] = new double[MAXLEN * tasks];
    long outLong[] = new long[MAXLEN * tasks];
    boolean outBoolean[] = new boolean[MAXLEN * tasks];
    char outChar[] = new char[MAXLEN * tasks];
    float outFloat[] = new float[MAXLEN * tasks];
    short outShort[] = new short[MAXLEN * tasks];
    byte outByte[] = new byte[MAXLEN * tasks];

    int inInt[] = new int[MAXLEN * tasks];
    double inDouble[] = new double[MAXLEN * tasks];
    long inLong[] = new long[MAXLEN * tasks];
    boolean inBoolean[] = new boolean[MAXLEN * tasks];
    char inChar[] = new char[MAXLEN * tasks];
    float inFloat[] = new float[MAXLEN * tasks];
    short inShort[] = new short[MAXLEN * tasks];
    byte inByte[] = new byte[MAXLEN * tasks];

    for (j = 1; j <= MAXLEN; j *= 10) {
      for (i = 0; i < j * tasks; i++) {
        outInt[i] = myself;
        outDouble[i] = (double)myself;
        outLong[i] = (long)myself;
        outBoolean[i] = true;
        outChar[i] = 'a';
        outFloat[i] = (float)myself;
        outShort[i] = (short)myself;
        outByte[i] = (byte)myself;
      }

      MPI.COMM_WORLD.allToAll(outInt, j, MPI.INT, inInt, j, MPI.INT);
      MPI.COMM_WORLD.allToAll(outDouble, j, MPI.DOUBLE, inDouble, j, MPI.DOUBLE);
      MPI.COMM_WORLD.allToAll(outFloat, j, MPI.FLOAT, inFloat, j, MPI.FLOAT);
      MPI.COMM_WORLD.allToAll(outShort, j, MPI.SHORT, inShort, j, MPI.SHORT);
      MPI.COMM_WORLD.allToAll(outChar, j, MPI.CHAR, inChar, j, MPI.CHAR);
      MPI.COMM_WORLD.allToAll(outByte, j, MPI.BYTE, inByte, j, MPI.BYTE);
      MPI.COMM_WORLD.allToAll(outLong, j, MPI.LONG, inLong, j, MPI.LONG);
      MPI.COMM_WORLD.allToAll(outBoolean, j, MPI.BOOLEAN, inBoolean, j, MPI.BOOLEAN);

      for (i = 0; i < tasks; i++) {
        for (k = 0; k < j; k++) {
          int valInt = inInt[i * j + k];
          double valDouble = inDouble[i * j + k];
          float valFloat = inFloat[i * j + k];
          long valLong = inLong[i * j + k];
          char valChar = inChar[i * j + k];
          short valShort = inShort[i * j + k];
          byte valByte = inByte[i * j + k];
          boolean valBool = inBoolean[i * j + k];

          if (valInt != i) {
            System.out.println(
                "Int: bad answer ("
                    + (valInt)
                    + ") at index "
                    + (i * j + k)
                    + " of "
                    + (j * tasks)
                    + "(should be "
                    + i
                    + ")");
            break;
          }
          if (valByte != (byte)i) {
            System.out.println(
                "Byte: bad answer ("
                    + (valByte)
                    + ") at index "
                    + (i * j + k)
                    + " of "
                    + (j * tasks)
                    + "(should be "
                    + (byte)i
                    + ")");
            break;
          }
          if (valDouble != (double)i) {
            System.out.println(
                "Double: bad answer ("
                    + (valDouble)
                    + ") at index "
                    + (i * j + k)
                    + " of "
                    + (j * tasks)
                    + "(should be "
                    + (double)i
                    + ")");
            break;
          }  
          if (valFloat != (float)i) {
            System.out.println(
                "Float: bad answer ("
                    + (valFloat)
                    + ") at index "
                    + (i * j + k)
                    + " of "
                    + (j * tasks)
                    + "(should be "
                    + (float)i
                    + ")");
            break;
          }
          if (valLong != (long)i) {
            System.out.println(
                "Long: bad answer ("
                    + (valLong)
                    + ") at index "
                    + (i * j + k)
                    + " of "
                    + (j * tasks)
                    + "(should be "
                    + (long)i
                    + ")");
            break;
          }
          if (valShort != (short)i) {
            System.out.println(
                "Short: bad answer ("
                    + (valShort)
                    + ") at index "
                    + (i * j + k)
                    + " of "
                    + (j * tasks)
                    + "(should be "
                    + (short)i
                    + ")");
            break;
          }
          if (valChar != 'a') {
            System.out.println(
                "Char: bad answer ("
                    + (valChar)
                    + ") at index "
                    + (i * j + k)
                    + " of "
                    + (j * tasks)
                    + "(should be "
                    + 'a'
                    + ")");
            break;
          }
          if (valBool != true) {
            System.out.println(
                "Bool: bad answer ("
                    + (valBool)
                    + ") at index "
                    + (i * j + k)
                    + " of "
                    + (j * tasks)
                    + "(should be "
                    + "true"
                    + ")");
            break;
          }
        }
      }
    }

    MPI.COMM_WORLD.barrier();

    if (myself == 0) System.out.println("AllToAllWithArrays TEST COMPLETE");

    MPI.Finalize();
  }

  public static void main(String[] args) throws Exception {
    try {
      AlltoAllWithArrays c = new AlltoAllWithArrays(args);
    } catch (Exception e) {
      e.printStackTrace();
    }
  }
}
