/* Copyright (c) 2001-2022, The Ohio State University. All rights
 * reserved.
 *
 * This file is part of the MVAPICH2-J software package developed by the
 * team members of The Ohio State University's Network-Based Computing
 * Laboratory (NBCL), headed by Professor Dhabaleswar K. (DK) Panda.
 *
 * For detailed copyright and licensing information, please refer to the
 * copyright file COPYRIGHT in the top level MVAPICH2-J directory.
 */

package mpi.ccl;

/*
MPI-Java version :
Sang Lim (slim@npac.syr.edu)
Northeast Parallel Architectures Center at Syracuse University
12/2/98
*/

import mpi.*;

import java.nio.*;

public class AlltoallvWithArrays {
  public AlltoallvWithArrays() {}

  public AlltoallvWithArrays(String[] args) throws Exception {

    final int MAXLEN = 10;

    int myself, tasks;
    MPI.Init(args);
    myself = MPI.COMM_WORLD.getRank();
    tasks = MPI.COMM_WORLD.getSize();

    if (tasks > 8) {
      if (myself == 0)
        System.out.println("AlltoallvWithArrays must run with 8 " + "or less tasks! ");
      MPI.Finalize();
      return;
    }

    int root, i = 0, j, k, stride = 15;
    int outInt[] = new int[stride * tasks];
    double outDouble[] = new double[stride * tasks];
    long outLong[] = new long[stride * tasks];
    boolean outBoolean[] = new boolean[stride * tasks * MPI.BOOLEAN.getByteSize()];
    char outChar[] = new char[stride * tasks * MPI.CHAR.getByteSize()];
    float outFloat[] = new float[stride * tasks];
    short outShort[] = new short[stride * tasks];
    byte outByte[] = new byte[stride * tasks];

    int inInt[] = new int[stride * tasks];
    double inDouble[] = new double[stride * tasks];
    long inLong[] = new long[stride * tasks];
    boolean inBoolean[] = new boolean[stride * tasks * MPI.BOOLEAN.getByteSize()];
    char inChar[] = new char[stride * tasks * MPI.CHAR.getByteSize()];
    float inFloat[] = new float[stride * tasks];
    short inShort[] = new short[stride * tasks];
    byte inByte[] = new byte[stride * tasks];    
    
    int sdis[] = new int[tasks];
    int scount[] = new int[tasks];
    int rdis[] = new int[tasks];
    int rcount[] = new int[tasks];
    int ans[] = {
      0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 0, 0, 0, 0, 0,
      15, 16, 17, 18, 19, 20, 21, 22, 23, 24, 25, 26, 27, 28, 29,
      30, 31, 32, 33, 34, 35, 36, 37, 38, 39, 40, 41, 42, 43, 44,
      45, 46, 47, 48, 49, 50, 51, 52, 53, 54, 55, 56, 57, 58, 59,
      60, 61, 62, 63, 64, 65, 66, 67, 68, 69, 70, 71, 72, 73, 74,
      75, 76, 77, 78, 79, 80, 81, 82, 83, 84, 85, 86, 87, 88, 89,
      90, 91, 92, 93, 94, 95, 96, 97, 98, 99, 100, 101, 102, 103, 104,
      105, 106, 107, 108, 109, 110, 111, 112, 113, 114, 115, 116, 117, 118, 119
    };

    char ansChar[] = {
      'a', 'a', 'a', 'a', 'a', 'a', 'a', 'a', 'a', 'a', '\0', '\0', '\0', '\0', '\0',
      'a', 'a', 'a', 'a', 'a', 'a', 'a', 'a', 'a', 'a', 'a', 'a', 'a', 'a', 'a',
      'a', 'a', 'a', 'a', 'a', 'a', 'a', 'a', 'a', 'a', 'a', 'a', 'a', 'a', 'a',
      'a', 'a', 'a', 'a', 'a', 'a', 'a', 'a', 'a', 'a', 'a', 'a', 'a', 'a', 'a',
      'a', 'a', 'a', 'a', 'a', 'a', 'a', 'a', 'a', 'a', 'a', 'a', 'a', 'a', 'a',
      'a', 'a', 'a', 'a', 'a', 'a', 'a', 'a', 'a', 'a', 'a', 'a', 'a', 'a', 'a',
      'a', 'a', 'a', 'a', 'a', 'a', 'a', 'a', 'a', 'a', 'a', 'a', 'a', 'a', 'a',
      'a', 'a', 'a', 'a', 'a', 'a', 'a', 'a', 'a', 'a', 'a', 'a', 'a', 'a', 'a'
    };

    boolean ansBool[] = {
      true, true, true, true, true, true, true, true, true, true, false, false, false, false, false,
      true, true, true, true, true, true, true, true, true, true, true, true, true, true, true,
      true, true, true, true, true, true, true, true, true, true, true, true, true, true, true,
      true, true, true, true, true, true, true, true, true, true, true, true, true, true, true,
      true, true, true, true, true, true, true, true, true, true, true, true, true, true, true,
      true, true, true, true, true, true, true, true, true, true, true, true, true, true, true,
      true, true, true, true, true, true, true, true, true, true, true, true, true, true, true,
      true, true, true, true, true, true, true, true, true, true, true, true, true, true, true
    };
    
    for (i = 0; i < tasks; i++) {
      sdis[i] = i * stride;
      scount[i] = stride;
      rdis[i] = i * stride;
      rcount[i] = stride;
    }

    if (myself == 0) for (i = 0; i < tasks; i++) scount[i] = 10;

    rcount[0] = 10;

    for (j = 0; j < tasks; j++)
      for (i = 0; i < stride; i++) {
        int val = i + myself * stride;
        int idx = i + j * stride;

        outInt[idx] = val;
        outDouble[idx] = (double)val;
        outLong[idx] = (long)val;
        outBoolean[idx] = true;
        outChar[idx] = 'a';
        outFloat[idx] = (float)val;
        outShort[idx] = (short)val;
        outByte[idx] = (byte)val;        

        inInt[idx] = 0;
        inDouble[idx] = (double)0;
        inLong[idx] = (long)0;
        inBoolean[i] = false;
        inChar[idx] = '\0';
        inFloat[idx] = (float)0;
        inShort[idx] = (short)0;
        inByte[idx] = (byte)0;
      }

    MPI.COMM_WORLD.allToAllv(outInt, scount, sdis, MPI.INT, inInt, rcount, rdis, MPI.INT);
    MPI.COMM_WORLD.allToAllv(outDouble, scount, sdis, MPI.DOUBLE, inDouble, rcount, rdis, MPI.DOUBLE);
    MPI.COMM_WORLD.allToAllv(outFloat, scount, sdis, MPI.FLOAT, inFloat, rcount, rdis, MPI.FLOAT);
    MPI.COMM_WORLD.allToAllv(outLong, scount, sdis, MPI.LONG, inLong, rcount, rdis, MPI.LONG);
    MPI.COMM_WORLD.allToAllv(outShort, scount, sdis, MPI.SHORT, inShort, rcount, rdis, MPI.SHORT);
    MPI.COMM_WORLD.allToAllv(outChar, scount, sdis, MPI.CHAR, inChar, rcount, rdis, MPI.CHAR);
    MPI.COMM_WORLD.allToAllv(outByte, scount, sdis, MPI.BYTE, inByte, rcount, rdis, MPI.BYTE);
    MPI.COMM_WORLD.allToAllv(outBoolean, scount, sdis, MPI.BOOLEAN, inBoolean, rcount, rdis, MPI.BOOLEAN);

    for (i = 0; i < tasks * stride; i++) {
      int valInt = inInt[i];
      double valDouble = inDouble[i];
      float valFloat = inFloat[i];
      long valLong = inLong[i];
      char valChar = inChar[i];
      short valShort = inShort[i];
      byte valByte = inByte[i];
      boolean valBool = inBoolean[i];

      MPI.COMM_WORLD.barrier();
      if (ans[i] != valInt) {
        System.out.println(
            "INT: received data : "
                + valInt
                + " at ["
                + i
                + "]  should be : "
                + ans[i]
                + " on proc. : "
                + myself);
      }
      if ((double)ans[i] != valDouble) {
        System.out.println(
            "DOUBLE: received data : "
                + valDouble
                + " at ["
                + i
                + "]  should be : "
                + (double)ans[i]
                + " on proc. : "
                + myself);
      }  
      if ((float)ans[i] != valFloat) {
        System.out.println(
            "FLOAT: received data : "
                + valFloat
                + " at ["
                + i
                + "]  should be : "
                + (float)ans[i]
                + " on proc. : "
                + myself);
      }
      if ((long)ans[i] != valLong) {
        System.out.println(
            "LONG: received data : "
                + valLong
                + " at ["
                + i
                + "]  should be : "
                + (long)ans[i]
                + " on proc. : "
                + myself);
      }
      if ((short)ans[i] != valShort) {
        System.out.println(
            "SHORT: received data : "
                + valShort
                + " at ["
                + i
                + "]  should be : "
                + (short)ans[i]
                + " on proc. : "
                + myself);
      }
      if ((byte)ans[i] != valByte) {
        System.out.println(
            "BYTE: received data : "
                + valByte
                + " at ["
                + i
                + "]  should be : "
                + (byte)ans[i]
                + " on proc. : "
                + myself);
      }
      if (ansChar[i] != valChar) {
        System.out.println(
            "CHAR: received data : "
                + valChar
                + " at ["
                + i
                + "]  should be : "
                + ansChar[i]
                + " on proc. : "
                + myself);
      }
      if (ansBool[i] != valBool) {
        System.out.println(
            "BOOL: received data : "
                + valBool
                + " at ["
                + i
                + "]  should be : "
                + ansBool[i]
                + " on proc. : "
                + myself);
      }
      MPI.COMM_WORLD.barrier();
    }

    MPI.COMM_WORLD.barrier();

    if (myself == 0) {
      System.out.println("AlltoallvWithArrays TEST COMPLETE");
    }

    MPI.Finalize();
  }

  public static void main(String[] args) throws Exception {
    try {
      AlltoallvWithArrays c = new AlltoallvWithArrays(args);
    } catch (Exception e) {
      e.printStackTrace();
    }
  }
}
