/* Copyright (c) 2001-2022, The Ohio State University. All rights
 * reserved.
 *
 * This file is part of the MVAPICH2-J software package developed by the
 * team members of The Ohio State University's Network-Based Computing
 * Laboratory (NBCL), headed by Professor Dhabaleswar K. (DK) Panda.
 *
 * For detailed copyright and licensing information, please refer to the
 * copyright file COPYRIGHT in the top level MVAPICH2-J directory.
 */

package mpi.ccl;

/****************************************************************************
 *
 * MESSAGE PASSING INTERFACE TEST CASE SUITE
 *
 * Copyright IBM Corp. 1995
 *
 * IBM Corp. hereby grants a non-exclusive license to use, copy, modify, and
 * distribute this software for any purpose and without fee provided that the
 * above copyright notice and the following paragraphs appear in all copies.
 *
 * IBM Corp. makes no representation that the test cases comprising this
 * suite are correct or are an accurate representation of any standard.
 *
 * In no event shall IBM be liable to any party for direct, indirect, special
 * incidental, or consequential damage arising out of the use of this software
 * even if IBM Corp. has been advised of the possibility of such damage.
 *
 * IBM CORP. SPECIFICALLY DISCLAIMS ANY WARRANTIES INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE.  THE SOFTWARE PROVIDED HEREUNDER IS ON AN "AS IS" BASIS AND IBM
 * CORP. HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
 * ENHANCEMENTS, OR MODIFICATIONS.
 ****************************************************************************
 *
 * These test cases reflect an interpretation of the MPI Standard.  They are
 * are, in most cases, unit tests of specific MPI behaviors.  If a user of any
 * test case from this set believes that the MPI Standard requires behavior
 * different than that implied by the test case we would appreciate feedback.
 *
 * Comments may be sent to:
 * Richard Treumann
 * treumann@kgn.ibm.com
 ****************************************************************************
 *
 * MPI-Java version :
 * Sung-Hoon Ko(shko@npac.syr.edu)
 * Northeast Parallel Architectures Center at Syracuse University
 * 03/22/98
 ****************************************************************************/

import mpi.*;

public class BcastWithArrays {
  public BcastWithArrays() {}

  public BcastWithArrays(String[] args) throws Exception {

    final int MAXLEN = 10000;

    int root, i, j, k;
    
    int outInt[] = new int[MAXLEN];
    double outDouble[] = new double[MAXLEN];
    long outLong[] = new long[MAXLEN];
    boolean outBoolean[] = new boolean[MAXLEN];
    char outChar[] = new char[MAXLEN];
    float outFloat[] = new float[MAXLEN];
    short outShort[] = new short[MAXLEN];
    byte outByte[] = new byte[MAXLEN];

    int myself, tasks;
    double time;

    MPI.Init(args);
    myself = MPI.COMM_WORLD.getRank();
    tasks = MPI.COMM_WORLD.getSize();

    root = tasks - 1;
    for (j = 1; j <= MAXLEN; j *= 10) {
      if (myself == root) {
        for (i = 0; i < j; i++) {
          outInt[i] = i;
          outDouble[i] = (double)i;
          outLong[i] = (long)i;
          outBoolean[i] = false;
          outChar[i] = 'a';
          outFloat[i] = (float)i;
          outShort[i] = (short)i;
          outByte[i] = (byte)i;
        }
      }

      MPI.COMM_WORLD.bcast(outInt, j, MPI.INT, root);
      MPI.COMM_WORLD.bcast(outDouble, j, MPI.DOUBLE, root);
      MPI.COMM_WORLD.bcast(outLong, j, MPI.LONG, root);
      MPI.COMM_WORLD.bcast(outBoolean, j, MPI.BOOLEAN, root);
      MPI.COMM_WORLD.bcast(outChar, j, MPI.CHAR, root);
      MPI.COMM_WORLD.bcast(outFloat, j, MPI.FLOAT, root);
      MPI.COMM_WORLD.bcast(outShort, j, MPI.SHORT, root);
      MPI.COMM_WORLD.bcast(outByte, j, MPI.BYTE, root);

      for (k = 0; k < j; k++) {
        if (outInt[k] != k) {
          System.out.println(
              "bad answer ("
                  + (outInt[k])
                  + ") at index "
                  + k
                  + " of "
                  + j
                  + " (should be "
                  + k
                  + ")");
          break;
        }
        if (outDouble[k] != (double)k) {
          System.out.println(
              "bad answer ("
                  + (outDouble[k])
                  + ") at index "
                  + k
                  + " of "
                  + j
                  + " (should be "
                  + k
                  + ")");
          break;
        }
        if (outLong[k] != (long)k) {
          System.out.println(
              "bad answer ("
                  + (outLong[k])
                  + ") at index "
                  + k
                  + " of "
                  + j
                  + " (should be "
                  + k
                  + ")");
          break;
        }
        if (outBoolean[k] != false) {
          System.out.println(
              "bad answer ("
                  + (outBoolean[k])
                  + ") at index "
                  + k
                  + " of "
                  + j
                  + " (should be "
                  + "false"
                  + ")");
          break;
        }
        if (outChar[k] != 'a') {
          System.out.println(
              "bad answer ("
                  + (outChar[k])
                  + ") at index "
                  + k
                  + " of "
                  + j
                  + " (should be "
                  + 'a'
                  + ")");
          break;
        }
        if (outFloat[k] != (float)k) {
          System.out.println(
              "bad answer ("
                  + (outFloat[k])
                  + ") at index "
                  + k
                  + " of "
                  + j
                  + " (should be "
                  + k
                  + ")");
          break;
        }
        if (outShort[k] != (short)k) {
          System.out.println(
              "bad answer ("
                  + (outShort[k])
                  + ") at index "
                  + k
                  + " of "
                  + j
                  + " (should be "
                  + k
                  + ")");
          break;
        }
        if (outByte[k] != (byte)k) {
          System.out.println(
              "bad answer ("
                  + (outByte[k])
                  + ") at index "
                  + k
                  + " of "
                  + j
                  + " (should be "
                  + k
                  + ")");
          break;
        }
      }
    }

    MPI.COMM_WORLD.barrier();
    if (myself == 0) System.out.println("Bcast TEST COMPLETE");
    MPI.Finalize();
  }

  public static void main(String[] args) throws Exception {
    try {
      BcastWithArrays c = new BcastWithArrays(args);
    } catch (Exception e) {
    }
  }
}
