/* Copyright (c) 2001-2022, The Ohio State University. All rights
 * reserved.
 *
 * This file is part of the MVAPICH2-J software package developed by the
 * team members of The Ohio State University's Network-Based Computing
 * Laboratory (NBCL), headed by Professor Dhabaleswar K. (DK) Panda.
 *
 * For detailed copyright and licensing information, please refer to the
 * copyright file COPYRIGHT in the top level MVAPICH2-J directory.
 */

package mpi.ccl;

/****************************************************************************
 *
 * MESSAGE PASSING INTERFACE TEST CASE SUITE
 *
 * Copyright IBM Corp. 1995
 *
 * IBM Corp. hereby grants a non-exclusive license to use, copy, modify, and
 * distribute this software for any purpose and without fee provided that the
 * above copyright notice and the following paragraphs appear in all copies.
 *
 * IBM Corp. makes no representation that the test cases comprising this
 * suite are correct or are an accurate representation of any standard.
 *
 * In no event shall IBM be liable to any party for direct, indirect, special
 * incidental, or consequential damage arising out of the use of this software
 * even if IBM Corp. has been advised of the possibility of such damage.
 *
 * IBM CORP. SPECIFICALLY DISCLAIMS ANY WARRANTIES INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE.  THE SOFTWARE PROVIDED HEREUNDER IS ON AN "AS IS" BASIS AND IBM
 * CORP. HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
 * ENHANCEMENTS, OR MODIFICATIONS.
 ****************************************************************************
 *
 * These test cases reflect an interpretation of the MPI Standard.  They are
 * are, in most cases, unit tests of specific MPI behaviors.  If a user of any
 * test case from this set believes that the MPI Standard requires behavior
 * different than that implied by the test case we would appreciate feedback.
 *
 * Comments may be sent to:
 * Richard Treumann
 * treumann@kgn.ibm.com
 ****************************************************************************
 *
 * MPI-Java version :
 * Sung-Hoon Ko(shko@npac.syr.edu)
 * Northeast Parallel Architectures Center at Syracuse University
 * 03/22/98
 ****************************************************************************/

import mpi.*;

public class GatherWithArrays {
  public GatherWithArrays() {}

  public GatherWithArrays(String[] args) throws Exception {

    final int MAXLEN = 10000;

    int root, i, j, k;
    
    int outInt[] = new int[MAXLEN];
    double outDouble[] = new double[MAXLEN];
    long outLong[] = new long[MAXLEN];
    boolean outBoolean[] = new boolean[MAXLEN];
    char outChar[] = new char[MAXLEN];
    float outFloat[] = new float[MAXLEN];
    short outShort[] = new short[MAXLEN];
    byte outByte[] = new byte[MAXLEN];

    int myself, tasks;
    double time;

    MPI.Init(args);
    myself = MPI.COMM_WORLD.getRank();
    tasks = MPI.COMM_WORLD.getSize();

    int inInt[] = new int[MAXLEN * tasks];
    double inDouble[] = new double[MAXLEN * tasks];
    long inLong[] = new long[MAXLEN * tasks];
    boolean inBoolean[] = new boolean[MAXLEN * tasks];
    char inChar[] = new char[MAXLEN * tasks];
    float inFloat[] = new float[MAXLEN * tasks];
    short inShort[] = new short[MAXLEN * tasks];
    byte inByte[] = new byte[MAXLEN * tasks];

    for (j = 1, root = 0; j <= MAXLEN; j *= 10, root = (root + 1) % tasks) {
      for (i = 0; i < j; i++) {
        outInt[i] = i;
        outDouble[i] = (double)i;
        outLong[i] = (long)i;
        outBoolean[i] = false;
        outChar[i] = 'a';
        outFloat[i] = (float)i;
        outShort[i] = (short)i;
        outByte[i] = (byte)i;
      }

      MPI.COMM_WORLD.gather(outByte, j, MPI.BYTE, inByte, j, MPI.BYTE, root);
      MPI.COMM_WORLD.gather(outInt, j, MPI.INT, inInt, j, MPI.INT, root);
      MPI.COMM_WORLD.gather(outDouble, j, MPI.DOUBLE, inDouble, j, MPI.DOUBLE, root);
      MPI.COMM_WORLD.gather(outLong, j, MPI.LONG, inLong, j, MPI.LONG, root);
      MPI.COMM_WORLD.gather(outChar, j, MPI.CHAR, inChar, j, MPI.CHAR, root);
      MPI.COMM_WORLD.gather(outShort, j, MPI.SHORT, inShort, j, MPI.SHORT, root);
      MPI.COMM_WORLD.gather(outFloat, j, MPI.FLOAT, inFloat, j, MPI.FLOAT, root);

      if (myself == root) {
        for (i = 0; i < tasks; i++) {
          for (k = 0; k < j; k++) {
            int valInt = inInt[i * j + k];
            Double valDouble = inDouble[i * j + k];
            Float valFloat = inFloat[i * j + k];
            Long valLong = inLong[i * j + k];
            char valChar = inChar[i * j + k];
            Short valShort = inShort[i * j + k];
            byte valByte = inByte[i * j + k];

            if (valInt != k) {
              System.out.println(
                  "Int: bad answer ("
                      + (valInt)
                      + ") at index "
                      + (i * j + k)
                      + " of "
                      + (j * tasks)
                      + "(should be "
                      + k
                      + ")");
              break;
            }
            if (valByte != (byte)k) {
              System.out.println(
                  "Byte: bad answer ("
                      + (valByte)
                      + ") at index "
                      + (i * j + k)
                      + " of "
                      + (j * tasks)
                      + "(should be "
                      + (byte)k
                      + ")");
              break;
            }
            if (valDouble != (double)k) {
              System.out.println(
                  "Double: bad answer ("
                      + (valDouble)
                      + ") at index "
                      + (i * j + k)
                      + " of "
                      + (j * tasks)
                      + "(should be "
                      + (double)k
                      + ")");
              break;
            }  
            if (valFloat != (float)k) {
              System.out.println(
                  "Float: bad answer ("
                      + (valFloat)
                      + ") at index "
                      + (i * j + k)
                      + " of "
                      + (j * tasks)
                      + "(should be "
                      + (float)k
                      + ")");
              break;
            }
            if (valLong != (long)k) {
              System.out.println(
                  "Long: bad answer ("
                      + (valLong)
                      + ") at index "
                      + (i * j + k)
                      + " of "
                      + (j * tasks)
                      + "(should be "
                      + (long)k
                      + ")");
              break;
            }
            if (valShort != (short)k) {
              System.out.println(
                  "Short: bad answer ("
                      + (valShort)
                      + ") at index "
                      + (i * j + k)
                      + " of "
                      + (j * tasks)
                      + "(should be "
                      + (short)k
                      + ")");
              break;
            }
            if (valChar != 'a') {
              System.out.println(
                  "Char: bad answer ("
                      + (valChar)
                      + ") at index "
                      + (i * j + k)
                      + " of "
                      + (j * tasks)
                      + "(should be "
                      + 'a'
                      + ")");
              break;
            }
          }
        }
      }
    }

    MPI.COMM_WORLD.barrier();
    if (myself == 0) System.out.println("Gather TEST COMPLETE");
    MPI.Finalize();
  }

  public static void main(String[] args) throws Exception {
    try {
      GatherWithArrays c = new GatherWithArrays(args);
    } catch (Exception e) {
    }
  }
}
