/* Copyright (c) 2001-2022, The Ohio State University. All rights
 * reserved.
 *
 * This file is part of the MVAPICH2-J software package developed by the
 * team members of The Ohio State University's Network-Based Computing
 * Laboratory (NBCL), headed by Professor Dhabaleswar K. (DK) Panda.
 *
 * For detailed copyright and licensing information, please refer to the
 * copyright file COPYRIGHT in the top level MVAPICH2-J directory.
 */

package mpi.ccl;

/*
MPI-Java version :
Sang Lim (slim@npac.syr.edu)
Northeast Parallel Architectures Center at Syracuse University
12/1/98
*/

import mpi.*;

public class GathervWithArrays {
  public GathervWithArrays() {}

  public GathervWithArrays(String[] args) throws Exception {

    final int MAXLEN = 10;

    int root, i, j, k;
    int myself, tasks, stride = 15;

    MPI.Init(args);
    myself = MPI.COMM_WORLD.getRank();
    tasks = MPI.COMM_WORLD.getSize();

    if (tasks > 8) {
      if (myself == 0) System.out.println("GathervWithArrays must run with less than 8 tasks");

      MPI.Finalize();
      return;
    }

    int outInt[] = new int[MAXLEN * MPI.INT.getByteSize()];
    double outDouble[] = new double[MAXLEN * MPI.DOUBLE.getByteSize()];
    long outLong[] = new long[MAXLEN * MPI.LONG.getByteSize()];
    float outFloat[] = new float[MAXLEN * MPI.FLOAT.getByteSize()];
    short outShort[] = new short[MAXLEN * MPI.SHORT.getByteSize()];
    char outChar[] = new char[MAXLEN * MPI.CHAR.getByteSize()];
    boolean outBoolean[] = new boolean[MAXLEN * MPI.BOOLEAN.getByteSize()];
    byte outByte[] = new byte[MAXLEN * MPI.BYTE.getByteSize()];

    int inInt[] = new int[MAXLEN * stride * tasks * MPI.INT.getByteSize()];
    double inDouble[] = new double[MAXLEN * stride * tasks * MPI.INT.getByteSize()];
    long inLong[] = new long[MAXLEN * stride * tasks * MPI.LONG.getByteSize()];
    float inFloat[] = new float[MAXLEN * stride * tasks * MPI.FLOAT.getByteSize()];
    short inShort[] = new short[MAXLEN * stride * tasks * MPI.SHORT.getByteSize()];
    char inChar[] = new char[MAXLEN * stride * tasks * MPI.CHAR.getByteSize()];
    boolean inBoolean[] = new boolean[MAXLEN * stride * tasks * MPI.BOOLEAN.getByteSize()];
    byte inByte[] = new byte[MAXLEN * stride * tasks * MPI.BYTE.getByteSize()];

    int dis[] = new int[MAXLEN];
    int rcount[] = new int[MAXLEN];
    int ans[] = {
      0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
      1, 1, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
      2, 2, 2, 2, 2, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
      3, 3, 3, 3, 3, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
      4, 4, 4, 4, 4, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
      5, 5, 5, 5, 5, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
      6, 6, 6, 6, 6, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
      7, 7, 7, 7, 7, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
    };

    boolean ansBool[] = {
      true, true, true, true, true, true, true, true, true, true, false, false, false, false, false, 
      true, true, true, true, true, false, false, false, false, false, false, false, false, false, false, 
      true, true, true, true, true, false, false, false, false, false, false, false, false, false, false, 
      true, true, true, true, true, false, false, false, false, false, false, false, false, false, false, 
      true, true, true, true, true, false, false, false, false, false, false, false, false, false, false, 
      true, true, true, true, true, false, false, false, false, false, false, false, false, false, false, 
      true, true, true, true, true, false, false, false, false, false, false, false, false, false, false, 
      true, true, true, true, true, false, false, false, false, false, false, false, false, false, false
    };

    char ansChar[] = {
      '0', '0', '0', '0', '0',  '0',   '0',   '0',   '0',   '0', '\0', '\0', '\0', '\0', '\0', 
      '1', '1', '1', '1', '1', '\0',  '\0',  '\0',  '\0',  '\0', '\0', '\0', '\0', '\0', '\0', 
      '2', '2', '2', '2', '2', '\0',  '\0',  '\0',  '\0',  '\0', '\0', '\0', '\0', '\0', '\0', 
      '3', '3', '3', '3', '3', '\0',  '\0',  '\0',  '\0',  '\0', '\0', '\0', '\0', '\0', '\0', 
      '4', '4', '4', '4', '4', '\0',  '\0',  '\0',  '\0',  '\0', '\0', '\0', '\0', '\0', '\0', 
      '5', '5', '5', '5', '5', '\0',  '\0',  '\0',  '\0',  '\0', '\0', '\0', '\0', '\0', '\0', 
      '6', '6', '6', '6', '6', '\0',  '\0',  '\0',  '\0',  '\0', '\0', '\0', '\0', '\0', '\0', 
      '7', '7', '7', '7', '7', '\0',  '\0',  '\0',  '\0',  '\0', '\0', '\0', '\0', '\0', '\0'
    };

    for (i = 0; i < MAXLEN; i++) {
      dis[i] = i * stride;
      rcount[i] = 5;
      outInt[i] = myself;
      outDouble[i] = (double) myself;
      outBoolean[i] = true;
      outFloat[i] = (float) myself;
      outShort[i] = (short) myself;
      outLong[i] = (long) myself;
      outByte[i] = (byte) myself;
      outChar[i] =  (char) (myself + '0');
    }

    rcount[0] = 10;

    for (i = 0; i < MAXLEN * tasks * stride; i++) {
      inInt[i] = -1;
      inDouble[i] = (double)0;
      inBoolean[i] = false;
      inFloat[i] = (float)0;
      inShort[i] = (short)0;
      inLong[i] = (long)0;
      inByte[i] = (byte)0;
      inChar[i] = '-';
    }

    if (myself == 0) {
      MPI.COMM_WORLD.gatherv(outInt, 10, MPI.INT, inInt, rcount, dis, MPI.INT, 0);
      MPI.COMM_WORLD.gatherv(outDouble, 10, MPI.DOUBLE, inDouble, rcount, dis, MPI.DOUBLE, 0);
      MPI.COMM_WORLD.gatherv(outFloat, 10, MPI.FLOAT, inFloat, rcount, dis, MPI.FLOAT, 0);
      MPI.COMM_WORLD.gatherv(outShort, 10, MPI.SHORT, inShort, rcount, dis, MPI.SHORT, 0);
      MPI.COMM_WORLD.gatherv(outLong, 10, MPI.LONG, inLong, rcount, dis, MPI.LONG, 0);
      MPI.COMM_WORLD.gatherv(outByte, 10, MPI.BYTE, inByte, rcount, dis, MPI.BYTE, 0);
      MPI.COMM_WORLD.gatherv(outChar, 10, MPI.CHAR, inChar, rcount, dis, MPI.CHAR, 0);
      MPI.COMM_WORLD.gatherv(outBoolean, 10, MPI.BOOLEAN, inBoolean, rcount, dis, MPI.BOOLEAN, 0);
    } else {
      MPI.COMM_WORLD.gatherv(outInt, 5, MPI.INT, inInt, rcount, dis, MPI.INT, 0);
      MPI.COMM_WORLD.gatherv(outDouble, 5, MPI.DOUBLE, inDouble, rcount, dis, MPI.DOUBLE, 0);
      MPI.COMM_WORLD.gatherv(outFloat, 5, MPI.FLOAT, inFloat, rcount, dis, MPI.FLOAT, 0);
      MPI.COMM_WORLD.gatherv(outShort, 5, MPI.SHORT, inShort, rcount, dis, MPI.SHORT, 0);
      MPI.COMM_WORLD.gatherv(outLong, 5, MPI.LONG, inLong, rcount, dis, MPI.LONG, 0);
      MPI.COMM_WORLD.gatherv(outByte, 5, MPI.BYTE, inByte, rcount, dis, MPI.BYTE, 0);
      MPI.COMM_WORLD.gatherv(outChar, 5, MPI.CHAR, inChar, rcount, dis, MPI.CHAR, 0);
      MPI.COMM_WORLD.gatherv(outBoolean, 5, MPI.BOOLEAN, inBoolean, rcount, dis, MPI.BOOLEAN, 0);
    }

    if (myself == 0) {

      int recvCountTotal = dis[tasks - 1] + rcount[tasks - 1];

      for (i = 0; i < recvCountTotal; i++) {
        int valInt = inInt[i];
        double valDouble = inDouble[i];
        long valLong = inLong[i];
        float valFloat = inFloat[i];
        short valShort = inShort[i];
        byte valByte = inByte[i];
        char valChar = inChar[i];
        boolean valBool = inBoolean[i];

        if (ans[i] != valInt) {
          System.out.println("received data : (" + valInt + ") at [" + i + "], should be : (" + ans[i] + ")");
        }

        if ((double)ans[i] != valDouble) {
          System.out.println("received data : " + valDouble + "at [" + i + "] should be : " + (double)ans[i]);
        }

        if ((float)ans[i] != valFloat) {
          System.out.println("received data : " + valFloat + "at [" + i + "] should be : " + (float)ans[i]);
        }

        if ((short)ans[i] != valShort) {
          System.out.println("received data : " + valShort + "at [" + i + "] should be : " + (short)ans[i]);
        }
        if ((long)ans[i] != valLong) {
          System.out.println("received data : " + valLong + "at [" + i + "] should be : " + (long)ans[i]);
        }
        if ((byte)ans[i] != valByte) {
          System.out.println("received data : " + valByte + "at [" + i + "] should be : " + (byte)ans[i]);
        }

        if (ansChar[i] != valChar) {
          System.out.println("received data : (" + valChar + ") at [" + i + "], should be : (" + ansChar[i] + ")");
        }

        if (ansBool[i] != valBool) {
          System.out.println("received data : " + valBool + "at [" + i + "] should be : " + ansBool[i]);
        }
      }
    }

    MPI.COMM_WORLD.barrier();

    if (myself == 0) System.out.println("GathervWithArrays TEST COMPLETE");

    MPI.Finalize();
  }

  public static void main(String[] args) throws Exception {
    try {
      GathervWithArrays c = new GathervWithArrays(args);
    } catch (Exception e) {
      e.printStackTrace();
    }
  }
}
