/* Copyright (c) 2001-2022, The Ohio State University. All rights
 * reserved.
 *
 * This file is part of the MVAPICH2-J software package developed by the
 * team members of The Ohio State University's Network-Based Computing
 * Laboratory (NBCL), headed by Professor Dhabaleswar K. (DK) Panda.
 *
 * For detailed copyright and licensing information, please refer to the
 * copyright file COPYRIGHT in the top level MVAPICH2-J directory.
 */

package mpi.ccl;

import mpi.*;
import java.nio.*;

public class ReduceMPIByteTest {
  public ReduceMPIByteTest() {}

  public ReduceMPIByteTest(String[] args) throws Exception {
    MPI.Init(args);
    int rank = MPI.COMM_WORLD.getRank();
    int size = MPI.COMM_WORLD.getSize();

    if (size != 4) {
        System.out.println("This application is meant to be run with 4 processes.\n");
        System.exit(0);
    } 

    byte vals[] = new byte[]{0, 1, 1, 0};
    byte myval[] = new byte[]{vals[rank]};
    byte result[] = new byte[]{0}; 
    MPI.COMM_WORLD.reduce(myval, result, 1, MPI.BYTE, MPI.BOR, 0);
    
    if (rank == 0 && result[0] != (byte)1) {
      System.out.println("Unexpected result, should be 1");
    }

    ByteBuffer valsBuffer = ByteBuffer.allocateDirect(4 * MPI.BYTE.getByteSize());
    ByteBuffer myvalBuffer = ByteBuffer.allocateDirect(1 * MPI.BYTE.getByteSize());
    ByteBuffer resultBuffer = ByteBuffer.allocateDirect(1 * MPI.BYTE.getByteSize());

    valsBuffer.put((byte)0);
    valsBuffer.put((byte)1);
    valsBuffer.put((byte)1);
    valsBuffer.put((byte)0);
    myvalBuffer.put((byte)valsBuffer.get(rank));
    resultBuffer.put((byte)0);
    
    MPI.COMM_WORLD.reduce(myvalBuffer, resultBuffer, 1, MPI.CHAR, MPI.BOR, 0);
 
    resultBuffer.flip();

    try {
      if (rank == 0 && resultBuffer.get() != 1) {
        System.out.println("Unexpected result, should be 1");
      }
    } catch (Exception e) {
      e.printStackTrace();
    }

    MPI.COMM_WORLD.barrier();
    if (rank == 0) System.out.println("Reduce TEST COMPLETE");
    MPI.Finalize();
  }
   
  public static void main(String[] args) throws Exception {
    try {
      ReduceMPIByteTest c = new ReduceMPIByteTest(args);
    } catch (Exception e) {
    }
  }
}

