/* Copyright (c) 2001-2022, The Ohio State University. All rights
 * reserved.
 *
 * This file is part of the MVAPICH2-J software package developed by the
 * team members of The Ohio State University's Network-Based Computing
 * Laboratory (NBCL), headed by Professor Dhabaleswar K. (DK) Panda.
 *
 * For detailed copyright and licensing information, please refer to the
 * copyright file COPYRIGHT in the top level MVAPICH2-J directory.
 */

package mpi.ccl;

import mpi.*;
import java.nio.ByteBuffer;
import java.nio.CharBuffer;

public class ReduceMPICharTest {
  public ReduceMPICharTest() {}

  public ReduceMPICharTest(String[] args) throws Exception {
    MPI.Init(args);
    int rank = MPI.COMM_WORLD.getRank();
    int size = MPI.COMM_WORLD.getSize();

    if (size != 4) {
        System.out.println("This application is meant to be run with 4 processes.\n");
        System.exit(0);
    } 

    char vals[] = new char[]{0, 1, 3, 8};
    char myval[] = new char[]{vals[rank]};
    char result[] = new char[]{0}; 
    MPI.COMM_WORLD.reduce(myval, result, 1, MPI.CHAR, MPI.BOR, 0);
    
    if (rank == 0 && (int)result[0] != 11) {
      System.out.println("Unexpected result, should be 11");
    }

    ByteBuffer valsBuffer = ByteBuffer.allocateDirect(4 * MPI.CHAR.getByteSize());
    ByteBuffer myvalBuffer = ByteBuffer.allocateDirect(1 * MPI.CHAR.getByteSize());
    ByteBuffer resultBuffer = ByteBuffer.allocateDirect(1 * MPI.CHAR.getByteSize());

    CharBuffer charValsBuffer = valsBuffer.asCharBuffer();

    charValsBuffer.put((char)0);
    charValsBuffer.put((char)1);
    charValsBuffer.put((char)3);
    charValsBuffer.put((char)8);
    resultBuffer.putChar((char)0);
    
    MPI.COMM_WORLD.reduce(myvalBuffer, resultBuffer, 1, MPI.CHAR, MPI.BOR, 0);
 
    CharBuffer charResBuffer = resultBuffer.asCharBuffer();

    try {
      if (rank == 0 && charResBuffer.get() != 11) {
        System.out.println("Unexpected result, should be 11");
      }
    } catch (Exception e) {
      e.printStackTrace();
    }

    MPI.COMM_WORLD.barrier();
    if (rank == 0) System.out.println("Reduce TEST COMPLETE");
    MPI.Finalize();
  }
   
  public static void main(String[] args) throws Exception {
    try {
      ReduceMPICharTest c = new ReduceMPICharTest(args);
    } catch (Exception e) {
    }
  }
}

