/* Copyright (c) 2001-2022, The Ohio State University. All rights
 * reserved.
 *
 * This file is part of the MVAPICH2-J software package developed by the
 * team members of The Ohio State University's Network-Based Computing
 * Laboratory (NBCL), headed by Professor Dhabaleswar K. (DK) Panda.
 *
 * For detailed copyright and licensing information, please refer to the
 * copyright file COPYRIGHT in the top level MVAPICH2-J directory.
 */

package mpi.ccl;

/****************************************************************************
 *
 * MESSAGE PASSING INTERFACE TEST CASE SUITE
 *
 * Copyright IBM Corp. 1995
 *
 * IBM Corp. hereby grants a non-exclusive license to use, copy, modify, and
 * distribute this software for any purpose and without fee provided that the
 * above copyright notice and the following paragraphs appear in all copies.
 *
 * IBM Corp. makes no representation that the test cases comprising this
 * suite are correct or are an accurate representation of any standard.
 *
 * In no event shall IBM be liable to any party for direct, indirect, special
 * incidental, or consequential damage arising out of the use of this software
 * even if IBM Corp. has been advised of the possibility of such damage.
 *
 * IBM CORP. SPECIFICALLY DISCLAIMS ANY WARRANTIES INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE.  THE SOFTWARE PROVIDED HEREUNDER IS ON AN "AS IS" BASIS AND IBM
 * CORP. HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
 * ENHANCEMENTS, OR MODIFICATIONS.
 ****************************************************************************
 *
 * These test cases reflect an interpretation of the MPI Standard.  They are
 * are, in most cases, unit tests of specific MPI behaviors.  If a user of any
 * test case from this set believes that the MPI Standard requires behavior
 * different than that implied by the test case we would appreciate feedback.
 *
 * Comments may be sent to:
 * Richard Treumann
 * treumann@kgn.ibm.com
 ****************************************************************************
 *
 * MPI-Java version :
 * Sung-Hoon Ko(shko@npac.syr.edu)
 * Northeast Parallel Architectures Center at Syracuse University
 * 03/22/98
 ****************************************************************************/

import mpi.*;

public class ReduceWithArrays {
  public ReduceWithArrays() {}

  public ReduceWithArrays(String[] args) throws Exception {

    final int MAXLEN = 10000;

    int root, i, j, k;

    int outInt[] = new int[MAXLEN];
    double outDouble[] = new double[MAXLEN];
    long outLong[] = new long[MAXLEN];
    float outFloat[] = new float[MAXLEN];
    short outShort[] = new short[MAXLEN];

    int inInt[] = new int[MAXLEN];
    double inDouble[] = new double[MAXLEN];
    long inLong[] = new long[MAXLEN];
    float inFloat[] = new float[MAXLEN];
    short inShort[] = new short[MAXLEN];

    int myself, tasks;

    MPI.Init(args);
    myself = MPI.COMM_WORLD.getRank();
    tasks = MPI.COMM_WORLD.getSize();

    root = tasks / 2;

    for (j = 1; j <= MAXLEN; j *= 10) {
      for (i = 0; i < j; i++) {
        outInt[i] = i;
        outDouble[i] = (double)i;
        outLong[i] = (long)i;
        outFloat[i] = (float)i;
        outShort[i] = (short)i;
      }

      MPI.COMM_WORLD.reduce(outInt, inInt, j, MPI.INT, MPI.SUM, root);
      MPI.COMM_WORLD.reduce(outDouble, inDouble, j, MPI.DOUBLE, MPI.SUM, root);
      MPI.COMM_WORLD.reduce(outShort, inShort, j, MPI.SHORT, MPI.SUM, root);
      MPI.COMM_WORLD.reduce(outLong, inLong, j, MPI.LONG, MPI.SUM, root);
      MPI.COMM_WORLD.reduce(outFloat, inFloat, j, MPI.FLOAT, MPI.SUM, root);

      if (myself == root) {
        for (k = 0; k < j; k++) {
          int valInt = inInt[k];
          double valDouble = inDouble[k];
          float valFloat = inFloat[k];
          long valLong = inLong[k];
          short valShort = inShort[k];
          
          if (valInt != k * tasks) {
            System.out.println(
                "bad answer ("
                    + (valInt)
                    + ") at index "
                    + k
                    + " of "
                    + j
                    + "(should be "
                    + (k * tasks)
                    + ")");
            break;
          }
          if (valDouble != (double)(k * tasks)) {
            System.out.println(
                "bad answer ("
                    + (valDouble)
                    + ") at index "
                    + k
                    + " of "
                    + j
                    + "(should be "
                    + (double)(k * tasks)
                    + ")");
            break;
          }
          if (valLong != (long)(k * tasks)) {
            System.out.println(
                "bad answer ("
                    + (valLong)
                    + ") at index "
                    + k
                    + " of "
                    + j
                    + "(should be "
                    + (long)(k * tasks)
                    + ")");
            break;
          }
          if (valShort != (short)(k * tasks)) {
            System.out.println(
                "bad answer ("
                    + (valShort)
                    + ") at index "
                    + k
                    + " of "
                    + j
                    + "(should be "
                    + (short)(k * tasks)
                    + ")");
            break;
          }
          if (valFloat != (float)(k * tasks)) {
            System.out.println(
                "bad answer ("
                    + (valFloat)
                    + ") at index "
                    + k
                    + " of "
                    + j
                    + "(should be "
                    + (float)(k * tasks)
                    + ")");
            break;
          }
        }
      }
    }

    MPI.COMM_WORLD.barrier();
    if (myself == 0) System.out.println("Reduce TEST COMPLETE");
    MPI.Finalize();
  }

  public static void main(String[] args) throws Exception {
    try {
      ReduceWithArrays c = new ReduceWithArrays(args);
    } catch (Exception e) {
    }
  }
}
