/* Copyright (c) 2001-2022, The Ohio State University. All rights
 * reserved.
 *
 * This file is part of the MVAPICH2-J software package developed by the
 * team members of The Ohio State University's Network-Based Computing
 * Laboratory (NBCL), headed by Professor Dhabaleswar K. (DK) Panda.
 *
 * For detailed copyright and licensing information, please refer to the
 * copyright file COPYRIGHT in the top level MVAPICH2-J directory.
 */

package mpi.ccl;

/****************************************************************************
 *
 * MESSAGE PASSING INTERFACE TEST CASE SUITE
 *
 * Copyright IBM Corp. 1995
 *
 * IBM Corp. hereby grants a non-exclusive license to use, copy, modify, and
 * distribute this software for any purpose and without fee provided that the
 * above copyright notice and the following paragraphs appear in all copies.
 *
 * IBM Corp. makes no representation that the test cases comprising this
 * suite are correct or are an accurate representation of any standard.
 *
 * In no event shall IBM be liable to any party for direct, indirect, special
 * incidental, or consequential damage arising out of the use of this software
 * even if IBM Corp. has been advised of the possibility of such damage.
 *
 * IBM CORP. SPECIFICALLY DISCLAIMS ANY WARRANTIES INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE.  THE SOFTWARE PROVIDED HEREUNDER IS ON AN "AS IS" BASIS AND IBM
 * CORP. HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
 * ENHANCEMENTS, OR MODIFICATIONS.
 ****************************************************************************
 *
 * These test cases reflect an interpretation of the MPI Standard.  They are
 * are, in most cases, unit tests of specific MPI behaviors.  If a user of any
 * test case from this set believes that the MPI Standard requires behavior
 * different than that implied by the test case we would appreciate feedback.
 *
 * Comments may be sent to:
 * Richard Treumann
 * treumann@kgn.ibm.com
 ****************************************************************************
 *
 * MPI-Java version :
 * Sung-Hoon Ko(shko@npac.syr.edu)
 * Northeast Parallel Architectures Center at Syracuse University
 * 03/22/98
 ****************************************************************************/

import mpi.*;

import java.nio.*;

public class ScatterWithArrays {
  public ScatterWithArrays() {}

  public ScatterWithArrays(String[] args) throws Exception {

    final int MAXLEN = 10000;

    int root, i, j, k;
    int myself, tasks;

    MPI.Init(args);
    myself = MPI.COMM_WORLD.getRank();
    tasks = MPI.COMM_WORLD.getSize();

    int outInt[] = new int[MAXLEN * tasks * MPI.INT.getByteSize()];
    double outDouble[] = new double[MAXLEN * tasks * MPI.DOUBLE.getByteSize()];
    long outLong[] = new long[MAXLEN * tasks * MPI.LONG.getByteSize()];
    boolean outBoolean[] = new boolean[MAXLEN * tasks * MPI.BOOLEAN.getByteSize()];
    char outChar[] = new char[MAXLEN * tasks * MPI.CHAR.getByteSize()];
    float outFloat[] = new float[MAXLEN * tasks * MPI.FLOAT.getByteSize()];
    short outShort[] = new short[MAXLEN * tasks * MPI.SHORT.getByteSize()];
    byte outByte[] = new byte[MAXLEN * tasks * MPI.BYTE.getByteSize()]; 

    int inInt[] = new int[MAXLEN * MPI.INT.getByteSize()];
    double inDouble[] = new double[MAXLEN * MPI.DOUBLE.getByteSize()];
    long inLong[] = new long[MAXLEN * MPI.LONG.getByteSize()];
    boolean inBoolean[] = new boolean[MAXLEN * MPI.BOOLEAN.getByteSize()];
    char inChar[] = new char[MAXLEN * MPI.CHAR.getByteSize()];
    float inFloat[] = new float[MAXLEN * MPI.FLOAT.getByteSize()];
    short inShort[] = new short[MAXLEN * MPI.SHORT.getByteSize()];
    byte inByte[] = new byte[MAXLEN * MPI.BYTE.getByteSize()]; 

    for (j = 1, root = 0; j <= MAXLEN; j *= 10, root = (root + 1) % tasks) {
      if (myself == root) {
        for (i = 0; i < j * tasks; i++) {
          outInt[i] = i;
          outDouble[i] = (double)i;
          outFloat[i] = (float)i;
          outShort[i] = (short)i;
          outLong[i] = (long)i;
          outByte[i] = (byte)i;
          outChar[i] = 'a';
        }
      }

      MPI.COMM_WORLD.scatter(outInt, j, MPI.INT, inInt, j, MPI.INT, root);
      MPI.COMM_WORLD.scatter(outLong, j, MPI.LONG, inLong, j, MPI.LONG, root);
      MPI.COMM_WORLD.scatter(outShort, j, MPI.SHORT, inShort, j, MPI.SHORT, root);
      MPI.COMM_WORLD.scatter(outByte, j, MPI.BYTE, inByte, j, MPI.BYTE, root);
      MPI.COMM_WORLD.scatter(outChar, j, MPI.CHAR, inChar, j, MPI.CHAR, root);
      MPI.COMM_WORLD.scatter(outFloat, j, MPI.FLOAT, inFloat, j, MPI.FLOAT, root);
      MPI.COMM_WORLD.scatter(outDouble, j, MPI.DOUBLE, inDouble, j, MPI.DOUBLE, root);

      for (k = 0; k < j; k++) {
        int valInt = inInt[k];
        short valShort = inShort[k];
        float valFloat = inFloat[k];
        char valChar = inChar[k];
        long valLong = inLong[k];
        double valDouble = inDouble[k];
        byte valByte = inByte[k];

        if (valInt != k + myself * j) {
          System.out.println(
              "task "
                  + myself
                  + ":"
                  + "bad answer ("
                  + (valInt)
                  + ") at index "
                  + k
                  + " of "
                  + j
                  + "(should be "
                  + (k + myself * j)
                  + ")");
          break;
        }
        if (valChar != 'a') {
          System.out.println(
              "task "
                  + myself
                  + ":"
                  + "bad answer ("
                  + (valChar)
                  + ") at index "
                  + k
                  + " of "
                  + j
                  + "(should be "
                  + 'a'
                  + ")");
          break;
        }
        if (valByte != (byte)(k + myself * j)) {
          System.out.println(
              "task "
                  + myself
                  + ":"
                  + "bad answer ("
                  + (valByte)
                  + ") at index "
                  + k
                  + " of "
                  + j
                  + "(should be "
                  + (byte)(k + myself * j)
                  + ")");
          break;
        }
        if (valDouble != (double)(k + myself * j)) {
          System.out.println(
              "task "
                  + myself
                  + ":"
                  + "bad answer ("
                  + (valDouble)
                  + ") at index "
                  + k
                  + " of "
                  + j
                  + "(should be "
                  + (double)(k + myself * j)
                  + ")");
          break;
        }
        if (valLong != (long)(k + myself * j)) {
          System.out.println(
              "task "
                  + myself
                  + ":"
                  + "bad answer ("
                  + (valLong)
                  + ") at index "
                  + k
                  + " of "
                  + j
                  + "(should be "
                  + (long)(k + myself * j)
                  + ")");
          break;
        }
        if (valFloat != (float)(k + myself * j)) {
          System.out.println(
              "task "
                  + myself
                  + ":"
                  + "bad answer ("
                  + (valFloat)
                  + ") at index "
                  + k
                  + " of "
                  + j
                  + "(should be "
                  + (float)(k + myself * j)
                  + ")");
          break;
        }
        if (valShort != (short)(k + myself * j)) {
          System.out.println(
              "task "
                  + myself
                  + ":"
                  + "bad answer ("
                  + (valShort)
                  + ") at index "
                  + k
                  + " of "
                  + j
                  + "(should be "
                  + (short)(k + myself * j)
                  + ")");
          break;
        }
      }
    }

    MPI.COMM_WORLD.barrier();
    if (myself == 0) System.out.println("ScatterWithArrays TEST COMPLETE");
    MPI.Finalize();
  }

  public static void main(String[] args) throws Exception {
    try {
      ScatterWithArrays c = new ScatterWithArrays(args);
    } catch (Exception e) {
    }
  }
}
