/* Copyright (c) 2001-2022, The Ohio State University. All rights
 * reserved.
 *
 * This file is part of the MVAPICH2-J software package developed by the
 * team members of The Ohio State University's Network-Based Computing
 * Laboratory (NBCL), headed by Professor Dhabaleswar K. (DK) Panda.
 *
 * For detailed copyright and licensing information, please refer to the
 * copyright file COPYRIGHT in the top level MVAPICH2-J directory.
 */

package mpi.ccl;

/*
MPI-Java version :
Sang Lim (slim@npac.syr.edu)
Northeast Parallel Architectures Center at Syracuse University
12/2/98
*/

import mpi.*;

import java.nio.*;

public class ScattervWithArrays {
  public ScattervWithArrays() {}

  public ScattervWithArrays(String[] args) throws Exception {

    final int MAXLEN = 10;

    int myself, tasks;
    MPI.Init(args);
    myself = MPI.COMM_WORLD.getRank();
    tasks = MPI.COMM_WORLD.getSize();

    if (tasks > 8) {
      if (myself == 0) System.out.println("ScattervWithArrays must run with 8 or less tasks! ");
      MPI.Finalize();
      return;
    }

    int root, i = 0, j, k, stride = 15;

    int outInt[] = new int[stride * tasks];
    double outDouble[] = new double[stride * tasks];
    long outLong[] = new long[stride * tasks];
    float outFloat[] = new float[stride * tasks];
    short outShort[] = new short[stride * tasks];
    char outChar[] = new char[stride * tasks];
    boolean outBoolean[] = new boolean[stride * tasks];
    byte outByte[] = new byte[stride * tasks];

    int inInt[] = new int[MAXLEN];
    double inDouble[] = new double[MAXLEN];
    long inLong[] = new long[MAXLEN];
    float inFloat[] = new float[MAXLEN];
    short inShort[] = new short[MAXLEN];
    char inChar[] = new char[MAXLEN];
    boolean inBoolean[] = new boolean[MAXLEN];
    byte inByte[] = new byte[MAXLEN];

    int dis[] = new int[MAXLEN];
    int scount[] = new int[MAXLEN];
    
    int ans[] = new int[tasks * stride];
    for (i = 0; i < tasks * stride; i++) ans[i] = i; 

    for (i = 0; i < MAXLEN; i++) {
      dis[i] = i * stride;
      scount[i] = 5;

      inInt[i] = 0;
      inDouble[i] = (double)0;
      inBoolean[i] = false;
      inFloat[i] = (float)0;
      inShort[i] = (short)0;
      inLong[i] = (long)0;
      inByte[i] = (byte)0;
      inChar[i] = 'b';
    }

    scount[0] = 10;

    for (i = 0; i < tasks * stride; i++) {
      outInt[i] = i;
      outDouble[i] = (double)i;
      outBoolean[i] = true;
      outFloat[i] = (float)i;
      outShort[i] = (short)i;
      outLong[i] = (long)i;
      outByte[i] = (byte)i;
      outChar[i] = 'a';
    }

    MPI.COMM_WORLD.scatterv(outInt, scount, dis, MPI.INT, inInt, scount[myself], MPI.INT, 0);
    MPI.COMM_WORLD.scatterv(outDouble, scount, dis, MPI.DOUBLE, inDouble, scount[myself], MPI.DOUBLE, 0);
    MPI.COMM_WORLD.scatterv(outFloat, scount, dis, MPI.FLOAT, inFloat, scount[myself], MPI.FLOAT, 0);
    MPI.COMM_WORLD.scatterv(outShort, scount, dis, MPI.SHORT, inShort, scount[myself], MPI.SHORT, 0);
    MPI.COMM_WORLD.scatterv(outLong, scount, dis, MPI.LONG, inLong, scount[myself], MPI.LONG, 0);
    MPI.COMM_WORLD.scatterv(outByte, scount, dis, MPI.BYTE, inByte, scount[myself], MPI.BYTE, 0);
    MPI.COMM_WORLD.scatterv(outChar, scount, dis, MPI.CHAR, inChar, scount[myself], MPI.CHAR, 0);
    MPI.COMM_WORLD.scatterv(outBoolean, scount, dis, MPI.BOOLEAN, inBoolean, scount[myself], MPI.BOOLEAN, 0);

    MPI.COMM_WORLD.barrier();

    for (i = dis[myself]; i < scount[myself]; i++) {
      int valInt = inInt[i];
      double valDouble = inDouble[i];
      long valLong = inLong[i];
      float valFloat = inFloat[i];
      short valShort = inShort[i];
      byte valByte = inByte[i];
      char valChar = inChar[i];
      boolean valBool = inBoolean[i];

      if (valInt != ans[i]) {
        System.out.println("received data : " + valInt + " at [" + i + "] should be : " + ans[i]);
      }
      if (valDouble != (double)ans[i]) {
        System.out.println("received data : " + valDouble + " at [" + i + "] should be : " + (double)ans[i]);
      }
      if (valLong != (long)ans[i]) {
        System.out.println("received data : " + valDouble + " at [" + i + "] should be : " + (long)ans[i]);
      }
      if (valShort != (short)ans[i]) {
        System.out.println("received data : " + valDouble + " at [" + i + "] should be : " + (short)ans[i]);
      }
      if (valFloat != (float)ans[i]) {
        System.out.println("received data : " + valDouble + " at [" + i + "] should be : " + (float)ans[i]);
      }
      if (valByte != (byte)ans[i]) {
        System.out.println("received data : " + valDouble + " at [" + i + "] should be : " + (byte)ans[i]);
      }
      if (valChar != 'a') {
        System.out.println("received data : " + valChar + " at [" + i + "] should be : " + 'a');
      }
      if (valBool != true) {
        System.out.println("received data : " + valChar + " at [" + i + "] should be : true");
      }
    }

    MPI.COMM_WORLD.barrier();
    if (myself == 0) System.out.println("ScattervWithArrays TEST COMPLETE");
    MPI.Finalize();
  }

  public static void main(String[] args) throws Exception {
    try {
      ScattervWithArrays c = new ScattervWithArrays(args);
    } catch (Exception e) {
      e.printStackTrace();
    }
  }
}
