/* Copyright (c) 2001-2022, The Ohio State University. All rights
 * reserved.
 *
 * This file is part of the MVAPICH2-J software package developed by the
 * team members of The Ohio State University's Network-Based Computing
 * Laboratory (NBCL), headed by Professor Dhabaleswar K. (DK) Panda.
 *
 * For detailed copyright and licensing information, please refer to the
 * copyright file COPYRIGHT in the top level MVAPICH2-J directory.
 */

package mpi.ccl;

/*
MPI-Java version :
Sang Lim (slim@npac.syr.edu)
Northeast Parallel Architectures Center at Syracuse University
12/2/98
*/

import mpi.*;

import java.nio.ByteBuffer;

public class ScattervWithByteBuffer {
  public ScattervWithByteBuffer() {}

  public ScattervWithByteBuffer(String[] args) throws Exception {

    final int MAXLEN = 10;

    int myself, tasks;
    MPI.Init(args);
    myself = MPI.COMM_WORLD.getRank();
    tasks = MPI.COMM_WORLD.getSize();

    if (tasks > 8) {
      if (myself == 0) System.out.println("ScattervWithByteBuffer must run with 8 or less tasks! ");
      MPI.Finalize();
      return;
    }

    int root, i = 0, j, k, stride = 15;

    ByteBuffer outByte = ByteBuffer.allocateDirect(stride * tasks * MPI.BYTE.getByteSize());

    ByteBuffer inByte = ByteBuffer.allocateDirect(MAXLEN * MPI.BYTE.getByteSize());

    int dis[] = new int[MAXLEN];
    int scount[] = new int[MAXLEN];
    
    int ans[] = new int[tasks * stride];
    for (i = 0; i < tasks * stride; i++) ans[i] = i; 

    for (i = 0; i < MAXLEN; i++) {
      dis[i] = i * stride;
      scount[i] = 5;
      
      inByte.put((byte)0);
    }

    scount[0] = 10;

    for (i = 0; i < tasks * stride; i++) {
      outByte.put((byte)i);
    }

    MPI.COMM_WORLD.scatterv(outByte, scount, dis, MPI.BYTE, inByte, scount[myself], MPI.BYTE, 0);

    inByte.flip();

    outByte.flip();

    MPI.COMM_WORLD.barrier();
    for (i = dis[myself]; i < scount[myself]; i++) {
      byte valByte = inByte.get();

      if (valByte != (byte)ans[i]) {
        System.out.println("received data : " + valByte + " at [" + i + "] should be : " + (byte)ans[i]);
      }
    }

    MPI.COMM_WORLD.barrier();
    if (myself == 0) System.out.println("ScattervWithByteBuffers TEST COMPLETE");
    MPI.Finalize();
  }

  public static void main(String[] args) throws Exception {
    try {
      ScattervWithByteBuffer c = new ScattervWithByteBuffer(args);
    } catch (Exception e) {
      e.printStackTrace();
    }
  }
}
