/* Copyright (c) 2001-2022, The Ohio State University. All rights
 * reserved.
 *
 * This file is part of the MVAPICH2-J software package developed by the
 * team members of The Ohio State University's Network-Based Computing
 * Laboratory (NBCL), headed by Professor Dhabaleswar K. (DK) Panda.
 *
 * For detailed copyright and licensing information, please refer to the
 * copyright file COPYRIGHT in the top level MVAPICH2-J directory.
 */

package mpi.ccl_ObjSer;

/***************************************************************************
 * Object version :
 * Sang Lim(slim@npac.syr.edu)
 * Northeast Parallel Architectures Center at Syracuse University
 * 11/1/98
 ****************************************************************************/

import mpi.*;

public class bcastO {
  public bcastO() {}

  public bcastO(String[] args) throws Exception {

    final int MAXLEN = 100;

    int root = 0, i, j, k;
    int out[][] = new int[MAXLEN][MAXLEN];
    int myself, tasks;
    double time;

    MPI.Init(args);
    myself = MPI.COMM_WORLD.getRank();
    tasks = MPI.COMM_WORLD.getSize();

    if (myself == root) {
      for (i = 0; i < MAXLEN; i++) for (k = 0; k < MAXLEN; k++) out[i][k] = k;
    } else {
      for (i = 0; i < MAXLEN; i++) for (k = 0; k < MAXLEN; k++) out[i][k] = k + 1;
    }

    MPI.COMM_WORLD.bcast(out, MAXLEN, MPI.OBJECT, root);

    for (k = 0; k < MAXLEN; k++) {
      for (i = 0; i < MAXLEN; i++)
        if (out[i][k] != k) {
          System.out.println(
              "bad answer out[" + i + "][" + k + "] = " + out[i][k] + " should be " + k);
          break;
        }
    }

    MPI.COMM_WORLD.barrier();
    if (myself == 0) System.out.println("BcastO TEST COMPLETE");
    MPI.Finalize();
  }

  public static void main(String[] args) throws Exception {
    try {
      bcastO c = new bcastO(args);
    } catch (Exception e) {
    }
  }
}
