/* Copyright (c) 2001-2022, The Ohio State University. All rights
 * reserved.
 *
 * This file is part of the MVAPICH2-J software package developed by the
 * team members of The Ohio State University's Network-Based Computing
 * Laboratory (NBCL), headed by Professor Dhabaleswar K. (DK) Panda.
 *
 * For detailed copyright and licensing information, please refer to the
 * copyright file COPYRIGHT in the top level MVAPICH2-J directory.
 */

/* Copyright (c) 2001-2022, The Ohio State University. All rights
 * reserved.
 *
 * This file is part of the MVAPICH2-J software package developed by the
 * team members of The Ohio State University's Network-Based Computing
 * Laboratory (NBCL), headed by Professor Dhabaleswar K. (DK) Panda.
 *
 * For detailed copyright and licensing information, please refer to the
 * copyright file COPYRIGHT in the top level MVAPICH2-J directory.
 */

package mpi.comm;

import mpjbuf.*;
import mpi.*;

import java.util.Arrays;

public class CommTest {

  public CommTest() {}

  public CommTest(String[] args) throws Exception {
    int DATA_SIZE = 100;
    MPI.Init(args);
    int size = MPI.COMM_WORLD.getSize();
    int rank = MPI.COMM_WORLD.getRank();

    if (size < 8) {
      if (rank == 0) System.out.println("comm->CommTest runs with eight processes");
      MPI.COMM_WORLD.barrier();
      MPI.Finalize();
      return;
    }

    /* Creating the new communicator */
    byte byteArray[] = new byte[DATA_SIZE];

    for (int i = 0; i < DATA_SIZE; i++) {
      byteArray[i] = 's';
    }

    int[] newIds = {2, 3, 4};
    Intracomm newComm = MPI.COMM_WORLD.create((MPI.COMM_WORLD.getGroup()).incl(newIds));

    int[] newIds1 = {5, 6, 7};
    Intracomm newComm1 = MPI.COMM_WORLD.create((MPI.COMM_WORLD.getGroup()).incl(newIds1));

    int[] newIds2 = {0, 1};
    Intracomm newComm2 = MPI.COMM_WORLD.create((MPI.COMM_WORLD.getGroup()).incl(newIds2));

    int[] newIds3 = {0, 2, 4, 6};
    Intracomm newComm3 = MPI.COMM_WORLD.create((MPI.COMM_WORLD.getGroup()).incl(newIds3));

    if (MPI.COMM_WORLD.getRank() == 0 && newComm3.getRank() == 0) {
      newComm3.send(byteArray, DATA_SIZE, MPI.BYTE, 1, 90);
    } else if (MPI.COMM_WORLD.getRank() == 2 && newComm3.getRank() == 1) {

      byte byteReadArray[] = new byte[DATA_SIZE];

      for (int i = 0; i < DATA_SIZE; i++) {
        byteReadArray[i] = 'x';
      }

      newComm3.recv(byteReadArray, DATA_SIZE, MPI.BYTE, 0, 90);

      if (Arrays.equals(byteArray, byteReadArray)) {
        System.out.println("\n#################" + "\n <<<<PASSED>>>> " + "\n################");
      } else {
        System.out.println("\n#################" + "\n <<<<FAILED>>>> " + "\n################");
      }
    } // end while(true)

    MPI.COMM_WORLD.barrier();

    if (rank == 0) System.out.println("CommTest TEST COMPLETE");

    MPI.Finalize();
  }

  public static void main(String args[]) throws Exception {
    try {
      CommTest a = new CommTest(args);
    } catch (Exception e) {
    }
  }
}
