/* Copyright (c) 2001-2022, The Ohio State University. All rights
 * reserved.
 *
 * This file is part of the MVAPICH2-J software package developed by the
 * team members of The Ohio State University's Network-Based Computing
 * Laboratory (NBCL), headed by Professor Dhabaleswar K. (DK) Panda.
 *
 * For detailed copyright and licensing information, please refer to the
 * copyright file COPYRIGHT in the top level MVAPICH2-J directory.
 */

/* Copyright (c) 2001-2022, The Ohio State University. All rights
 * reserved.
 *
 * This file is part of the MVAPICH2-J software package developed by the
 * team members of The Ohio State University's Network-Based Computing
 * Laboratory (NBCL), headed by Professor Dhabaleswar K. (DK) Panda.
 *
 * For detailed copyright and licensing information, please refer to the
 * copyright file COPYRIGHT in the top level MVAPICH2-J directory.
 */

package mpi.comm;

import mpi.*;

import java.util.Arrays;

public class intercomm_test {

  public intercomm_test() {}

  public intercomm_test(String[] args) throws Exception {

    MPI.Init(args);
    int rank = MPI.COMM_WORLD.getRank();
    Group grp = MPI.COMM_WORLD.getGroup();
    int size = MPI.COMM_WORLD.getSize();

    /* creating first intra-communicator */
    Intracomm comm1 = null;
    int[] incl1 = {0, 2, 4, 6};

    if (rank == 0 || rank == 2 || rank == 4 || rank == 6) {
      Group grp1 = grp.incl(incl1);
      comm1 = MPI.COMM_WORLD.create(grp1);
    }

    /* creating first intra-communicator */
    Intracomm comm2 = null;
    int[] incl2 = {1, 3, 5, 7};

    if (rank == 1 || rank == 3 || rank == 5 || rank == 7) {
      Group grp2 = grp.incl(incl2);
      comm2 = MPI.COMM_WORLD.create(grp2);
    }

    Intercomm icomm = null;

    if (rank == 0 || rank == 2 || rank == 4 || rank == 6) {
      icomm = MPI.COMM_WORLD.createIntercomm(comm1, 1, 3, 56);
    }

    if (rank == 1 || rank == 3 || rank == 5 || rank == 7) {
      icomm = MPI.COMM_WORLD.createIntercomm(comm2, 1, 2, 56);
    }

    int[] testArray1 = new int[10];
    int[] testArray2 = new int[10];

    for (int i = 0; i < testArray1.length; i++) {
      testArray1[i] = i;
      testArray2[i] = 0;
    }

    if (rank == 4) {
      System.out.println(icomm.getRank() + " sending ..");
      icomm.send(testArray1, 10, MPI.INT, 2, 78);
    } else if (rank == 5) {
      System.out.println(icomm.getRank() + " receiving ..");
      icomm.recv(testArray2, 10, MPI.INT, 2, 78);
      if (Arrays.equals(testArray1, testArray2)) {

        System.out.println("****************\n" + "***PASSED*******\n" + "****************");
      }
    }

    if (rank == 0 || rank == 2 || rank == 4 || rank == 6) {
      comm1.free();
    }

    if (rank == 1 || rank == 3 || rank == 5 || rank == 7) comm2.free();

    icomm.free();

    MPI.COMM_WORLD.barrier();

    if (rank == 0) {
      System.out.println("intercomm_test TEST COMPLETE");
    }

    MPI.Finalize();
  }

  public static void main(String args[]) throws Exception {
    try {
      intercomm_test a = new intercomm_test(args);
    } catch (Exception e) {
    }
  }
}
