/* Copyright (c) 2001-2022, The Ohio State University. All rights
 * reserved.
 *
 * This file is part of the MVAPICH2-J software package developed by the
 * team members of The Ohio State University's Network-Based Computing
 * Laboratory (NBCL), headed by Professor Dhabaleswar K. (DK) Panda.
 *
 * For detailed copyright and licensing information, please refer to the
 * copyright file COPYRIGHT in the top level MVAPICH2-J directory.
 */

package mpi.dtyp;

import mpi.*;

import java.util.Arrays;

/**
 * This example application, is to check and demonstrat the functonality of contiguous datatypes.
 * I'm writing an MPJ program by coverting a C program by Forest Hoffman at following URL
 * http://www.linux-mag.com/2003-04/extreme_01.html
 */
public class Contiguous {
  public static int NUM_PARAMS = 12;

  public Contiguous() {}

  public Contiguous(String[] args) throws Exception {

    MPI.Init(args);
    int i, rank, size;

    double[] params = new double[NUM_PARAMS];
    double[] rparams = new double[NUM_PARAMS];

    rank = MPI.COMM_WORLD.getRank();
    size = MPI.COMM_WORLD.getSize();
    Datatype parameterType;

    /* only the first process reads the parameters */
    if (rank == 0) {
      for (i = 0; i < NUM_PARAMS; i++) {
        params[i] = (double) (i * i);
      }
    }

    parameterType = Datatype.createContiguous(NUM_PARAMS, MPI.DOUBLE);
    parameterType.commit(); // this is doing nothing at the moment.

    if (rank == 1) {
      MPI.COMM_WORLD.recv(rparams, 1, parameterType, 0, 10);
      System.out.println("recv completed");

      for (int j = 0; j < rparams.length; j++) {
        System.out.print("rparams[" + j + "]=" + rparams[j] + "\t");
      }

    } else if (rank == 0) {
      MPI.COMM_WORLD.send(params, 1, parameterType, 1, 10);
      System.out.println("Send Completed");
    }

    MPI.COMM_WORLD.barrier();
    MPI.Finalize();
  }

  public static void main(String args[]) throws Exception {
    try {
      Contiguous c = new Contiguous(args);
    } catch (Exception e) {
    }
  }
}
