/* Copyright (c) 2001-2022, The Ohio State University. All rights
 * reserved.
 *
 * This file is part of the MVAPICH2-J software package developed by the
 * team members of The Ohio State University's Network-Based Computing
 * Laboratory (NBCL), headed by Professor Dhabaleswar K. (DK) Panda.
 *
 * For detailed copyright and licensing information, please refer to the
 * copyright file COPYRIGHT in the top level MVAPICH2-J directory.
 */

package mpi.dtyp;

import mpi.*;

import java.util.Arrays;

/**
 * This example application, is to check and demonstrate the functonality of contiguous datatypes.
 * I'm writing an MPJ program by coverting a C program by Forest Hoffman at following URL
 * http://www.linux-mag.com/2003-04/extreme_01.html
 */
public class Indexed {

  public Indexed() {}

  public Indexed(String[] args) throws Exception {
    int DIM = 8;
    MPI.Init(args);
    int i, j, k, rank, size, dest, src, tag;

    int[] blklen = new int[DIM];
    int[] displ = new int[DIM];
    double[] data = new double[(DIM * DIM)];
    rank = MPI.COMM_WORLD.getRank();
    size = MPI.COMM_WORLD.getSize();
    Datatype parameterType;

    /* only the first process reads the parameters */
    for (i = 0; i < DIM; i++) {
      blklen[i] = DIM - i;
      displ[i] = (i * DIM) + i;
    }
    src = 0;
    dest = 1;
    tag = 10;
    Datatype indexedDT = Datatype.createIndexed(blklen, displ, MPI.DOUBLE);
    indexedDT.commit();

    if (rank == 1) {

      MPI.COMM_WORLD.recv(data, 1, indexedDT, src, tag);
      System.out.println("recv completed");

      for (j = 0; j < 64; j++) {
        System.out.print("data[" + j + "]=" + data[j] + " ");
      }

    } else if (rank == 0) {
      for (i = 0; i < (DIM * DIM); i++) data[i] = (double) (i + 1);

      MPI.COMM_WORLD.send(data, 1, indexedDT, dest, tag);
      System.out.println("Send Completed");
    }
    try {
      Thread.currentThread().sleep(1000);
    } catch (Exception e) {
    }
    MPI.COMM_WORLD.barrier();
    MPI.Finalize();
  }

  public static void main(String args[]) throws Exception {
    try {
      Indexed c = new Indexed(args);
    } catch (Exception e) {
    }
  }
}
