/* Copyright (c) 2001-2022, The Ohio State University. All rights
 * reserved.
 *
 * This file is part of the MVAPICH2-J software package developed by the
 * team members of The Ohio State University's Network-Based Computing
 * Laboratory (NBCL), headed by Professor Dhabaleswar K. (DK) Panda.
 *
 * For detailed copyright and licensing information, please refer to the
 * copyright file COPYRIGHT in the top level MVAPICH2-J directory.
 */

package mpi.dtyp;

// This test is from MPICH test suite.

/* The original version of this was sent by
ung-Hoon Ko
I've modified it to fit the automated tests requirements
*/
/****************************************************************************
 *
 * MPI-Java version :
 * Sung-Hoon Ko(shko@npac.syr.edu)
 * Northeast Parallel Architectures Center at Syracuse University
 * 03/22/98
 ****************************************************************************/

import mpi.*;

public class hvec {
  public hvec() {}

  public hvec(String[] args) throws Exception {
    int my_rank;

    MPI.Init(args);
    my_rank = MPI.COMM_WORLD.getRank();

    Datatype messtyp, messtyp2;
    int root = 0;
    int i, istat, big_offset;
    final int DL = 32;

    int dar[] = new int[DL];

    for (i = 0; i < DL; i++) dar[i] = my_rank;

    int count = 2;
    int bllen = 3;
    int gap = 1;
    int str = bllen + gap;

    messtyp = Datatype.createVector(count, bllen, str, MPI.INT);
    messtyp.commit();
    MPI.COMM_WORLD.bcast(dar, 1, messtyp, root);
    if (my_rank == 1) {
      System.out.println("  0 = Sent, 1 = Not Sent");
      System.out.println("  Vector Type with Gap : ");
    }
    if (my_rank == 1) {
      for (i = 0; i < DL; i++) System.out.print(dar[i] + " ");
      System.out.println();
      System.out.println();
    }

    for (big_offset = -1; big_offset <= 2; big_offset++) {
      if (my_rank == 1)
        System.out.println(
            " Three of above vector types combined, with offset = " + big_offset + " ints");

      for (i = 0; i < DL; i++) dar[i] = my_rank;

      count = 3;
      int ext = messtyp.extent();
      messtyp2 = Datatype.createHVector(count, 1, ext + big_offset, messtyp);
      messtyp2.commit();
      MPI.COMM_WORLD.bcast(dar, 1, messtyp2, root);

      MPI.COMM_WORLD.barrier();

      if (my_rank == 1) {
        for (i = 0; i < DL; i++) System.out.print(dar[i] + " ");
        System.out.println();
        System.out.println();
      }
    }

    if (my_rank == 1) {
      System.out.println("hvec TEST Completed");
    }

    MPI.Finalize();
  }

  public static void main(String[] args) throws MPIException {
    try {
      hvec c = new hvec(args);
    } catch (Exception e) {
    }
  }
}
