/* Copyright (c) 2001-2022, The Ohio State University. All rights
 * reserved.
 *
 * This file is part of the MVAPICH2-J software package developed by the
 * team members of The Ohio State University's Network-Based Computing
 * Laboratory (NBCL), headed by Professor Dhabaleswar K. (DK) Panda.
 *
 * For detailed copyright and licensing information, please refer to the
 * copyright file COPYRIGHT in the top level MVAPICH2-J directory.
 */

package mpi.group;

import mpi.*;

public class Group_com {

  public Group_com() {}

  public Group_com(String[] args) throws Exception {

    MPI.Init(args);
    int res = -1;
    Group grp = MPI.COMM_WORLD.getGroup();
    int me = MPI.COMM_WORLD.getRank();
    int tasks = MPI.COMM_WORLD.getSize();

    if (tasks < 8) {
      if (me == 0) System.out.println("group->Group_com: MUST RUN with 8 tasks");
      MPI.COMM_WORLD.barrier();
      MPI.Finalize();
      return;
    }

    int[] incl1 = {4, 0, 5, 6};
    int[] incl2 = {6, 7, 4};
    int[] incl3 = {6, 0, 5, 4};

    Group grp0 = grp.incl(incl1);
    Group grp1 = grp.incl(incl1);
    Group grp2 = grp.incl(incl2);
    Group grp3 = MPI.GROUP_EMPTY;
    Group grp4 = grp.incl(incl3);

    res = Group.compare(grp0, grp1);
    if (res != MPI.IDENT) {
      System.out.println("ERROR(1): grp1{4 0 5 6} and grp0{4 0 5 6} " + "are IDENTICAL");
    }

    res = Group.compare(grp1, grp2);

    if (res != MPI.UNEQUAL) {
      System.out.println(me + ": res <" + res + ">, " + " MPI.UNEQUAL <" + MPI.UNEQUAL + ">");
      System.out.println(me + "ERROR(2): grp1{4 0 5 6} and grp2{6 7 4} " + "are UNEQUAL");
    }

    res = Group.compare(grp4, grp1);
    if (res != MPI.SIMILAR) {
      System.out.println("ERROR(3): grp1{4 0 5 6} and grp4{6 0 5 4} " + "are SIMILAR");
    }

    res = Group.compare(grp3, grp1);
    if (res != MPI.UNEQUAL) {
      System.out.println("ERROR(4): grp1{4 0 5 6} and grp3{  " + "are UNEQUAL");
    }

    res = Group.compare(grp3, grp3);

    if (res != MPI.IDENT) {
      System.out.println("ERROR(5): grp3{ } and grp3{ } are IDENTICAL");
    }

    MPI.COMM_WORLD.barrier();

    if (me == 0) System.out.println("Group_com TEST COMPLETED");

    MPI.Finalize();
  }

  public static void main(String args[]) throws Exception {
    try {
      Group_com a = new Group_com(args);
    } catch (Exception e) {
    }
  }
}
