/* Copyright (c) 2001-2022, The Ohio State University. All rights
 * reserved.
 *
 * This file is part of the MVAPICH2-J software package developed by the
 * team members of The Ohio State University's Network-Based Computing
 * Laboratory (NBCL), headed by Professor Dhabaleswar K. (DK) Panda.
 *
 * For detailed copyright and licensing information, please refer to the
 * copyright file COPYRIGHT in the top level MVAPICH2-J directory.
 */

package mpi.group;

import mpjbuf.*;
import mpi.*;

import java.util.Arrays;

public class Group_incl {

  public Group_incl() {}

  public Group_incl(String[] args) throws Exception {
    MPI.Init(args);
    Group grp = MPI.COMM_WORLD.getGroup();
    int me = grp.getRank();
    int tasks = grp.getSize();

    if (tasks < 8) {
      if (me == 0) System.out.println("group->Group_incl: MUST RUN with 8 tasks");
      MPI.COMM_WORLD.barrier();
      MPI.Finalize();
      return;
    }

    int[] incl1 = {4, 0, 5};
    Group ngrp = grp.incl(incl1);
    int new_me = ngrp.getRank();

    if (me == 0) {
      if (new_me != 0 && new_me != 1 && new_me != 2) {
        System.out.println("Error(1): ");
      }
    } else if (me == 1) {
      if (new_me != -1) {
        System.out.println(
            "Error(2): This process <"
                + me
                + ">is not in the new "
                + "group and its rank should be -1, instead it is <"
                + new_me
                + ">");
      }
    } else if (me == 2) {
      if (new_me != -1) {
        System.out.println(
            "Error(3): This process <"
                + me
                + ">is not in the new "
                + "group and its rank should be -1, instead it is <"
                + new_me
                + ">");
      }
    } else if (me == 3) {
      if (new_me != -1) {
        System.out.println(
            "Error(4): This process <"
                + me
                + ">is not in the new "
                + "group and its rank should be -1, instead it is <"
                + new_me
                + ">");
      }
    } else if (me == 4) {
      if (new_me != 0 && new_me != 1 && new_me != 2) {
        System.out.println("Error(5): ");
      }
    } else if (me == 5) {
      if (new_me != 0 && new_me != 1 && new_me != 2) {
        System.out.println("Error(6): ");
      }
    } else if (me == 6) {
      if (new_me != -1) {
        System.out.println(
            "Error(7): This process <"
                + me
                + ">is not in the new "
                + "group and its rank should be -1, instead it is <"
                + new_me
                + ">");
      }
    } else if (me == 7) {
      if (new_me != -1) {
        System.out.println(
            "Error(8): This process <"
                + me
                + ">is not in the new "
                + "group and its rank should be -1, instead it is <"
                + new_me
                + ">");
      }
    }
    MPI.COMM_WORLD.barrier();
    if (me == 0) System.out.println("Group_incl TEST COMPLETED");

    MPI.Finalize();
  }

  public static void main(String args[]) throws Exception {
    try {
      Group_incl a = new Group_incl(args);
    } catch (Exception e) {
    }
  }
}
