/* Copyright (c) 2001-2022, The Ohio State University. All rights
 * reserved.
 *
 * This file is part of the MVAPICH2-J software package developed by the
 * team members of The Ohio State University's Network-Based Computing
 * Laboratory (NBCL), headed by Professor Dhabaleswar K. (DK) Panda.
 *
 * For detailed copyright and licensing information, please refer to the
 * copyright file COPYRIGHT in the top level MVAPICH2-J directory.
 */

package mpi.group;

import mpjbuf.*;
import mpi.*;

import java.util.Arrays;

public class Group_intersect {

  public Group_intersect() {}

  public Group_intersect(String[] args) throws Exception {

    MPI.Init(args);
    Group grp = MPI.COMM_WORLD.getGroup();
    int me = grp.getRank();
    int tasks = grp.getSize();

    if (tasks < 8) {
      if (me == 0) System.out.println("group->Group_intersect: MUST RUN with 8 tasks");
      MPI.COMM_WORLD.barrier();
      MPI.Finalize();
      return;
    }

    int[] incl1 = {4, 0, 5, 6};
    int[] incl2 = {6, 7, 4};
    Group grp1 = grp.incl(incl1);
    Group grp2 = grp.incl(incl2);

    if (grp1.getSize() != 4) {
      System.out.println("Error(1): ?|?|?|? ");
    } else if (grp2.getSize() != 3) {
      System.out.println("Error(2): ?|?|?|? ");
    }

    Group grp3 = Group.intersection(grp1, grp2);

    if (grp3.getSize() != 2) {
      System.out.println("Error(3): ");
    }

    MPI.COMM_WORLD.barrier();

    if (me == 0) System.out.println("Group_intersect TEST COMPLETED");

    MPI.Finalize();
  }

  public static void main(String args[]) throws Exception {
    try {
      Group_intersect a = new Group_intersect(args);
    } catch (Exception e) {
    }
  }
}
