/* Copyright (c) 2001-2022, The Ohio State University. All rights
 * reserved.
 *
 * This file is part of the MVAPICH2-J software package developed by the
 * team members of The Ohio State University's Network-Based Computing
 * Laboratory (NBCL), headed by Professor Dhabaleswar K. (DK) Panda.
 *
 * For detailed copyright and licensing information, please refer to the
 * copyright file COPYRIGHT in the top level MVAPICH2-J directory.
 */

package mpi.group;

import mpjbuf.*;
import mpi.*;

import java.util.Arrays;

public class Group_self {

  public Group_self() {}

  public Group_self(String[] args) throws Exception {
    MPI.Init(args);
    Group grp = MPI.COMM_WORLD.getGroup();
    int me = grp.getRank();
    int tasks = grp.getSize();
    int data[] = new int[10];
    int rdata[] = new int[10];
    int src_dest = 0;
    int tag = 10;

    for (int i = 0; i < data.length; i++) {
      data[i] = i;
    }

    mpi.Request req = MPI.COMM_SELF.iSend(data, 10, MPI.INT, src_dest, tag);
    MPI.COMM_SELF.recv(rdata, 10, MPI.INT, src_dest, tag);
    req.waitStatus();

    for (int i = 0; i < rdata.length; i++) {
      if (rdata[i] != i) {
        System.out.println(
            "Error at index " + i + ": it is " + data[i] + "while " + "we are expecting " + i);
        break;
      }
    }

    MPI.COMM_WORLD.barrier();

    if (me == 0) System.out.println("Group_self TEST COMPLETED");

    MPI.Finalize();
  }

  public static void main(String args[]) throws Exception {
    try {
      Group_self a = new Group_self(args);
    } catch (Exception e) {
    }
  }
}
