/* Copyright (c) 2001-2022, The Ohio State University. All rights
 * reserved.
 *
 * This file is part of the MVAPICH2-J software package developed by the
 * team members of The Ohio State University's Network-Based Computing
 * Laboratory (NBCL), headed by Professor Dhabaleswar K. (DK) Panda.
 *
 * For detailed copyright and licensing information, please refer to the
 * copyright file COPYRIGHT in the top level MVAPICH2-J directory.
 */

package mpi.group;

import mpjbuf.*;
import mpi.*;

import java.util.Arrays;

public class Group_tranks {

  public Group_tranks() {}

  public Group_tranks(String[] args) throws Exception {

    MPI.Init(args);
    int me = MPI.COMM_WORLD.getRank();
    int tasks = MPI.COMM_WORLD.getSize();

    if (tasks < 8) {
      if (me == 0) System.out.println("group->Group_tranks: MUST RUN 8 TASKS");

      MPI.COMM_WORLD.barrier();
      MPI.Finalize();
      return;
    }
    Group grp = MPI.COMM_WORLD.getGroup();
    int[] incl1 = {7, 6, 5, 4};
    int[] ranks1 = {0, 1, 2, 3};
    Group grp1 = grp.incl(incl1);
    int[] ranks2 = Group.translateRanks(grp1, ranks1, grp);

    if (java.util.Arrays.equals(incl1, ranks2)) {
      if (me == 0) System.out.println("Group_tranks TEST COMPLETED");
    } else {
      System.out.println("Error(1): Test failed");
      for (int i = 0; i < ranks1.length; i++) {
        System.out.println(" expected [" + i + "]=" + incl1[i]);
        System.out.println(" actual   [" + i + "]=" + ranks2[i]);
      }
    }

    MPI.COMM_WORLD.barrier();

    MPI.Finalize();
  }

  public static void main(String args[]) throws Exception {
    try {
      Group_tranks a = new Group_tranks(args);
    } catch (Exception e) {
    }
  }
}
