/* Copyright (c) 2001-2022, The Ohio State University. All rights
 * reserved.
 *
 * This file is part of the MVAPICH2-J software package developed by the
 * team members of The Ohio State University's Network-Based Computing
 * Laboratory (NBCL), headed by Professor Dhabaleswar K. (DK) Panda.
 *
 * For detailed copyright and licensing information, please refer to the
 * copyright file COPYRIGHT in the top level MVAPICH2-J directory.
 */

package mpi.group;

import mpjbuf.*;
import mpi.*;

import java.util.Arrays;

public class Group_union {

  public Group_union() {}

  public Group_union(String[] args) throws Exception {
    MPI.Init(args);
    int me = MPI.COMM_WORLD.getRank();
    int size = MPI.COMM_WORLD.getSize();

    if (size < 8) {
      if (me == 0) {
        System.out.println("group->Group_union: MUST HAVE 8 TASKS");
      }
      MPI.COMM_WORLD.barrier();
      MPI.Finalize();
      return;
    }

    Group grp = MPI.COMM_WORLD.getGroup();
    me = grp.getRank();
    int[] incl1 = {4, 0, 5};
    int[] incl2 = {6, 7, 4};
    Group grp1 = grp.incl(incl1);
    Group grp2 = grp.incl(incl2);
    Group grp3 = null;
    grp3 = Group.union(grp1, grp2);

    if (grp3.getSize() != 5) {
      System.out.println("Error(1): the new group should've five processes " + "in it ... ");
    }

    MPI.COMM_WORLD.barrier();

    if (me == 0) System.out.println("Group_union TEST COMPLETED");

    MPI.Finalize();
  }

  public static void main(String args[]) throws Exception {
    try {
      Group_union a = new Group_union(args);
    } catch (Exception e) {
    }
  }
}
