/* Copyright (c) 2001-2022, The Ohio State University. All rights
 * reserved.
 *
 * This file is part of the MVAPICH2-J software package developed by the
 * team members of The Ohio State University's Network-Based Computing
 * Laboratory (NBCL), headed by Professor Dhabaleswar K. (DK) Panda.
 *
 * For detailed copyright and licensing information, please refer to the
 * copyright file COPYRIGHT in the top level MVAPICH2-J directory.
 */

/*
 * Compile :- javac -classpath $HOME/aamirmpj/aamir-niompj/mpjdev/src:$MV2J_HOME/src/:$HOME/install/junit3.8.1/junit.jar ContiguousTest.java
 * Execute :- java -classpath $HOME/aamirmpj/aamir-niompj/mpjdev/src:$MV2J_HOME/src/:$HOME/install/junit3.8.1/junit.jar mpi.ContiguousTest  */

import junit.framework.*;
import mpjbuf.Buffer;

/**
 * This example application, is to check and demonstrat the functonality of contiguous datatypes.
 * I'm writing an MPJ program by coverting a C program by Forest Hoffman at following URL
 * http://www.linux-mag.com/2003-04/extreme_01.html
 */
public class ContiguousTest extends TestCase {

  public static int NUM_PARAMS = 12;

  public ContiguousTest(String testName) {
    super(testName);
  }

  public static Test suite() {

    TestSuite suite = new TestSuite();

    suite.addTest(
        new ContiguousTest("simple add") {
          public void runTest() {
            try {
              testSimplePackUnpack();

            } catch (Exception e) {
              e.printStackTrace();
              fail("exception throw while calling testSimplePackUnpack method: " + e.getMessage());
            }
          }
        });
    return suite;
  }

  public static void main(String args[]) {
    junit.textui.TestRunner.run(suite());
  }

  public void testSimplePackUnpack() throws Exception {

    int i, rank, size;

    double[] params = new double[NUM_PARAMS];
    double[] rparams = new double[NUM_PARAMS];

    Datatype parameterType;

    /* only the first process reads the parameters */
    for (i = 0; i < NUM_PARAMS; i++) {
      params[i] = (double) (i * i);
    }

    parameterType = Datatype.Contiguous(NUM_PARAMS, MPI.DOUBLE);
    parameterType.Commit(); // this is doing nothing at the moment.

    mpjbuf.Buffer buf = parameterType.createWriteBuffer(1);
    mpi.Packer packer = parameterType.getPacker();

    packer.pack(buf, params, 0, 1);

    buf.commit();

    // Check it got packed OK.

    int numEls;
    numEls = buf.getSectionHeader(Buffer.DOUBLE);

    double[] packed = new double[NUM_PARAMS];
    buf.read(packed, 0, numEls);

    for (i = 0; i < NUM_PARAMS; i++) {
      Assert.assertTrue(packed[i] == (double) (i * i));
    }
  }
}
