/* Copyright (c) 2001-2022, The Ohio State University. All rights
 * reserved.
 *
 * This file is part of the MVAPICH2-J software package developed by the
 * team members of The Ohio State University's Network-Based Computing
 * Laboratory (NBCL), headed by Professor Dhabaleswar K. (DK) Panda.
 *
 * For detailed copyright and licensing information, please refer to the
 * copyright file COPYRIGHT in the top level MVAPICH2-J directory.
 */

package mpi.perf;

import mpi.*;

import java.nio.ByteBuffer;
import java.io.*;

public class Bandwidth {
  static double tripsPerMS = 1000000;
  static int dummy;

  public Bandwidth() {}

  public Bandwidth(String[] args) throws Exception {
    int WARM_UP = 10000;
    int REPEAT = 20000;
    long[] delays = new long[REPEAT];
    long[] latency = new long[REPEAT];
    long start = 0L, stop = 0L, init = 0L;
    MPI.Init(args);
    int me = MPI.COMM_WORLD.getRank();
    System.out.println("Name =" + MPI.getProcessorName());
    byte byteArray[] = null;
    int j = 1, i = 0;
    int LOG2N_MAX = 1000000, log2nbyte = 0, padding = 0;
    long timed = 0L;
    byteArray = new byte[16777216];
    PrintStream out = null;
    FileOutputStream fos = null;

    for (i = 0; i < byteArray.length; i++) {
      byteArray[i] = 's';
    }

    /* Can do a warm-up here */
    if (me == 1) {
      fos = new FileOutputStream("bw_delays.out");
      out = new PrintStream(fos);
    } else {
      fos = new FileOutputStream("bw.out");
      out = new PrintStream(fos);
    }

    for (i = 0; i < 100; i++) {
      if (me == 0) {
        MPI.COMM_WORLD.recv(byteArray, 1, MPI.BYTE, 1, 998);
        MPI.COMM_WORLD.send(byteArray, 1, MPI.BYTE, 1, 998);
      } else if (me == 1) {
        MPI.COMM_WORLD.send(byteArray, 1, MPI.BYTE, 0, 998);
        MPI.COMM_WORLD.recv(byteArray, 1, MPI.BYTE, 0, 998);
      }
    }

    /* Logrithmic Loop */
    for (log2nbyte = 0; log2nbyte <= LOG2N_MAX && j < 16 * 1024 * 1024; ++log2nbyte) {

      j = (1 << log2nbyte);

      /* Latency Calculation Loop */
      for (i = 0; i < REPEAT; i++) {
        if (me == 0) {
          init = System.nanoTime();
          MPI.COMM_WORLD.send(byteArray, j, MPI.BYTE, 1, 998);
          MPI.COMM_WORLD.recv(byteArray, j, MPI.BYTE, 1, 998);
          latency[i] = ((System.nanoTime() - init) / 1000);
        } else if (me == 1) {
          MPI.COMM_WORLD.recv(byteArray, j, MPI.BYTE, 0, 998);
          start = System.nanoTime();
          delays[i] = (System.nanoTime() - start) / 1000;
          MPI.COMM_WORLD.send(byteArray, j, MPI.BYTE, 0, 998);
        }
      }

      if (me == 0) {
        for (int k = WARM_UP; k < REPEAT; k++) {
          out.println(latency[k] + "   ");
        }
      } else {
        for (i = WARM_UP; i < REPEAT; i++) {
          out.println(delays[i] + "   ");
        }
      }

      MPI.COMM_WORLD.barrier();
    } // end logrithmic loop

    fos.close();
    out.close();

    MPI.Finalize();
  } // end args constructor.

  static void myDelay(int us) {

    int trips = (int) (tripsPerMS * us);
    long start = System.nanoTime();

    for (int i = 0; i < trips; i++) {
      dummy++;
    }

    long actualDelay = System.nanoTime() - start;

    if (actualDelay > 0) {
      long newTripsPerMS = (trips * 1000) / actualDelay;

      if (newTripsPerMS > 0) {
        tripsPerMS = newTripsPerMS;
      }
    }
  }

  public static void main(String[] args) throws Exception {
    Bandwidth bw = new Bandwidth(args);
  }
} // end class.
