/* Copyright (c) 2001-2022, The Ohio State University. All rights
 * reserved.
 *
 * This file is part of the MVAPICH2-J software package developed by the
 * team members of The Ohio State University's Network-Based Computing
 * Laboratory (NBCL), headed by Professor Dhabaleswar K. (DK) Panda.
 *
 * For detailed copyright and licensing information, please refer to the
 * copyright file COPYRIGHT in the top level MVAPICH2-J directory.
 */

package mpi.perf;

import mpi.*;

import java.nio.ByteBuffer;
import java.io.*;

public class SimpleBandwidth {

  public SimpleBandwidth() {}

  public SimpleBandwidth(String[] args) throws Exception {

    int WARM_UP = 10000;
    int REPEAT = 10000;
    double latency = 0L;
    long start = 0L, stop = 0L, init = 0L;

    MPI.Init(args);

    int me = MPI.COMM_WORLD.getRank();
    System.out.println("Name =" + MPI.getProcessorName());

    byte byteArray[] = null;
    int j = 1, i = 0;
    int LOG2N_MAX = 1000000, log2nbyte = 0, padding = 0;
    long timed = 0L;
    byteArray = new byte[16 * 1024 * 1024];

    for (i = 0; i < byteArray.length; i++) {
      byteArray[i] = 's';
    }

    for (i = 0; i < WARM_UP; i++) {
      if (me == 0) {
        MPI.COMM_WORLD.recv(byteArray, 1, MPI.BYTE, 1, 998);
        MPI.COMM_WORLD.send(byteArray, 1, MPI.BYTE, 1, 998);
      } else if (me == 1) {
        MPI.COMM_WORLD.send(byteArray, 1, MPI.BYTE, 0, 998);
        MPI.COMM_WORLD.recv(byteArray, 1, MPI.BYTE, 0, 998);
      }
    }

    /* Logrithmic Loop */
    for (log2nbyte = 0; log2nbyte <= LOG2N_MAX && j < 16 * 1024 * 1024; ++log2nbyte) {

      j = (1 << log2nbyte);

      init = System.nanoTime();

      /* Latency Calculation Loop */
      for (i = 0; i < REPEAT; i++) {

        if (me == 0) {
          MPI.COMM_WORLD.send(byteArray, j, MPI.BYTE, 1, 998);
          MPI.COMM_WORLD.recv(byteArray, j, MPI.BYTE, 1, 998);
        } else if (me == 1) {
          MPI.COMM_WORLD.recv(byteArray, j, MPI.BYTE, 0, 998);
          MPI.COMM_WORLD.send(byteArray, j, MPI.BYTE, 0, 998);
        }
      }

      latency = (System.nanoTime() - init) / (2.0 * REPEAT * 1000);

      if (me == 1) {
        System.out.println(j + "\t\t" + (latency) + "\t\t" + ((8 * j) / (1000 * (latency))));
      }

      MPI.COMM_WORLD.barrier();
    } // end logrithmic loop

    MPI.Finalize();
  } // end args constructor.

  public static void main(String[] args) throws Exception {
    SimpleBandwidth bw = new SimpleBandwidth(args);
  }
} // end class.
