/* Copyright (c) 2001-2022, The Ohio State University. All rights
 * reserved.
 *
 * This file is part of the MVAPICH2-J software package developed by the
 * team members of The Ohio State University's Network-Based Computing
 * Laboratory (NBCL), headed by Professor Dhabaleswar K. (DK) Panda.
 *
 * For detailed copyright and licensing information, please refer to the
 * copyright file COPYRIGHT in the top level MVAPICH2-J directory.
 */

package mpi.pt2pt;

import mpi.*;

import java.util.Arrays;

public class BreakANYSRC {
  public BreakANYSRC() {}

  public BreakANYSRC(String args[]) throws Exception {

    MPI.Init(args);
    int me = MPI.COMM_WORLD.getRank();
    int DATA_SIZE = 10;
    int intArray[] = new int[DATA_SIZE];
    int intReadArray[] = new int[DATA_SIZE];
    Status st;
    for (int i = 0; i < DATA_SIZE; i++) {
      intArray[i] = i;
      intReadArray[i] = 0;
    }

    if (me == 0) {

      /* test #1 */
      System.out.println(" sending ");
      Request req = MPI.COMM_WORLD.iSend(intArray, DATA_SIZE, MPI.INT, 0, 1);
      System.out.println(" receiving");
      MPI.COMM_WORLD.recv(intReadArray, DATA_SIZE, MPI.INT, 0, 1);
      System.out.println(" received it ..");
      st = req.waitStatus();
      st.free();
      System.out.println(" called wait for send");

      if (Arrays.equals(intArray, intReadArray)) {
        System.out.println("test #1 passed");
      } else {
        System.out.println("test #1 failed");
      }

      /* test #2 */
      for (int j = 0; j < DATA_SIZE; j++) {
        intReadArray[j] = 0;
      }

      req = MPI.COMM_WORLD.iRecv(intReadArray, DATA_SIZE, MPI.INT, 0, 1);
      MPI.COMM_WORLD.send(intArray, DATA_SIZE, MPI.INT, 0, 1);
      st = req.waitStatus();
      st.free();

      if (Arrays.equals(intArray, intReadArray)) {
        System.out.println("test #2 passed");
      } else {
        System.out.println("test #2 failed");
      }

      /* test #3 */
      for (int j = 0; j < DATA_SIZE; j++) {
        intReadArray[j] = 0;
      }
      System.out.println(" calling recv ");
      req = MPI.COMM_WORLD.iRecv(intReadArray, DATA_SIZE, MPI.INT, MPI.ANY_SOURCE, 1);
      System.out.println(" calling send");
      MPI.COMM_WORLD.send(intArray, DATA_SIZE, MPI.INT, 0, 1);
      System.out.println(" sent ");
      st = req.waitStatus();
      st.free();
      System.out.println(" received ");

      if (Arrays.equals(intArray, intReadArray)) {
        System.out.println("test #3 passed");
      } else {
        System.out.println("test #3 passed");
      }

      System.out.println(" BreakANYSRC TEST COMPLETED ");
    }

    MPI.COMM_WORLD.barrier();
    MPI.Finalize();
    System.out.println(" BreakANYSRC TEST COMPLETED " + me);
  }

  public static void main(String[] args) throws MPIException {
    try {
      BreakANYSRC c = new BreakANYSRC(args);
    } catch (Exception e) {
    }
  }
}
