/* Copyright (c) 2001-2022, The Ohio State University. All rights
 * reserved.
 *
 * This file is part of the MVAPICH2-J software package developed by the
 * team members of The Ohio State University's Network-Based Computing
 * Laboratory (NBCL), headed by Professor Dhabaleswar K. (DK) Panda.
 *
 * For detailed copyright and licensing information, please refer to the
 * copyright file COPYRIGHT in the top level MVAPICH2-J directory.
 */

package mpi.pt2pt;

import mpi.*;

import java.util.Arrays;

public class BufferRoller {
  public BufferRoller() {}

  public BufferRoller(String args[]) throws Exception {

    MPI.Init(args);
    int me = MPI.COMM_WORLD.getRank();
    int iterations = 10;
    int dsize = 2000;
    int sendArray[] = new int[dsize];
    int recvArray[] = new int[dsize];

    if (MPI.COMM_WORLD.getRank() == 1) Thread.currentThread().sleep(10000);

    for (int j = 0; j < iterations; j++) {

      if (MPI.COMM_WORLD.getRank() == 0) {

        for (int i = 0; i < sendArray.length; i++) {
          sendArray[i] = (iterations * j) + j;
        }

        MPI.COMM_WORLD.send(sendArray, dsize, MPI.INT, 1, j);

      } else if (MPI.COMM_WORLD.getRank() == 1) {

        for (int i = 0; i < recvArray.length; i++) {
          recvArray[i] = 0;
          sendArray[i] = (iterations * j) + j;
        }

        MPI.COMM_WORLD.recv(recvArray, dsize, MPI.INT, 0, j);

        if (Arrays.equals(sendArray, recvArray)) {
          System.out.println(" <<<<PASSED>>>> ");
        } else {
          System.out.println(" <<<<FAILED>>>> ");
        }
      }
    }

    MPI.COMM_WORLD.barrier();
    MPI.Finalize();
  }

  public static void main(String[] args) throws Exception {
    try {
      BufferRoller c = new BufferRoller(args);
    } catch (Exception e) {
    }
  }
}
