/* Copyright (c) 2001-2022, The Ohio State University. All rights
 * reserved.
 *
 * This file is part of the MVAPICH2-J software package developed by the
 * team members of The Ohio State University's Network-Based Computing
 * Laboratory (NBCL), headed by Professor Dhabaleswar K. (DK) Panda.
 *
 * For detailed copyright and licensing information, please refer to the
 * copyright file COPYRIGHT in the top level MVAPICH2-J directory.
 */

package mpi.pt2pt;

import mpi.*;

import java.util.Arrays;

public class BufferTest1 {
  public BufferTest1() {}

  public BufferTest1(String args[]) throws Exception {

    int SIZE = 100;
    int SENDER = 0;
    int RECEIVER = 1;

    int INT_TAG = 999;
    int BYTE_TAG = 998;
    int CHAR_TAG = 997;
    int DOUBLE_TAG = 996;
    int LONG_TAG = 995;
    int BOOL_TAG = 994;
    int SHORT_TAG = 993;
    int FLOAT_TAG = 992;

    int BYTES_IN_INT = 4;
    int BYTES_IN_FLOAT = 4;
    int BYTES_IN_DOUBLE = 8;
    int BYTES_IN_LONG = 8;
    int BYTES_IN_SHORT = 2;
    int BYTES_IN_CHAR = 2;
    int BYTES_IN_BYTE = 1;
    int BYTES_IN_BOOL = 1;

    MPI.Init(args);

    Comm comm = MPI.COMM_WORLD;

    int me = comm.getRank();

    Status intStatus, floatStatus, doubleStatus, longStatus;
    Status boolStatus, shortStatus, charStatus, byteStatus;

    int intArray[] = new int[SIZE];
    float floatArray[] = new float[SIZE];
    double doubleArray[] = new double[SIZE];
    long longArray[] = new long[SIZE];
    boolean booleanArray[] = new boolean[SIZE];
    short shortArray[] = new short[SIZE];
    char charArray[] = new char[SIZE];
    byte byteArray[] = new byte[SIZE];

    int intReadArray[] = new int[SIZE];
    float floatReadArray[] = new float[SIZE];
    double doubleReadArray[] = new double[SIZE];
    long longReadArray[] = new long[SIZE];
    boolean booleanReadArray[] = new boolean[SIZE];
    short shortReadArray[] = new short[SIZE];
    char charReadArray[] = new char[SIZE];
    byte byteReadArray[] = new byte[SIZE];

    for (int i = 0; i < intArray.length; i++) {
      intArray[i] = i + 1;
      floatArray[i] = i + 11;
      doubleArray[i] = i + 11.11;
      longArray[i] = i + 11;
      booleanArray[i] = true;
      shortArray[i] = 1;
      charArray[i] = 's';
      byteArray[i] = 's';

      intReadArray[i] = 3;
      floatReadArray[i] = i + 19;
      doubleReadArray[i] = i + 99.11;
      longReadArray[i] = i + 9;
      shortReadArray[i] = 2;
      booleanReadArray[i] = false;
      charReadArray[i] = 'x';
      byteReadArray[i] = 'x';
    }

    if (comm.getRank() == SENDER) {

      comm.send(intArray, SIZE, MPI.INT, 1, INT_TAG);
      comm.send(byteArray, SIZE, MPI.BYTE, 1, BYTE_TAG);
      comm.send(charArray, SIZE, MPI.CHAR, 1, CHAR_TAG);
      comm.send(doubleArray, SIZE, MPI.DOUBLE, 1, DOUBLE_TAG);
      comm.send(longArray, SIZE, MPI.LONG, 1, LONG_TAG);
      comm.send(booleanArray, SIZE, MPI.BOOLEAN, 1, BOOL_TAG);
      comm.send(shortArray, SIZE, MPI.SHORT, 1, SHORT_TAG);
      comm.send(floatArray, SIZE, MPI.FLOAT, 1, FLOAT_TAG);

    } else if (comm.getRank() == RECEIVER) {
      intStatus = comm.recv(intReadArray, SIZE, MPI.INT, 0, INT_TAG);
      byteStatus = comm.recv(byteReadArray, SIZE, MPI.BYTE, 0, BYTE_TAG);
      charStatus = comm.recv(charReadArray, SIZE, MPI.CHAR, 0, CHAR_TAG);
      doubleStatus = comm.recv(doubleReadArray, SIZE, MPI.DOUBLE, 0, DOUBLE_TAG);
      longStatus = comm.recv(longReadArray, SIZE, MPI.LONG, 0, LONG_TAG);
      boolStatus = comm.recv(booleanReadArray, SIZE, MPI.BOOLEAN, 0, BOOL_TAG);
      shortStatus = comm.recv(shortReadArray, SIZE, MPI.SHORT, 0, SHORT_TAG);
      floatStatus = comm.recv(floatReadArray, SIZE, MPI.FLOAT, 0, FLOAT_TAG);

      if (Arrays.equals(intArray, intReadArray)
          && Arrays.equals(floatArray, floatReadArray)
          && Arrays.equals(doubleArray, doubleReadArray)
          && Arrays.equals(longArray, longReadArray)
          && Arrays.equals(shortArray, shortReadArray)
          && Arrays.equals(charArray, charReadArray)
          && Arrays.equals(byteArray, byteReadArray)
          && Arrays.equals(booleanArray, booleanReadArray)) {

        System.out.println(
            "\n#################" + "\n <<<<CONTENTS PASSED>>>> " + "\n################");
      } else {
        System.out.println(
            "\n#################" + "\n <<<< CONTENTS FAILED>>>> " + "\n################");
      }

      /* int status */
      if (intStatus.getSource() == SENDER
          && intStatus.getTag() == INT_TAG
          && intStatus.getCount(MPI.INT) == SIZE
          && intStatus.getElements(MPI.INT) == SIZE
          &&

          /* float status */
          floatStatus.getSource() == SENDER
          && floatStatus.getTag() == FLOAT_TAG
          && floatStatus.getCount(MPI.FLOAT) == SIZE
          && floatStatus.getElements(MPI.FLOAT) == SIZE
          &&

          /* double status */
          doubleStatus.getSource() == SENDER
          && doubleStatus.getTag() == DOUBLE_TAG
          && doubleStatus.getCount(MPI.DOUBLE) == SIZE
          && doubleStatus.getElements(MPI.DOUBLE) == SIZE
          &&

          /* long status */
          longStatus.getSource() == SENDER
          && longStatus.getTag() == LONG_TAG
          && longStatus.getCount(MPI.LONG) == SIZE
          && longStatus.getElements(MPI.LONG) == SIZE
          &&

          /* short status */
          shortStatus.getSource() == SENDER
          && shortStatus.getTag() == SHORT_TAG
          && shortStatus.getCount(MPI.SHORT) == SIZE
          && shortStatus.getElements(MPI.SHORT) == SIZE
          &&

          /* char status */
          charStatus.getSource() == SENDER
          && charStatus.getTag() == CHAR_TAG
          && charStatus.getCount(MPI.CHAR) == SIZE
          && charStatus.getElements(MPI.CHAR) == SIZE
          &&

          /* byte status */
          byteStatus.getSource() == SENDER
          && byteStatus.getTag() == BYTE_TAG
          && byteStatus.getCount(MPI.BYTE) == SIZE
          && byteStatus.getElements(MPI.BYTE) == SIZE
          &&

          /* boolean status */
          boolStatus.getSource() == SENDER
          && boolStatus.getTag() == BOOL_TAG
          && boolStatus.getCount(MPI.BOOLEAN) == SIZE
          && boolStatus.getElements(MPI.BOOLEAN) == SIZE) {

        System.out.println(
            "\n#################" + "\n <<<<STATUS PASSED>>>> " + "\n################");
      } else {
        System.out.println(
            "\n#################" + "\n <<<< STATUS FAILED>>>> " + "\n################");
      }
    }

    MPI.COMM_WORLD.barrier();

    if (comm.getRank() == RECEIVER)
      System.out.println("BufferTest1:Standard:Blocking send/recv test completed");

    MPI.Finalize();
  }

  public static void main(String args[]) throws Exception {
    BufferTest1 test = new BufferTest1(args);
  }
}
