/* Copyright (c) 2001-2022, The Ohio State University. All rights
 * reserved.
 *
 * This file is part of the MVAPICH2-J software package developed by the
 * team members of The Ohio State University's Network-Based Computing
 * Laboratory (NBCL), headed by Professor Dhabaleswar K. (DK) Panda.
 *
 * For detailed copyright and licensing information, please refer to the
 * copyright file COPYRIGHT in the top level MVAPICH2-J directory.
 */

package mpi.pt2pt;

import mpi.*;
import mpjbuf.*;

import java.util.Arrays;

public class BufferTest3 {
  public BufferTest3() {}

  public BufferTest3(String args[]) throws Exception {
    int offset = 0, size = 100;
    int DATA_SIZE = 2;
    int[] data = new int[DATA_SIZE];
    int[] rdata = new int[DATA_SIZE];

    for (int i = 0; i < data.length; i++) {
      data[i] = i;
    }

    RawBuffer rawBuffer = BufferFactory.create(size);

    // 1. create mpjbuf.Buffer object ..
    mpjbuf.Buffer mpjbuf = new mpjbuf.Buffer(rawBuffer, offset, size);

    // 2. write message ...
    mpjbuf.putSectionHeader(Type.INT);
    mpjbuf.write(data, 0, 2);

    mpjbuf.commit();

    // 3. read message ...
    Type sectionHeader = mpjbuf.getSectionHeader();
    int sectionSize = mpjbuf.getSectionSize();
    mpjbuf.read(rdata, 0, 2);
    mpjbuf.clear();

    // 4. compare results ...
    for (int i = 0; i < rdata.length; i++) {
      if (rdata[i] != i) {
        System.out.println(" Error: arrays don't match");
        break;
      }
    }

    System.out.println("BufferTest3 (mpjbuf.Buffers only) TEST Completed");

    BufferFactory.destroy(mpjbuf.getStaticBuffer());
  }

  public static void main(String args[]) throws Exception {
    BufferTest3 test = new BufferTest3(args);
  }
}
