/* Copyright (c) 2001-2022, The Ohio State University. All rights
 * reserved.
 *
 * This file is part of the MVAPICH2-J software package developed by the
 * team members of The Ohio State University's Network-Based Computing
 * Laboratory (NBCL), headed by Professor Dhabaleswar K. (DK) Panda.
 *
 * For detailed copyright and licensing information, please refer to the
 * copyright file COPYRIGHT in the top level MVAPICH2-J directory.
 */

package mpi.pt2pt;

import mpi.*;

import java.util.Arrays;

public class Isend_eager {
  int DATA_SIZE = 100;

  public Isend_eager() {}

  public Isend_eager(String args[]) throws Exception {

    MPI.Init(args);
    int me = MPI.COMM_WORLD.getRank();

    Request req[] = new Request[8];

    int intArray[] = new int[DATA_SIZE];
    float floatArray[] = new float[DATA_SIZE];
    double doubleArray[] = new double[DATA_SIZE];
    long longArray[] = new long[DATA_SIZE];
    boolean booleanArray[] = new boolean[DATA_SIZE];
    short shortArray[] = new short[DATA_SIZE];
    char charArray[] = new char[DATA_SIZE];
    byte byteArray[] = new byte[DATA_SIZE];

    int intReadArray[] = new int[DATA_SIZE];
    float floatReadArray[] = new float[DATA_SIZE];
    double doubleReadArray[] = new double[DATA_SIZE];
    long longReadArray[] = new long[DATA_SIZE];
    boolean booleanReadArray[] = new boolean[DATA_SIZE];
    short shortReadArray[] = new short[DATA_SIZE];
    char charReadArray[] = new char[DATA_SIZE];
    byte byteReadArray[] = new byte[DATA_SIZE];

    for (int i = 0; i < intArray.length; i++) {
      intArray[i] = i + 1;
      floatArray[i] = i + 11;
      doubleArray[i] = i + 11.11;
      longArray[i] = i + 11;
      booleanArray[i] = true;
      shortArray[i] = 1;
      charArray[i] = 's';
      byteArray[i] = 's';

      intReadArray[i] = 3;
      floatReadArray[i] = i + 19;
      doubleReadArray[i] = i + 99.11;
      longReadArray[i] = i + 9;
      shortReadArray[i] = 2;
      booleanReadArray[i] = false;
      charReadArray[i] = 'x';
      byteReadArray[i] = 'x';
    }

    if (MPI.COMM_WORLD.getRank() == 0) {
      for (int i = 0; i < 100; i = i + 8) {

        req[0] = MPI.COMM_WORLD.iSend(intArray, DATA_SIZE, MPI.INT, 1, i);
        req[1] = MPI.COMM_WORLD.iSend(byteArray, DATA_SIZE, MPI.BYTE, 1, i + 1);
        req[2] = MPI.COMM_WORLD.iSend(charArray, DATA_SIZE, MPI.CHAR, 1, i + 2);
        req[3] = MPI.COMM_WORLD.iSend(doubleArray, DATA_SIZE, MPI.DOUBLE, 1, i + 3);
        req[4] = MPI.COMM_WORLD.iSend(longArray, DATA_SIZE, MPI.LONG, 1, i + 4);
        req[5] = MPI.COMM_WORLD.iSend(booleanArray, DATA_SIZE, MPI.BOOLEAN, 1, i + 5);
        req[6] = MPI.COMM_WORLD.iSend(shortArray, DATA_SIZE, MPI.SHORT, 1, i + 6);
        req[7] = MPI.COMM_WORLD.iSend(floatArray, DATA_SIZE, MPI.FLOAT, 1, i + 7);

        for (int j = 0; j < 8; j++) {
          Status st = req[j].waitStatus();
          st.getTag();
          st.getSource();
          st.free();
        }
      }

    } else if (MPI.COMM_WORLD.getRank() == 1) {

      for (int i = 0; i < 100; i = i + 8) {
        req[0] = MPI.COMM_WORLD.iRecv(intReadArray, DATA_SIZE, MPI.INT, 0, i);
        req[1] = MPI.COMM_WORLD.iRecv(byteReadArray, DATA_SIZE, MPI.BYTE, 0, i + 1);
        req[2] = MPI.COMM_WORLD.iRecv(charReadArray, DATA_SIZE, MPI.CHAR, 0, i + 2);
        req[3] = MPI.COMM_WORLD.iRecv(doubleReadArray, DATA_SIZE, MPI.DOUBLE, 0, i + 3);
        req[4] = MPI.COMM_WORLD.iRecv(longReadArray, DATA_SIZE, MPI.LONG, 0, i + 4);
        req[5] = MPI.COMM_WORLD.iRecv(booleanReadArray, DATA_SIZE, MPI.BOOLEAN, 0, i + 5);
        req[6] = MPI.COMM_WORLD.iRecv(shortReadArray, DATA_SIZE, MPI.SHORT, 0, i + 6);
        req[7] = MPI.COMM_WORLD.iRecv(floatReadArray, DATA_SIZE, MPI.FLOAT, 0, i + 7);

        for (int j = 0; j < 8; j++) {
          Status status = req[j].waitStatus();
          status.free();
        }

        if (Arrays.equals(intArray, intReadArray)
            && Arrays.equals(floatArray, floatReadArray)
            && Arrays.equals(doubleArray, doubleReadArray)
            && Arrays.equals(longArray, longReadArray)
            && Arrays.equals(shortArray, shortReadArray)
            && Arrays.equals(charArray, charReadArray)
            && Arrays.equals(byteArray, byteReadArray)
            && Arrays.equals(booleanArray, booleanReadArray)) {
        } else {
          System.out.println("\n#################" + "\n <<<<FAILED>>>> " + "\n################");
        }
      }
    }

    MPI.COMM_WORLD.barrier();
    if (MPI.COMM_WORLD.getRank() == 0) {
      System.out.println("Isend_eager TEST Completed");
    }
    try {
      MPI.Finalize();
    } catch (Exception e) {
    }
  }

  public static void main(String args[]) throws Exception {
    Isend_eager test = new Isend_eager(args);
  }
}
