/* Copyright (c) 2001-2022, The Ohio State University. All rights
 * reserved.
 *
 * This file is part of the MVAPICH2-J software package developed by the
 * team members of The Ohio State University's Network-Based Computing
 * Laboratory (NBCL), headed by Professor Dhabaleswar K. (DK) Panda.
 *
 * For detailed copyright and licensing information, please refer to the
 * copyright file COPYRIGHT in the top level MVAPICH2-J directory.
 */

package mpi.pt2pt;

import mpi.*;

import java.util.Arrays;

public class Isend_rendez {

  int DATA_SIZE = 26000;

  public Isend_rendez() {}

  public Isend_rendez(String args[]) throws Exception {
    MPI.Init(args);
    int me = MPI.COMM_WORLD.getRank();
    Request rreq[] = new Request[8];
    Request sreq[] = new Request[8];

    int intArray[] = new int[DATA_SIZE];
    float floatArray[] = new float[DATA_SIZE];
    double doubleArray[] = new double[DATA_SIZE];
    long longArray[] = new long[DATA_SIZE];
    boolean booleanArray[] = new boolean[DATA_SIZE];
    short shortArray[] = new short[DATA_SIZE];
    char charArray[] = new char[DATA_SIZE];
    byte byteArray[] = new byte[DATA_SIZE];

    int intReadArray[] = new int[DATA_SIZE];
    float floatReadArray[] = new float[DATA_SIZE];
    double doubleReadArray[] = new double[DATA_SIZE];
    long longReadArray[] = new long[DATA_SIZE];
    boolean booleanReadArray[] = new boolean[DATA_SIZE];
    short shortReadArray[] = new short[DATA_SIZE];
    char charReadArray[] = new char[DATA_SIZE];
    byte byteReadArray[] = new byte[DATA_SIZE];

    for (int i = 0; i < intArray.length; i++) {
      intArray[i] = i + 1;
      floatArray[i] = i + 11;
      doubleArray[i] = i + 11.11;
      longArray[i] = i + 11;
      booleanArray[i] = true;
      shortArray[i] = 1;
      charArray[i] = 's';
      byteArray[i] = 's';
    }

    int peer = (me == 0) ? 1 : 0;

    for (int i = 1; i < 10; i = i + 4) {

      for (int j = 0; j < intArray.length; j++) {
        intReadArray[j] = 3;
        floatReadArray[j] = j + 19;
        doubleReadArray[j] = j + 99.11;
        longReadArray[j] = j + 9;
        shortReadArray[j] = 2;
        booleanReadArray[j] = false;
        charReadArray[j] = 'x';
        byteReadArray[j] = 'x';
      }

      // send calls
      sreq[0] = MPI.COMM_WORLD.iSend(intArray, DATA_SIZE, MPI.INT, peer, i);
      sreq[1] = MPI.COMM_WORLD.iSend(byteArray, DATA_SIZE, MPI.BYTE, peer, i + 1);
      sreq[2] = MPI.COMM_WORLD.iSend(charArray, DATA_SIZE, MPI.CHAR, peer, i + 2);
      sreq[3] = MPI.COMM_WORLD.iSend(doubleArray, DATA_SIZE, MPI.DOUBLE, peer, i + 3);
      sreq[4] = MPI.COMM_WORLD.iSend(longArray, DATA_SIZE, MPI.LONG, peer, i + 4);
      sreq[5] = MPI.COMM_WORLD.iSend(booleanArray, DATA_SIZE, MPI.BOOLEAN, peer, i + 5);
      sreq[6] = MPI.COMM_WORLD.iSend(shortArray, DATA_SIZE, MPI.SHORT, peer, i + 6);
      sreq[7] = MPI.COMM_WORLD.iSend(floatArray, DATA_SIZE, MPI.FLOAT, peer, i + 7);

      // recv calls
      rreq[0] = MPI.COMM_WORLD.iRecv(intReadArray, DATA_SIZE, MPI.INT, peer, i);
      rreq[1] = MPI.COMM_WORLD.iRecv(byteReadArray, DATA_SIZE, MPI.BYTE, peer, i + 1);
      rreq[2] = MPI.COMM_WORLD.iRecv(charReadArray, DATA_SIZE, MPI.CHAR, peer, i + 2);
      rreq[3] = MPI.COMM_WORLD.iRecv(doubleReadArray, DATA_SIZE, MPI.DOUBLE, peer, i + 3);
      rreq[4] = MPI.COMM_WORLD.iRecv(longReadArray, DATA_SIZE, MPI.LONG, peer, i + 4);
      rreq[5] = MPI.COMM_WORLD.iRecv(booleanReadArray, DATA_SIZE, MPI.BOOLEAN, peer, i + 5);
      rreq[6] = MPI.COMM_WORLD.iRecv(shortReadArray, DATA_SIZE, MPI.SHORT, peer, i + 6);
      rreq[7] = MPI.COMM_WORLD.iRecv(floatReadArray, DATA_SIZE, MPI.FLOAT, peer, i + 7);

      Status st;

      for (int j = 0; j < 8; j++) {
        st = sreq[j].waitStatus();
        st.free();
      }

      for (int j = 0; j < 8; j++) {
        st = rreq[j].waitStatus();
        st.free();
      }

      if (Arrays.equals(intArray, intReadArray)
          && Arrays.equals(charArray, charReadArray)
          && Arrays.equals(doubleArray, doubleReadArray)
          && Arrays.equals(floatArray, floatReadArray)
          && Arrays.equals(longArray, longReadArray)
          && Arrays.equals(shortArray, shortReadArray)
          && Arrays.equals(booleanArray, booleanReadArray)
          && Arrays.equals(byteArray, byteReadArray)) {

        System.out.println(
            "\n#################" + "\n <<<<sender PASSED>>>> " + "\n################");

      } else {
        System.out.println(
            "\n#################" + "\n <<<<sender FAILED>>>> " + "\n################");
      }
    }

    MPI.COMM_WORLD.barrier();

    if (MPI.COMM_WORLD.getRank() == 1) {
      System.out.println("Isend_rendez TEST Completed <" + me + ">");
    }

    MPI.Finalize();
  }

  public static void main(String args[]) throws Exception {
    Isend_rendez test = new Isend_rendez(args);
  }
}
