/* Copyright (c) 2001-2022, The Ohio State University. All rights
 * reserved.
 *
 * This file is part of the MVAPICH2-J software package developed by the
 * team members of The Ohio State University's Network-Based Computing
 * Laboratory (NBCL), headed by Professor Dhabaleswar K. (DK) Panda.
 *
 * For detailed copyright and licensing information, please refer to the
 * copyright file COPYRIGHT in the top level MVAPICH2-J directory.
 */

package mpi.pt2pt;

import mpi.*;

import java.util.Arrays;

public class Isend_rendez_fairness {
  int DATA_SIZE = 32 * 1024 * 1024;

  public Isend_rendez_fairness() {}

  public Isend_rendez_fairness(String args[]) throws Exception {

    MPI.Init(args);
    int me = MPI.COMM_WORLD.getRank();
    int size = MPI.COMM_WORLD.getRank();

    int intArray1[] = new int[DATA_SIZE];
    int intArray2[] = new int[DATA_SIZE];

    for (int i = 0; i < DATA_SIZE; i++) {
      if (me != 0) {
        intArray1[i] = -24;
        intArray2[i] = -80;
      }
    }

    if (size != 3) {
      System.out.println("Isend_rendez_fairness needs 3 processes");
      MPI.Finalize();
    }

    if (me == 0) {

      Request req2 = MPI.COMM_WORLD.iRecv(intArray2, DATA_SIZE, MPI.INT, 2, 992);
      Request req1 = MPI.COMM_WORLD.iRecv(intArray1, DATA_SIZE, MPI.INT, 1, 991);
      Status st1 = req1.waitStatus();
      st1.free();

      Status st2 = req2.waitStatus();
      st2.free();

      for (int j = 0; j < DATA_SIZE; j++) {
        if (intArray1[j] != 3 || intArray2[j] != 4) {
          System.out.println(" FAILED");
          break;
        }
      }

      System.out.println(" Process O Exiting");

    } else if (me == 1) {
      MPI.COMM_WORLD.send(intArray1, DATA_SIZE, MPI.INT, 0, 991);
    } else if (me == 2) {
      MPI.COMM_WORLD.send(intArray2, DATA_SIZE, MPI.INT, 0, 992);
    }

    MPI.COMM_WORLD.barrier();
    MPI.Finalize();
  }

  public static void main(String args[]) throws Exception {
    Isend_rendez_fairness test = new Isend_rendez_fairness(args);
  }
}
