/* Copyright (c) 2001-2022, The Ohio State University. All rights
 * reserved.
 *
 * This file is part of the MVAPICH2-J software package developed by the
 * team members of The Ohio State University's Network-Based Computing
 * Laboratory (NBCL), headed by Professor Dhabaleswar K. (DK) Panda.
 *
 * For detailed copyright and licensing information, please refer to the
 * copyright file COPYRIGHT in the top level MVAPICH2-J directory.
 */

package mpi.pt2pt;

import mpi.*;

import java.util.Arrays;

public class NonBlockingPt2ptWithArraysAndTest {
  public NonBlockingPt2ptWithArraysAndTest(String args[]) throws Exception {

    MPI.Init(args);

    int msgSize;

    if (args.length != 1) {
      msgSize = 1024; // default value
    } else {
      msgSize = Integer.parseInt(args[0]);
    }

    int tag = 10;

    Request req1, req2, req3, req4, req5, req6, req7, req8;
    Status s1 = null, s2 = null, s3 = null, s4 = null, s5 = null, s6 = null, s7 = null, s8 = null;

    int intArray[] = new int[msgSize];
    float floatArray[] = new float[msgSize];
    double doubleArray[] = new double[msgSize];
    long longArray[] = new long[msgSize];
    boolean booleanArray[] = new boolean[msgSize];
    short shortArray[] = new short[msgSize];
    char charArray[] = new char[msgSize];
    byte byteArray[] = new byte[msgSize];

    int intReadArray[] = new int[msgSize];
    float floatReadArray[] = new float[msgSize];
    double doubleReadArray[] = new double[msgSize];
    long longReadArray[] = new long[msgSize];
    boolean booleanReadArray[] = new boolean[msgSize];
    short shortReadArray[] = new short[msgSize];
    char charReadArray[] = new char[msgSize];
    byte byteReadArray[] = new byte[msgSize];

    for (int i = 0; i < intArray.length; i++) {
      intArray[i] = (int) 1;
      floatArray[i] = (float) 1.1;
      doubleArray[i] = (double) 1.1;
      longArray[i] = 0L;
      booleanArray[i] = true;
      shortArray[i] = (short) 1;
      charArray[i] = '1';
      byteArray[i] = (byte) 1;

      intReadArray[i] = (int) 0;
      floatReadArray[i] = (float) 0.0;
      doubleReadArray[i] = (double) 0.0;
      longReadArray[i] = 0L;
      shortReadArray[i] = (short) 0;
      booleanReadArray[i] = false;
      charReadArray[i] = '0';
      byteReadArray[i] = (byte) 0;
    }

    if (MPI.COMM_WORLD.getRank() == 0) {
      int sTag = tag;
      req1 = MPI.COMM_WORLD.iSend(intArray, msgSize, MPI.INT, 1, sTag++);
      req2 = MPI.COMM_WORLD.iSend(byteArray, msgSize, MPI.BYTE, 1, sTag++);
      req3 = MPI.COMM_WORLD.iSend(charArray, msgSize, MPI.CHAR, 1, sTag++);
      req4 = MPI.COMM_WORLD.iSend(doubleArray, msgSize, MPI.DOUBLE, 1, sTag++);
      req5 = MPI.COMM_WORLD.iSend(longArray, msgSize, MPI.LONG, 1, sTag++);
      req6 = MPI.COMM_WORLD.iSend(booleanArray, msgSize, MPI.BOOLEAN, 1, sTag++);
      req7 = MPI.COMM_WORLD.iSend(shortArray, msgSize, MPI.SHORT, 1, sTag++);
      req8 = MPI.COMM_WORLD.iSend(floatArray, msgSize, MPI.FLOAT, 1, sTag++);

      while (s1 == null) s1 = req1.testStatus();

      while (s2 == null) s2 = req2.testStatus();

      while (s3 == null) s3 = req3.testStatus();

      while (s4 == null) s4 = req4.testStatus();

      while (s5 == null) s5 = req5.testStatus();

      while (s6 == null) s6 = req6.testStatus();

      while (s7 == null) s7 = req7.testStatus();

      while (s8 == null) s8 = req8.testStatus();

    } else if (MPI.COMM_WORLD.getRank() == 1) {
      int rTag = tag;
      req1 = MPI.COMM_WORLD.iRecv(intReadArray, msgSize, MPI.INT, 0, rTag++);
      req2 = MPI.COMM_WORLD.iRecv(byteReadArray, msgSize, MPI.BYTE, 0, rTag++);
      req3 = MPI.COMM_WORLD.iRecv(charReadArray, msgSize, MPI.CHAR, 0, rTag++);
      req4 = MPI.COMM_WORLD.iRecv(doubleReadArray, msgSize, MPI.DOUBLE, 0, rTag++);
      req5 = MPI.COMM_WORLD.iRecv(longReadArray, msgSize, MPI.LONG, 0, rTag++);
      req6 = MPI.COMM_WORLD.iRecv(booleanReadArray, msgSize, MPI.BOOLEAN, 0, rTag++);
      req7 = MPI.COMM_WORLD.iRecv(shortReadArray, msgSize, MPI.SHORT, 0, rTag++);
      req8 = MPI.COMM_WORLD.iRecv(floatReadArray, msgSize, MPI.FLOAT, 0, rTag++);

      while (s1 == null) s1 = req1.testStatus();

      while (s2 == null) s2 = req2.testStatus();

      while (s3 == null) s3 = req3.testStatus();

      while (s4 == null) s4 = req4.testStatus();

      while (s5 == null) s5 = req5.testStatus();

      while (s6 == null) s6 = req6.testStatus();

      while (s7 == null) s7 = req7.testStatus();

      while (s8 == null) s8 = req8.testStatus();

      if (s1.getSource() != 0
          || s1.getTag() != tag
          || s1.getElements(MPI.INT) != msgSize
          || s1.getCount(MPI.INT) != msgSize) {
        System.out.println("MPI.INT");
        System.out.println(
            "Incorrect values for s1: s1.source=<"
                + s1.getSource()
                + ">, "
                + "s1.tag=<"
                + s1.getTag()
                + ">, s1.numEls=<"
                + s1.getElements(MPI.INT)
                + ">, s1.count=<"
                + s1.getCount(MPI.INT)
                + ">");
        System.out.println(
            "Correct values for s1: s1.source=<0>, "
                + "s1.tag=<"
                + tag
                + ">, s1.numEls=<"
                + msgSize
                + ">, s1.count=<"
                + msgSize
                + ">\n");
      }

      if (s2.getSource() != 0
          || s2.getTag() != tag + 1
          || s2.getElements(MPI.BYTE) != msgSize
          || s2.getCount(MPI.BYTE) != msgSize) {
        System.out.println("MPI.BYTE");
        System.out.println(
            "Incorrect values for s2: s2.source=<"
                + s2.getSource()
                + ">, "
                + "s2.tag=<"
                + s2.getTag()
                + ">, s2.numEls=<"
                + s2.getElements(MPI.BYTE)
                + ">, s2.count=<"
                + s2.getCount(MPI.BYTE)
                + ">");
        System.out.println(
            "Correct values for s2: s2.source=<0>, "
                + "s2.tag=<"
                + (tag + 1)
                + ">, s2.numEls=<"
                + msgSize
                + ">, s2.count=<"
                + msgSize
                + ">\n");
      }

      if (s3.getSource() != 0
          || s3.getTag() != tag + 2
          || s3.getElements(MPI.CHAR) != msgSize
          || s3.getCount(MPI.CHAR) != msgSize) {
        System.out.println("MPI.CHAR");
        System.out.println(
            "Incorrect values for s3: s3.source=<"
                + s3.getSource()
                + ">, "
                + "s3.tag=<"
                + s3.getTag()
                + ">, s3.numEls=<"
                + s3.getElements(MPI.CHAR)
                + ">, s3.count=<"
                + s3.getCount(MPI.CHAR)
                + ">");
        System.out.println(
            "Correct values for s3: s3.source=<0>, "
                + "s3.tag=<"
                + (tag + 2)
                + ">, s3.numEls=<"
                + msgSize
                + ">, s3.count=<"
                + msgSize
                + ">\n");
      }

      if (s4.getSource() != 0
          || s4.getTag() != tag + 3
          || s4.getElements(MPI.DOUBLE) != msgSize
          || s4.getCount(MPI.DOUBLE) != msgSize) {
        System.out.println("MPI.DOUBLE");
        System.out.println(
            "Incorrect values for s4: s4.source=<"
                + s4.getSource()
                + ">, "
                + "s4.tag=<"
                + s4.getTag()
                + ">, s4.numEls=<"
                + s4.getElements(MPI.DOUBLE)
                + ">, s4.count=<"
                + s4.getCount(MPI.DOUBLE)
                + ">");
        System.out.println(
            "Correct values for s4: s4.source=<0>, "
                + "s4.tag=<"
                + (tag + 3)
                + ">, s4.numEls=<"
                + msgSize
                + ">, s4.count=<"
                + msgSize
                + ">\n");
      }

      if (s5.getSource() != 0
          || s5.getTag() != tag + 4
          || s5.getElements(MPI.LONG) != msgSize
          || s5.getCount(MPI.LONG) != msgSize) {
        System.out.println("MPI.LONG");
        System.out.println(
            "Incorrect values for s5: s5.source=<"
                + s5.getSource()
                + ">, "
                + "s5.tag=<"
                + s5.getTag()
                + ">, s5.numEls=<"
                + s5.getElements(MPI.LONG)
                + ">, s5.count=<"
                + s5.getCount(MPI.LONG)
                + ">");
        System.out.println(
            "Correct values for s5: s5.source=<0>, "
                + "s5.tag=<"
                + (tag + 4)
                + ">, s5.numEls=<"
                + msgSize
                + ">, s5.count=<"
                + msgSize
                + ">\n");
      }

      if (s6.getSource() != 0
          || s6.getTag() != tag + 5
          || s6.getElements(MPI.BOOLEAN) != msgSize
          || s6.getCount(MPI.BOOLEAN) != msgSize) {
        System.out.println("MPI.BOOLEAN");
        System.out.println(
            "Incorrect values for s6: s6.source=<"
                + s6.getSource()
                + ">, "
                + "s6.tag=<"
                + s6.getTag()
                + ">, s6.numEls=<"
                + s6.getElements(MPI.BOOLEAN)
                + ">, s6.count=<"
                + s6.getCount(MPI.BOOLEAN)
                + ">");
        System.out.println(
            "Correct values for s6: s6.source=<0>, "
                + "s6.tag=<"
                + (tag + 5)
                + ">, s6.numEls=<"
                + msgSize
                + ">, s6.count=<"
                + msgSize
                + ">\n");
      }

      if (s7.getSource() != 0
          || s7.getTag() != tag + 6
          || s7.getElements(MPI.SHORT) != msgSize
          || s7.getCount(MPI.SHORT) != msgSize) {
        System.out.println("MPI.SHORT");
        System.out.println(
            "Incorrect values for s7: s7.source=<"
                + s7.getSource()
                + ">, "
                + "s7.tag=<"
                + s7.getTag()
                + ">, s7.numEls=<"
                + s7.getElements(MPI.SHORT)
                + ">, s5.count=<"
                + s7.getCount(MPI.SHORT)
                + ">");
        System.out.println(
            "Correct values for s7: s7.source=<0>, "
                + "s7.tag=<"
                + (tag + 6)
                + ">, s7.numEls=<"
                + msgSize
                + ">, s7.count=<"
                + msgSize
                + ">\n");
      }

      if (s8.getSource() != 0
          || s8.getTag() != tag + 7
          || s8.getElements(MPI.FLOAT) != msgSize
          || s8.getCount(MPI.FLOAT) != msgSize) {
        System.out.println("MPI.FLOAT");
        System.out.println(
            "Incorrect values for s8: s8.source=<"
                + s8.getSource()
                + ">, "
                + "s8.tag=<"
                + s8.getTag()
                + ">, s8.numEls=<"
                + s8.getElements(MPI.FLOAT)
                + ">, s8.count=<"
                + s8.getCount(MPI.FLOAT)
                + ">");
        System.out.println(
            "Correct values for s8: s8.source=<0>, "
                + "s8.tag=<"
                + (tag + 7)
                + ">, s8.numEls=<"
                + msgSize
                + ">, s8.count=<"
                + msgSize
                + ">\n");
      }

      if (Arrays.equals(intArray, intReadArray)
          && Arrays.equals(byteArray, byteReadArray)
          && Arrays.equals(charArray, charReadArray)
          && Arrays.equals(doubleArray, doubleReadArray)
          && Arrays.equals(longArray, longReadArray)
          && Arrays.equals(booleanArray, booleanReadArray)
          && Arrays.equals(shortArray, shortReadArray)
          && Arrays.equals(floatArray, floatReadArray)) {

        System.out.println("#################" + "\n <<<<PASSED>>>> " + "\n#################");

      } else {

        System.out.println("#################" + "\n <<<<FAILED>>>> " + "\n#################");
      }
    }

    s1.free();
    s2.free();
    s3.free();
    s4.free();
    s5.free();
    s6.free();
    s7.free();
    s8.free();

    MPI.COMM_WORLD.barrier();

    try {
      MPI.Finalize();
    } catch (Exception e) {
    }
  }

  public static void main(String args[]) throws Exception {
    NonBlockingPt2ptWithArraysAndTest test = new NonBlockingPt2ptWithArraysAndTest(args);
  }
}
