/* Copyright (c) 2001-2022, The Ohio State University. All rights
 * reserved.
 *
 * This file is part of the MVAPICH2-J software package developed by the
 * team members of The Ohio State University's Network-Based Computing
 * Laboratory (NBCL), headed by Professor Dhabaleswar K. (DK) Panda.
 *
 * For detailed copyright and licensing information, please refer to the
 * copyright file COPYRIGHT in the top level MVAPICH2-J directory.
 */

package mpi.pt2pt;

import mpi.*;

import java.util.Arrays;

public class NonBlockingPt2ptWithObjects {
  public NonBlockingPt2ptWithObjects(String args[]) throws Exception {

    Request req;
    MPI.Init(args);

    java.util.Vector vector1 = null;
    java.util.Vector vector = new java.util.Vector();

    for (int i = 0; i < 10; i++) {
      vector.add(i + "");
    }

    Object[] source = new Object[5];
    source[0] = vector;
    source[1] = vector;
    source[2] = vector;
    source[3] = vector;
    source[4] = vector;

    Object[] dest = new Object[5];
    dest[0] = null;
    dest[1] = null;
    dest[2] = null;
    dest[3] = null;
    dest[4] = null;

    Status status;

    if (MPI.COMM_WORLD.getRank() == 0) {
      req = MPI.COMM_WORLD.iSend(source, 5, MPI.OBJECT, 1, 999);
      status = req.waitStatus();
      status.free();
    } else if (MPI.COMM_WORLD.getRank() == 1) {

      req = MPI.COMM_WORLD.iRecv(dest, 5, MPI.OBJECT, 0, 999);
      status = req.waitStatus();
      status.free();

      if (Arrays.equals(source, dest)) {
        System.out.println("#################" + "\n <<<<PASSED>>>> " + "\n################");
      } else {
        System.out.println("#################" + "\n <<<<FAILED>>>> " + "\n################");
      }
    }

    MPI.COMM_WORLD.barrier();
    MPI.Finalize();
  }

  public static void main(String args[]) throws Exception {
    NonBlockingPt2ptWithObjects test = new NonBlockingPt2ptWithObjects(args);
  }
}
