/* Copyright (c) 2001-2022, The Ohio State University. All rights
 * reserved.
 *
 * This file is part of the MVAPICH2-J software package developed by the
 * team members of The Ohio State University's Network-Based Computing
 * Laboratory (NBCL), headed by Professor Dhabaleswar K. (DK) Panda.
 *
 * For detailed copyright and licensing information, please refer to the
 * copyright file COPYRIGHT in the top level MVAPICH2-J directory.
 */

/****************************************************************************
 *
 * MESSAGE PASSING INTERFACE TEST CASE SUITE
 *
 * Copyright IBM Corp. 1995
 *
 * IBM Corp. hereby grants a non-exclusive license to use, copy, modify, and
 * distribute this software for any purpose and without fee provided that the
 * above copyright notice and the following paragraphs appear in all copies.
 *
 * IBM Corp. makes no representation that the test cases comprising this
 * suite are correct or are an accurate representation of any standard.
 *
 * In no event shall IBM be liable to any party for direct, indirect, special
 * incidental, or consequential damage arising out of the use of this software
 * even if IBM Corp. has been advised of the possibility of such damage.
 *
 * IBM CORP. SPECIFICALLY DISCLAIMS ANY WARRANTIES INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE.  THE SOFTWARE PROVIDED HEREUNDER IS ON AN "AS IS" BASIS AND IBM
 * CORP. HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
 * ENHANCEMENTS, OR MODIFICATIONS.
 ****************************************************************************
 *
 * These test cases reflect an interpretation of the MPI Standard.  They are
 * are, in most cases, unit tests of specific MPI behaviors.  If a user of any
 * test case from this set believes that the MPI Standard requires behavior
 * different than that implied by the test case we would appreciate feedback.
 *
 * Comments may be sent to:
 * Richard Treumann
 * treumann@kgn.ibm.com
 ****************************************************************************
 *
 * MPI-Java version :
 * Sung-Hoon Ko(shko@npac.syr.edu)
 * Northeast Parallel Architectures Center at Syracuse University
 * 03/22/98
 ****************************************************************************
 */
package mpi.pt2pt;

import mpi.*;

class getcount {
  public static void main(String[] args) throws MPIException {
    int me, count;
    byte dataBYTE[] = new byte[5];
    char dataCHAR[] = new char[5];
    int dataINT[] = new int[5];
    float dataFLOAT[] = new float[5];
    double dataDOUBLE[] = new double[5];
    short dataSHORT[] = new short[5];
    long dataLONG[] = new long[5];
    Status status;
    Status st2;

    MPI.Init(args);
    me = MPI.COMM_WORLD.getRank();

    if (me == 0) {
      MPI.COMM_WORLD.send(dataBYTE, 5, MPI.BYTE, 1, 1);

      MPI.COMM_WORLD.send(dataCHAR, 5, MPI.CHAR, 1, 1);

      MPI.COMM_WORLD.send(dataINT, 5, MPI.INT, 1, 1);

      MPI.COMM_WORLD.send(dataFLOAT, 5, MPI.FLOAT, 1, 1);

      MPI.COMM_WORLD.send(dataDOUBLE, 5, MPI.DOUBLE, 1, 1);

      MPI.COMM_WORLD.send(dataSHORT, 5, MPI.SHORT, 1, 1);

      MPI.COMM_WORLD.send(dataLONG, 5, MPI.LONG, 1, 1);

    } else if (me == 1) {
      status = MPI.COMM_WORLD.recv(dataBYTE, 5, MPI.BYTE, 0, 1);
      count = status.getCount(MPI.BYTE);
      if (count != 5)
        System.out.println("ERROR(1) in getCount(MPI.BYTE), count = " + count + ", should be 5");

      status = MPI.COMM_WORLD.recv(dataCHAR, 5, MPI.CHAR, 0, 1);
      count = status.getCount(MPI.CHAR);
      if (count != 5)
        System.out.println("ERROR(2) in getCount(MPI.CHAR), count = " + count + ", should be 5");

      status = MPI.COMM_WORLD.recv(dataINT, 5, MPI.INT, 0, 1);
      count = status.getCount(MPI.INT);
      if (count != 5)
        System.out.println("ERROR(3) in getCount(MPI.INT), count = " + count + ", should be 5");

      status = MPI.COMM_WORLD.recv(dataFLOAT, 5, MPI.FLOAT, 0, 1);
      count = status.getCount(MPI.FLOAT);
      if (count != 5)
        System.out.println("ERROR(4) in getCount(MPI.FLOAT), count = " + count + ", should be 5");

      status = MPI.COMM_WORLD.recv(dataDOUBLE, 5, MPI.DOUBLE, 0, 1);
      count = status.getCount(MPI.DOUBLE);
      if (count != 5)
        System.out.println("ERROR(5) in getCount(MPI.DOUBLE), count = " + count + ", should be 5");

      status = MPI.COMM_WORLD.recv(dataSHORT, 5, MPI.SHORT, 0, 1);
      count = status.getCount(MPI.SHORT);
      if (count != 5)
        System.out.println("ERROR(6) in getCount(MPI.SHORT), count = " + count + ", should be 5");

      status = MPI.COMM_WORLD.recv(dataLONG, 5, MPI.LONG, 0, 1);
      count = status.getCount(MPI.LONG);
      if (count != 5)
        System.out.println("ERROR(7) in getCount(MPI.LONG), count = " + count + ", should be 5");
    }

    MPI.COMM_WORLD.barrier();
    if (me == 1) System.out.println("getCount TEST COMPLETE.\n");
    MPI.Finalize();
  }
}
