/* Copyright (c) 2001-2022, The Ohio State University. All rights
 * reserved.
 *
 * This file is part of the MVAPICH2-J software package developed by the
 * team members of The Ohio State University's Network-Based Computing
 * Laboratory (NBCL), headed by Professor Dhabaleswar K. (DK) Panda.
 *
 * For detailed copyright and licensing information, please refer to the
 * copyright file COPYRIGHT in the top level MVAPICH2-J directory.
 */

/****************************************************************************
 *
 * MESSAGE PASSING INTERFACE TEST CASE SUITE
 *
 * Copyright IBM Corp. 1995
 *
 * IBM Corp. hereby grants a non-exclusive license to use, copy, modify, and
 * distribute this software for any purpose and without fee provided that the
 * above copyright notice and the following paragraphs appear in all copies.
 *
 * IBM Corp. makes no representation that the test cases comprising this
 * suite are correct or are an accurate representation of any standard.
 *
 * In no event shall IBM be liable to any party for direct, indirect, special
 * incidental, or consequential damage arising out of the use of this software
 * even if IBM Corp. has been advised of the possibility of such damage.
 *
 * IBM CORP. SPECIFICALLY DISCLAIMS ANY WARRANTIES INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE.  THE SOFTWARE PROVIDED HEREUNDER IS ON AN "AS IS" BASIS AND IBM
 * CORP. HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
 * ENHANCEMENTS, OR MODIFICATIONS.
 ****************************************************************************
 *
 * These test cases reflect an interpretation of the MPI Standard.  They are
 * are, in most cases, unit tests of specific MPI behaviors.  If a user of any
 * test case from this set believes that the MPI Standard requires behavior
 * different than that implied by the test case we would appreciate feedback.
 *
 * Comments may be sent to:
 * Richard Treumann
 * treumann@kgn.ibm.com
 ****************************************************************************
 *
 * MPI-Java version :
 * Sung-Hoon Ko(shko@npac.syr.edu)
 * Northeast Parallel Architectures Center at Syracuse University
 * 09/10/99
 ****************************************************************************
 */

package mpi.pt2pt;

import mpi.*;

public class iprobe {
  public iprobe() {}

  public iprobe(String[] args) throws Exception {
    int me, cnt = 1, src = -1, tag;
    int data[] = new int[1];
    boolean flag;
    Intracomm comm;
    Status status = null;

    MPI.Init(args);
    comm = MPI.COMM_WORLD;
    me = comm.getRank();

    if (me == 0) {

      data[0] = 7;
      comm.send(data, 1, MPI.INT, 1, 1);

    } else if (me == 1) {

      for (; ; ) {
        status = comm.iProbe(0, 1);

        if (status != null) {
          break;
        }
      }

      src = status.getSource();
      if (src != 0) System.out.println("ERROR in MPI_Probe: src = " + src + ", should be 0");

      tag = status.getTag();
      if (tag != 1) System.out.println("ERROR in MPI_Probe: tag = " + tag + ", should be 1");

      cnt = status.getCount(MPI.INT);

      if (cnt != 1) System.out.println("ERROR in MPI_Probe: cnt = " + cnt + ", should be 1");

      status.free();

      status = comm.recv(data, cnt, MPI.INT, src, tag);

      if (status.getTag() != tag)
        System.out.println("ERROR in MPI_Recv, status.tag = " + status.tag + ", should be " + tag);

      if (status.getSource() != src)
        System.out.println(
            "ERROR in MPI_Recv, status.source = " + status.source + ", should be " + src);

      if (status.getCount(MPI.INT) != cnt)
        System.out.println(
            "ERROR in MPI_Recv, status.getCount(MPI.INT) = "
                + status.getCount(MPI.INT)
                + ", should be "
                + cnt);

      if (data[0] != 7) System.out.println("ERROR in MPI_Recv,data[0]=" + data[0] + "should be 7");

      status.free();
    }

    comm.barrier();

    if (me == 1) System.out.println("Iprobe TEST COMPLETE ");

    MPI.Finalize();
  }

  public static void main(String[] args) throws Exception {
    try {
      iprobe c = new iprobe(args);
    } catch (Exception e) {
    }
  }
}
