/* Copyright (c) 2001-2022, The Ohio State University. All rights
 * reserved.
 *
 * This file is part of the MVAPICH2-J software package developed by the
 * team members of The Ohio State University's Network-Based Computing
 * Laboratory (NBCL), headed by Professor Dhabaleswar K. (DK) Panda.
 *
 * For detailed copyright and licensing information, please refer to the
 * copyright file COPYRIGHT in the top level MVAPICH2-J directory.
 */

package mpi.pt2pt;

import mpi.*;

import java.nio.ByteBuffer;

public class isend2 {

  static int tasks, bytes, i;
  static int rank;
  static int DATA_SIZE = 1000;
  static int data[] = new int[DATA_SIZE];
  static Request req = null;
  static Status stats = null;

  public isend2() {}

  public isend2(String[] args) throws Exception {

    MPI.Init(args);
    rank = MPI.COMM_WORLD.getRank();
    tasks = MPI.COMM_WORLD.getSize();

    if (rank == 0) {
      System.out.println("> Testing Isend/Irecv...");
      for (i = 0; i < DATA_SIZE; i++) data[i] = -1;
      req = MPI.COMM_WORLD.iSend(data, DATA_SIZE, MPI.INT, 1, 1);
      stats = req.waitStatus();
    } else if (rank == 1) {
      req = MPI.COMM_WORLD.iRecv(data, DATA_SIZE, MPI.INT, 0, 1);
      stats = req.waitStatus();
      int num = stats.getCount(MPI.INT);
      int els = stats.getElements(MPI.INT);

      for (int j = 0; j < DATA_SIZE; j++) {
        if (data[j] != -1) {
          System.out.println("ERROR(0): incorrect data");
        }
      }
      if (num != DATA_SIZE) {
        System.out.println("ERROR(1): received ints are not equal " + "to num sent");
      }
      if (stats.getSource() != 0) {
        System.out.println("ERROR(2): source not equal to 0");
      }
      if (stats.getTag() != 1) {
        System.out.println("ERROR(3): tag not equal to 1");
      }
      if (els != DATA_SIZE * 4) {
        System.out.println("ERROR(4): received bytes are not equal " + "to bytes sent");
      }

      for (i = 0; i < DATA_SIZE; i++) data[i] = 0;
    }

    stats.free();

    if (rank == 0) {
      System.out.println("> Testing ibSend/Irecv...");
      ByteBuffer buf = ByteBuffer.allocateDirect(MPI.COMM_WORLD.packSize(DATA_SIZE, MPI.INT));
      MPI.attachBuffer(buf);
      for (i = 0; i < DATA_SIZE; i++) data[i] = -1;
      req = MPI.COMM_WORLD.ibSend(data, DATA_SIZE, MPI.INT, 1, 1);
      stats = req.waitStatus();
    } else if (rank == 1) {
      req = MPI.COMM_WORLD.iRecv(data, DATA_SIZE, MPI.INT, 0, 1);
      stats = req.waitStatus();
      int num = stats.getCount(MPI.INT);
      int els = stats.getElements(MPI.INT);

      for (int j = 0; j < DATA_SIZE; j++) {
        if (data[j] != -1) {
          System.out.println("ERROR(5): incorrect data");
        }
      }
      if (num != DATA_SIZE) {
        System.out.println("ERROR(6): received ints are not equal " + "to num sent");
      }
      if (stats.getSource() != 0) {
        System.out.println("ERROR(7): source not equal to 0");
      }
      if (stats.getTag() != 1) {
        System.out.println("ERROR(8): tag not equal to 1");
      }
      if (els != DATA_SIZE * 4) {
        System.out.println("ERROR(9): received bytes are not equal " + "to bytes sent");
      }

      for (i = 0; i < DATA_SIZE; i++) data[i] = 0;
    }

    stats.free();

    if (rank == 0) {
      System.out.println("> Testing Irsend/Irecv...");
      for (i = 0; i < DATA_SIZE; i++) data[i] = -1;
      req = MPI.COMM_WORLD.irSend(data, DATA_SIZE, MPI.INT, 1, 1);
      stats = req.waitStatus();
    } else if (rank == 1) {
      req = MPI.COMM_WORLD.iRecv(data, DATA_SIZE, MPI.INT, 0, 1);
      stats = req.waitStatus();
      int num = stats.getCount(MPI.INT);
      int els = stats.getElements(MPI.INT);

      for (int j = 0; j < DATA_SIZE; j++) {
        if (data[j] != -1) {
          System.out.println("ERROR(10): incorrect data");
        }
      }
      if (num != DATA_SIZE) {
        System.out.println("ERROR(11): received ints are not equal " + "to num sent");
      }
      if (stats.getSource() != 0) {
        System.out.println("ERROR(12): source not equal to 0");
      }
      if (stats.getTag() != 1) {
        System.out.println("ERROR(13): tag not equal to 1");
      }
      if (els != DATA_SIZE * 4) {
        System.out.println("ERROR(14): received bytes are not equal " + "to bytes sent");
      }

      for (i = 0; i < DATA_SIZE; i++) data[i] = 0;
    }

    stats.free();

    if (rank == 0) {
      System.out.println("> Testing Issend/Irecv...");
      for (i = 0; i < DATA_SIZE; i++) data[i] = -1;
      req = MPI.COMM_WORLD.isSend(data, DATA_SIZE, MPI.INT, 1, 1);
      stats = req.waitStatus();
    } else if (rank == 1) {
      req = MPI.COMM_WORLD.iRecv(data, DATA_SIZE, MPI.INT, 0, 1);
      stats = req.waitStatus();
      int num = stats.getCount(MPI.INT);
      int els = stats.getElements(MPI.INT);

      for (int j = 0; j < DATA_SIZE; j++) {
        if (data[j] != -1) {
          System.out.println("ERROR(15): incorrect data");
        }
      }
      if (num != DATA_SIZE) {
        System.out.println("ERROR(16): received ints are not equal " + "to num sent");
      }
      if (stats.getSource() != 0) {
        System.out.println("ERROR(17): source not equal to 0");
      }
      if (stats.getTag() != 1) {
        System.out.println("ERROR(18): tag not equal to 1");
      }
      if (els != DATA_SIZE * 4) {
        System.out.println("ERROR(19): received bytes are not equal " + "to bytes sent");
      }
    }

    stats.free();

    MPI.COMM_WORLD.barrier();
    if (rank == 1) System.out.println("Isend2 TEST COMPLETE\n");
    MPI.Finalize();
  }

  public static void main(String[] args) throws MPIException {
    try {
      isend2 c = new isend2(args);
    } catch (Exception e) {
      e.printStackTrace();
    }
  }
}
