/* Copyright (c) 2001-2022, The Ohio State University. All rights
 * reserved.
 *
 * This file is part of the MVAPICH2-J software package developed by the
 * team members of The Ohio State University's Network-Based Computing
 * Laboratory (NBCL), headed by Professor Dhabaleswar K. (DK) Panda.
 *
 * For detailed copyright and licensing information, please refer to the
 * copyright file COPYRIGHT in the top level MVAPICH2-J directory.
 */

package mpi.pt2pt;

import mpi.*;

import java.util.Arrays;

public class non_overtaking {
  public non_overtaking() {}

  public non_overtaking(String args[]) throws Exception {

    MPI.Init(args);

    int me = MPI.COMM_WORLD.getRank();

    int intArray[] = new int[100];
    float floatArray[] = new float[100];

    int intReadArray[] = new int[100];
    float floatReadArray[] = new float[100];

    for (int i = 0; i < intArray.length; i++) {
      intArray[i] = i + 1;
      floatArray[i] = i + 11;

      intReadArray[i] = 3;
      floatReadArray[i] = i + 19;
    }

    if (MPI.COMM_WORLD.getRank() == 0) {
      MPI.COMM_WORLD.sSend(intArray, 100, MPI.INT, 1, 999);
      MPI.COMM_WORLD.sSend(floatArray, 100, MPI.FLOAT, 1, 992);
    } else if (MPI.COMM_WORLD.getRank() == 1) {
      MPI.COMM_WORLD.recv(intReadArray, 100, MPI.INT, 0, MPI.ANY_TAG);
      MPI.COMM_WORLD.recv(floatReadArray, 100, MPI.FLOAT, 0, 992);

      if (Arrays.equals(intArray, intReadArray) && Arrays.equals(floatArray, floatReadArray)) {

        System.out.println("\n#################" + "\n <<<<PASSED>>>> " + "\n################");

      } else {
        System.out.println("\n#################" + "\n <<<<FAILED>>>> " + "\n################");
      }
    }

    MPI.COMM_WORLD.barrier();

    if (MPI.COMM_WORLD.getRank() == 0) {
      System.out.println("non_overtaking TEST Completed");
    }

    MPI.Finalize();
  }

  public static void main(String args[]) throws Exception {
    non_overtaking test = new non_overtaking(args);
  }
}
