/* Copyright (c) 2001-2022, The Ohio State University. All rights
 * reserved.
 *
 * This file is part of the MVAPICH2-J software package developed by the
 * team members of The Ohio State University's Network-Based Computing
 * Laboratory (NBCL), headed by Professor Dhabaleswar K. (DK) Panda.
 *
 * For detailed copyright and licensing information, please refer to the
 * copyright file COPYRIGHT in the top level MVAPICH2-J directory.
 */

/***************************************************************************
 *
 * MESSAGE PASSING INTERFACE TEST CASE SUITE
 *
 * Copyright IBM Corp. 1995
 *
 * IBM Corp. hereby grants a non-exclusive license to use, copy, modify, and
 * distribute this software for any purpose and without fee provided that the
 * above copyright notice and the following paragraphs appear in all copies.
 *
 * IBM Corp. makes no representation that the test cases comprising this
 * suite are correct or are an accurate representation of any standard.
 *
 * In no event shall IBM be liable to any party for direct, indirect, special
 * incidental, or consequential damage arising out of the use of this software
 * even if IBM Corp. has been advised of the possibility of such damage.
 *
 * IBM CORP. SPECIFICALLY DISCLAIMS ANY WARRANTIES INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE.  THE SOFTWARE PROVIDED HEREUNDER IS ON AN "AS IS" BASIS AND IBM
 * CORP. HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
 * ENHANCEMENTS, OR MODIFICATIONS.
 ****************************************************************************
 *
 * These test cases reflect an interpretation of the MPI Standard.  They are
 * are, in most cases, unit tests of specific MPI behaviors.  If a user of any
 * test case from this set believes that the MPI Standard requires behavior
 * different than that implied by the test case we would appreciate feedback.
 *
 * Comments may be sent to:
 * Richard Treumann
 * treumann@kgn.ibm.com
 ****************************************************************************
 *
 * MPI-Java version :
 * Sung-Hoon Ko(shko@npac.syr.edu)
 * Northeast Parallel Architectures Center at Syracuse University
 * 03/22/98
 ***************************************************************************/

package mpi.pt2pt;

import mpi.*;

import java.nio.ByteBuffer;

class probe {
  public static void main(String[] args) throws MPIException {

    int me, i, cnt, src, tag, tasks;
    int data[] = new int[1];
    ByteBuffer bufferData = ByteBuffer.allocateDirect(4);
    Intracomm comm;
    Status status;

    MPI.Init(args);
    comm = MPI.COMM_WORLD;
    me = comm.getRank();
    tasks = comm.getSize();

    // probe for specific source, tag
    if (me > 0) {
      data[0] = me;
      comm.send(data, 1, MPI.INT, 0, me);
    } else {

      for (i = 1; i < tasks; i++) {
        status = comm.probe(i, i);

        src = status.getSource();
        if (src != i)
          System.out.println("ERROR in MPI_Probe(1): src = " + src + ", should be " + i);

        tag = status.getTag();
        if (tag != i)
          System.out.println("ERROR in MPI_Probe(1): tag = " + tag + ", should be " + i);

        cnt = status.getCount(MPI.INT);
        if (cnt != 1) {
          System.out.println("ERROR in MPI_Probe(1): cnt = " + cnt + ", should be 1");
        }

        status = comm.recv(data, cnt, MPI.INT, src, tag);

        if (data[0] != i)
          System.out.println("ERROR in MPI_Recv(1), data = " + data[0] + ", should be " + i);

        if (status.getTag() != tag)
          System.out.println(
              "ERROR in MPI_Recv(1), status.tag = " + status.getTag() + ", should be " + tag);

        if (status.getSource() != src)
          System.out.println(
              "ERROR in MPI_Recv(1), status.source = " + status.getSource() + ", should be " + src);

        if (status.getCount(MPI.INT) != cnt)
          System.out.println(
              "ERROR in MPI_Recv(1), status.getCount(MPI.INT) = "
                  + status.getCount(MPI.INT)
                  + ", should be "
                  + cnt);
      }
    }

    comm.barrier();

    // probe for specific source, tag = MPI_ANY_TAG
    if (me > 0) {
      data[0] = me;
      comm.send(data, 1, MPI.INT, 0, me);
    } else {

      for (i = 1; i < tasks; i++) {
        status = comm.probe(i, MPI.ANY_TAG);

        src = status.getSource();
        if (src != i)
          System.out.println("ERROR in MPI_Probe(2): src = " + src + ", should be " + i);

        tag = status.getTag();
        if (tag != i)
          System.out.println("ERROR in MPI_Probe(2): tag = " + tag + ", should be " + i);

        cnt = status.getCount(MPI.INT);
        if (cnt != 1) System.out.println("ERROR in MPI_Probe(2): cnt = " + cnt + ", should be 1");

        status = comm.recv(data, cnt, MPI.INT, src, tag);

        if (data[0] != i)
          System.out.println("ERROR in MPI_Recv(2), data = " + data[0] + ", should be " + i);

        if (status.getTag() != tag)
          System.out.println(
              "ERROR in MPI_Recv(2), status.tag = " + status.getTag() + ", should be " + tag);

        if (status.getSource() != src)
          System.out.println(
              "ERROR in MPI_Recv(2), status.source = " + status.getSource() + ", should be " + src);

        if (status.getCount(MPI.INT) != cnt)
          System.out.println(
              "ERROR in MPI_Recv(2), status.getCount(MPI.INT) = "
                  + status.getCount(MPI.INT)
                  + ", should be "
                  + cnt);
      }
    }

    comm.barrier();

    // probe for specific tag, source = MPI_ANY_SOURCE
    if (me > 0) {
      data[0] = me;
      comm.send(data, 1, MPI.INT, 0, me);
    } else {

      for (i = 1; i < tasks; i++) {
        status = comm.probe(MPI.ANY_SOURCE, i);

        src = status.getSource();
        if (src != i)
          System.out.println("ERROR in MPI_Probe(3): src = " + src + ", should be " + i);

        tag = status.getTag();
        if (tag != i)
          System.out.println("ERROR in MPI_Probe(3): tag = " + tag + ", should be " + i);

        cnt = status.getCount(MPI.INT);
        if (cnt != 1) System.out.println("ERROR in MPI_Probe(3): cnt = " + cnt + ", should be 1");

        status = comm.recv(data, cnt, MPI.INT, src, tag);

        if (data[0] != i)
          System.out.println("ERROR in MPI_Recv(3), data = " + data[0] + ", should be " + i);

        if (status.getTag() != tag)
          System.out.println(
              "ERROR in MPI_Recv(3), status.tag = " + status.getTag() + ", should be " + tag);

        if (status.getSource() != src)
          System.out.println(
              "ERROR in MPI_Recv(3), status.source = " + status.getSource() + ", should be " + src);

        if (status.getCount(MPI.INT) != cnt)
          System.out.println(
              "ERROR in MPI_Recv(3), status.getCount(MPI.INT) = "
                  + status.getCount(MPI.INT)
                  + ", should be "
                  + cnt);
      }
    }

    comm.barrier();

    if (me > 0) {
      data[0] = me;
      comm.send(data, 1, MPI.INT, 0, me);
    } else {

      for (i = 1; i < tasks; i++) {
        status = comm.probe(MPI.ANY_SOURCE, MPI.ANY_TAG);

        src = status.getSource();
        tag = status.getTag();

        if (src != tag)
          System.out.println("ERROR in MPI_Probe(4): tag = " + tag + ", should be " + src);

        cnt = status.getCount(MPI.INT);
        if (cnt != 1)
          System.out.println("ERROR in MPI_Probe(4): cnt = " + cnt + ", should be 1 int");

        status = comm.recv(data, cnt, MPI.INT, src, tag);

        if (data[0] != src)
          System.out.println("ERROR in MPI_Recv(4), data = " + data + ", should be " + src);

        if (status.getTag() != tag)
          System.out.println(
              "ERROR in MPI_Recv(4), status.tag = " + status.getTag() + ", should be " + tag);

        if (status.getSource() != src)
          System.out.println(
              "ERROR in MPI_Recv(4), status.source = " + status.getSource() + ", should be " + src);

        if (status.getCount(MPI.INT) != cnt)
          System.out.println(
              "ERROR in MPI_Recv(4), status.getCount(MPI.INT) = "
                  + status.getCount(MPI.INT)
                  + ", should be "
                  + cnt);
      }
    }

    comm.barrier();

    if (me > 0) {
      data[0] = me;
      bufferData.putInt(me);
      bufferData.flip();
      comm.send(bufferData, 4, MPI.BYTE, 0, me);
    } else {
      for (i = 1; i < tasks; i++) {

        status = comm.probe(MPI.ANY_SOURCE, MPI.ANY_TAG);

        src = status.getSource();
        tag = status.getTag();
        if (src != tag)
          System.out.println("ERROR in MPI_Probe(5): tag = " + tag + ", should be " + src);

        cnt = status.getCount(MPI.BYTE);
        if (cnt != 4)
          System.out.println("ERROR in MPI_Probe(5): cnt = " + cnt + ", should be 4 bytes");
        status.free();
        System.out.println("src: " + src + ", tag: " + tag + ", cnt: " + cnt);

        status = comm.recv(bufferData, 4, MPI.BYTE, src, tag);

        System.out.println(
            "src: "
                + status.getSource()
                + ", tag: "
                + status.getTag()
                + ", cnt: "
                + status.getCount(MPI.BYTE));

        if (bufferData.getInt() != src)
          System.out.println("ERROR in MPI_Recv(5), data = " + data + ", should be " + src);

        if (status.getTag() != tag)
          System.out.println(
              "ERROR in MPI_Recv(5), status.tag = " + status.getTag() + ", should be " + tag);

        if (status.getSource() != src)
          System.out.println(
              "ERROR in MPI_Recv(5), status.source = " + status.getSource() + ", should be " + src);

        if (status.getCount(MPI.BYTE) != 4)
          System.out.println(
              "ERROR in MPI_Recv(5), status.getCount(MPI.BYTE) = "
                  + status.getCount(MPI.BYTE)
                  + ", should be "
                  + cnt);

        status.free();
        bufferData.clear();
      }
    }

    comm.barrier();
    if (me == 0) System.out.println("Probe TEST COMPLETE\n");
    MPI.Finalize();
  }
}
