/* Copyright (c) 2001-2022, The Ohio State University. All rights
 * reserved.
 *
 * This file is part of the MVAPICH2-J software package developed by the
 * team members of The Ohio State University's Network-Based Computing
 * Laboratory (NBCL), headed by Professor Dhabaleswar K. (DK) Panda.
 *
 * For detailed copyright and licensing information, please refer to the
 * copyright file COPYRIGHT in the top level MVAPICH2-J directory.
 */

package mpi.pt2pt;

import mpi.*;

import java.util.Arrays;

public class req_test {

  public static void main(String args[]) throws Exception {

    MPI.Init(args);
    int intArray[] = new int[100];
    int intReadArray[] = new int[100];
    mpi.Request req = null;
    mpi.Status status = null;

    for (int i = 0; i < intArray.length; i++) {
      intArray[i] = i + 1;
      intReadArray[i] = 3;
    }

    int rank = MPI.COMM_WORLD.getRank();

    if (rank == 0) {
      req = MPI.COMM_WORLD.iSend(intArray, 100, MPI.INT, 1, 999);
      status = req.waitStatus();
    } else if (rank == 1) {
      try {
        Thread.currentThread().sleep(1000);
      } catch (Exception e) {
      }
      req = MPI.COMM_WORLD.iRecv(intReadArray, 100, MPI.INT, 0, 999);
      status = req.waitStatus();

      if (Arrays.equals(intArray, intReadArray)) {
        System.out.println("#################" + "\n <<<<PASSED>>>> " + "\n################");
      } else {
        System.out.println("#################" + "\n <<<<FAILED>>>> " + "\n################");
      }

      System.out.println("(r)status.index " + status.index);
      System.out.println("(r)status.source " + status.getSource());
      System.out.println("(r)status.tag " + status.getTag());
    }

    status.free();

    if (rank == 0) {
      req = MPI.COMM_WORLD.iSend(intArray, 100, MPI.INT, 1, 999);
      status = req.waitStatus();
    } else if (rank == 1) {
      try {
        Thread.currentThread().sleep(1000);
      } catch (Exception e) {
      }
      req = MPI.COMM_WORLD.iRecv(intReadArray, 100, MPI.INT, 0, 999);
      status = req.waitStatus();

      if (Arrays.equals(intArray, intReadArray)) {
        System.out.println("#################" + "\n <<<<PASSED>>>> " + "\n################");
      } else {
        System.out.println("#################" + "\n <<<<FAILED>>>> " + "\n################");
      }
      System.out.println("(r)status.index " + status.index);
      System.out.println("(r)status.source " + status.getSource());
      System.out.println("(r)status.tag " + status.getTag());
    }

    status.free();

    MPI.COMM_WORLD.barrier();
    MPI.Finalize();
  }
}
