/* Copyright (c) 2001-2022, The Ohio State University. All rights
 * reserved.
 *
 * This file is part of the MVAPICH2-J software package developed by the
 * team members of The Ohio State University's Network-Based Computing
 * Laboratory (NBCL), headed by Professor Dhabaleswar K. (DK) Panda.
 *
 * For detailed copyright and licensing information, please refer to the
 * copyright file COPYRIGHT in the top level MVAPICH2-J directory.
 */

package mpi.pt2pt;

/****************************************************************************
 *
 * MESSAGE PASSING INTERFACE TEST CASE SUITE
 *
 * Copyright IBM Corp. 1995
 *
 * IBM Corp. hereby grants a non-exclusive license to use, copy, modify, and
 * distribute this software for any purpose and without fee provided that the
 * above copyright notice and the following paragraphs appear in all copies.
 *
 * IBM Corp. makes no representation that the test cases comprising this
 * suite are correct or are an accurate representation of any standard.
 *
 * In no event shall IBM be liable to any party for direct, indirect, special
 * incidental, or consequential damage arising out of the use of this software
 * even if IBM Corp. has been advised of the possibility of such damage.
 *
 * IBM CORP. SPECIFICALLY DISCLAIMS ANY WARRANTIES INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE.  THE SOFTWARE PROVIDED HEREUNDER IS ON AN "AS IS" BASIS AND IBM
 * CORP. HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
 * ENHANCEMENTS, OR MODIFICATIONS.
 ****************************************************************************
 *
 * These test cases reflect an interpretation of the MPI Standard.  They are
 * are, in most cases, unit tests of specific MPI behaviors.  If a user of any
 * test case from this set believes that the MPI Standard requires behavior
 * different than that implied by the test case we would appreciate feedback.
 *
 * Comments may be sent to:
 * Richard Treumann
 * treumann@kgn.ibm.com
 ****************************************************************************
 *
 * MPI-Java version :
 * Sung-Hoon Ko(shko@npac.syr.edu)
 * Northeast Parallel Architectures Center at Syracuse University
 * 03/22/98
 ****************************************************************************
 */

import mpi.*;

import java.nio.ByteBuffer;

public class sendRecvRepWithByteBuffer {

  public sendRecvRepWithByteBuffer() {}

  public sendRecvRepWithByteBuffer(String[] args) throws Exception {
    int src, dest, sendtag, recvtag, tasks, me, i;
    Status status;

    MPI.Init(args);
    tasks = MPI.COMM_WORLD.getSize();
    me = MPI.COMM_WORLD.getRank();

    ByteBuffer sendbuf = ByteBuffer.allocateDirect(1000);

    if (me < 2) {
      src = dest = 1 - me;
      sendtag = me;
      recvtag = src;

      for (i = 0; i < 100; i++) {
        sendbuf.put((byte)1);
      }

      status =
          MPI.COMM_WORLD.sendRecvReplace(
              sendbuf, 100, MPI.BYTE, dest, sendtag, src, recvtag);

      sendbuf.flip();

      for (i = 0; i < 2000000; i++)
        ;

      for (i = 0; i < 100; i++)
        if (sendbuf.get() != (byte)1) {
          System.out.println("ERROR in MPI.sendRecvReplace: incorrect data\n");
          break;
        }

      if (status.getSource() != src)
        System.out.println("ERROR in MPI.sendRecvReplace: incorrect source\n");
      if (status.getTag() != recvtag)
        System.out.println("ERROR in MPI.sendRecvReplace: incorrect tag (" + status.getTag() + ")");
    }

    sendbuf.clear();

    src = (me == 0) ? tasks - 1 : me - 1;
    dest = (me == tasks - 1) ? 0 : me + 1;
    sendtag = me;
    recvtag = src;
    for (i = 0; i < 100; i++) {
      sendbuf.put((byte)1);
    }

    status =
        MPI.COMM_WORLD.sendRecvReplace(
            sendbuf, 100, MPI.BYTE, dest, sendtag, src, recvtag);

    sendbuf.flip();

    for (i = 0; i < 2000000; i++)
      ;
    for (i = 0; i < 100; i++)
      if (sendbuf.get(i) != (byte)1) {
        System.out.println("ERROR in MPI.sendRecvReplace: incorrect data\n");
        break;
      }

    if (status.getSource() != src) System.out.println("ERROR in MPI.sendRecvReplace: incorrect source\n");
    if (status.getTag() != recvtag)
      System.out.println("ERROR in MPI.sendRecvReplace: incorrect tag (" + status.getTag() + ")");

    MPI.COMM_WORLD.barrier();
    if (me == 1) System.out.println("SendRecv TEST COMPLETE");
    MPI.Finalize();
  }

  public static void main(String[] args) throws Exception {
    try {
      sendRecvRepWithByteBuffer c = new sendRecvRepWithByteBuffer(args);
    } catch (Exception e) {
    }
  }
}
