/* Copyright (c) 2001-2022, The Ohio State University. All rights
 * reserved.
 *
 * This file is part of the MVAPICH2-J software package developed by the
 * team members of The Ohio State University's Network-Based Computing
 * Laboratory (NBCL), headed by Professor Dhabaleswar K. (DK) Panda.
 *
 * For detailed copyright and licensing information, please refer to the
 * copyright file COPYRIGHT in the top level MVAPICH2-J directory.
 */

package mpi.pt2pt;

/****************************************************************************
 *
 * MESSAGE PASSING INTERFACE TEST CASE SUITE
 *
 * Copyright IBM Corp. 1995
 *
 * IBM Corp. hereby grants a non-exclusive license to use, copy, modify, and
 * distribute this software for any purpose and without fee provided that the
 * above copyright notice and the following paragraphs appear in all copies.
 *
 * IBM Corp. makes no representation that the test cases comprising this
 * suite are correct or are an accurate representation of any standard.
 *
 * In no event shall IBM be liable to any party for direct, indirect, special
 * incidental, or consequential damage arising out of the use of this software
 * even if IBM Corp. has been advised of the possibility of such damage.
 *
 * IBM CORP. SPECIFICALLY DISCLAIMS ANY WARRANTIES INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE.  THE SOFTWARE PROVIDED HEREUNDER IS ON AN "AS IS" BASIS AND IBM
 * CORP. HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
 * ENHANCEMENTS, OR MODIFICATIONS.
 ****************************************************************************
 *
 * These test cases reflect an interpretation of the MPI Standard.  They are
 * are, in most cases, unit tests of specific MPI behaviors.  If a user of any
 * test case from this set believes that the MPI Standard requires behavior
 * different than that implied by the test case we would appreciate feedback.
 *
 * Comments may be sent to:
 * Richard Treumann
 * treumann@kgn.ibm.com
 ****************************************************************************
 *
 * MPI-Java version :
 * Sung-Hoon Ko(shko@npac.syr.edu)
 * Northeast Parallel Architectures Center at Syracuse University
 * 09/10/99
 ****************************************************************************
 */

import mpi.*;

public class sendrecv_rep {
  public sendrecv_rep() {}

  public sendrecv_rep(String[] args) throws Exception {

    int src, dest, sendtag, recvtag, tasks, me, i;
    Status s1 = null, s2 = null, s3 = null, s4 = null, s5 = null, s6 = null, s7 = null, s8 = null;
    
    int bufInt[] = new int[1000];
    float bufFloat[] = new float[1000];
    double bufDouble[] = new double[1000];
    long bufLong[] = new long[1000];
    short bufShort[] = new short[1000];
    boolean bufBoolean[] = new boolean[1000];
    char bufChar[] = new char[1000];
    byte bufByte[] = new byte[1000];
    
    MPI.Init(args);
    tasks = MPI.COMM_WORLD.getSize();
    me = MPI.COMM_WORLD.getRank();

    if (me < 2) {
      src = dest = 1 - me;
      sendtag = me;
      recvtag = src;
      for (i = 0; i < 100; i++) {
        bufInt[i] = 1;
        bufFloat[i] = (float)1;
        bufDouble[i] = (double)1;
        bufLong[i] = (long)1;
        bufShort[i] = (short)1;
        bufBoolean[i] = true;
        bufChar[i] = 'a';
        bufByte[i] = (byte)1;
      }

      s1 = MPI.COMM_WORLD.sendRecvReplace(bufInt, 100, MPI.INT, dest, sendtag, src, recvtag);
      s2 = MPI.COMM_WORLD.sendRecvReplace(bufDouble, 100, MPI.DOUBLE, dest, sendtag, src, recvtag);
      s3 = MPI.COMM_WORLD.sendRecvReplace(bufFloat, 100, MPI.FLOAT, dest, sendtag, src, recvtag);
      s4 = MPI.COMM_WORLD.sendRecvReplace(bufLong, 100, MPI.LONG, dest, sendtag, src, recvtag);
      s5 = MPI.COMM_WORLD.sendRecvReplace(bufShort, 100, MPI.SHORT, dest, sendtag, src, recvtag);
      s6 = MPI.COMM_WORLD.sendRecvReplace(bufBoolean, 100, MPI.BOOLEAN, dest, sendtag, src, recvtag);
      s7 = MPI.COMM_WORLD.sendRecvReplace(bufChar, 100, MPI.CHAR, dest, sendtag, src, recvtag);
      s8 = MPI.COMM_WORLD.sendRecvReplace(bufByte, 100, MPI.BYTE, dest, sendtag, src, recvtag);

      for (i = 0; i < 100; i++) {
        int valInt = bufInt[i];
        double valDouble = bufDouble[i];
        long valLong = bufLong[i];
        float valFloat = bufFloat[i];
        short valShort = bufShort[i];
        byte valByte = bufByte[i];
        char valChar = bufChar[i];
        boolean valBoolean = bufBoolean[i]; 
      
        if (valInt != 1) {
          System.out.println("INT: ERROR in MPI.Sendrecv: incorrect data\n");
          break;
        }
        if (valDouble != (double)1) {
          System.out.println("DBL: ERROR in MPI.Sendrecv: incorrect data\n");
          break;
        }
        if (valFloat != (float)1) {
          System.out.println("FLOAT: ERROR in MPI.Sendrecv: incorrect data\n");
          break;
        }
        if (valLong != (long)1) {
          System.out.println("LONG: ERROR in MPI.Sendrecv: incorrect data\n");
          break;
        }
        if (valShort != (short)1) {
          System.out.println("SHORT: ERROR in MPI.Sendrecv: incorrect data\n");
          break;
        }
        if (valByte != (byte)1) {
          System.out.println("BYTE: ERROR in MPI.Sendrecv: incorrect data\n");
          break;
        }
        if (valChar != 'a') {
          System.out.println("CHAR: ERROR in MPI.Sendrecv: incorrect data\n");
          break;
        }
        if (valBoolean != true) {
          System.out.println("BOOL: ERROR in MPI.Sendrecv: incorrect data\n");
          break;
        } 
      }

      if (s1.getSource() != src) {
        System.out.println("ERROR in MPI.Sendrecv: incorrect source\n");
      }
      if (s1.getTag() != recvtag) {
        System.out.println("ERROR in MPI.Sendrecv: incorrect tag (" + s1.getTag() + ")");
      }
      if (s2.getSource() != src) {
        System.out.println("ERROR in MPI.Sendrecv: incorrect source\n");
      }
      if (s2.getTag() != recvtag) {
        System.out.println("ERROR in MPI.Sendrecv: incorrect tag (" + s2.getTag() + ")");
      }
      if (s3.getSource() != src) {
        System.out.println("ERROR in MPI.Sendrecv: incorrect source\n");
      }
      if (s3.getTag() != recvtag) {
        System.out.println("ERROR in MPI.Sendrecv: incorrect tag (" + s3.getTag() + ")");
      }
      if (s4.getSource() != src) {
        System.out.println("ERROR in MPI.Sendrecv: incorrect source\n");
      }
      if (s4.getTag() != recvtag) {
        System.out.println("ERROR in MPI.Sendrecv: incorrect tag (" + s4.getTag() + ")");
      }
      if (s5.getSource() != src) {
        System.out.println("ERROR in MPI.Sendrecv: incorrect source\n");
      }
      if (s5.getTag() != recvtag) {
        System.out.println("ERROR in MPI.Sendrecv: incorrect tag (" + s5.getTag() + ")");
      }
      if (s6.getSource() != src) {
        System.out.println("ERROR in MPI.Sendrecv: incorrect source\n");
      }
      if (s6.getTag() != recvtag) {
        System.out.println("ERROR in MPI.Sendrecv: incorrect tag (" + s6.getTag() + ")");
      }
      if (s7.getSource() != src) {
        System.out.println("ERROR in MPI.Sendrecv: incorrect source\n");
      }
      if (s7.getTag() != recvtag) {
        System.out.println("ERROR in MPI.Sendrecv: incorrect tag (" + s7.getTag() + ")");
      }
      if (s8.getSource() != src) {
        System.out.println("ERROR in MPI.Sendrecv: incorrect source\n");
      }
      if (s8.getTag() != recvtag) {
        System.out.println("ERROR in MPI.Sendrecv: incorrect tag (" + s8.getTag() + ")");
      }

      s1.free();
      s2.free();
      s3.free();
      s4.free();
      s5.free();
      s6.free();
      s7.free();
      s8.free();
    }

    src = (me == 0) ? tasks - 1 : me - 1;
    dest = (me == tasks - 1) ? 0 : me + 1;
    sendtag = me;
    recvtag = src;
    for (i = 0; i < 100; i++) {
      bufInt[i] = 1;
      bufFloat[i] = (float)1;
      bufDouble[i] = (double)1;
      bufLong[i] = (long)1;
      bufShort[i] = (short)1;
      bufBoolean[i] = true;
      bufChar[i] = 'a';
      bufByte[i] = (byte)1;
    }

    s1 = MPI.COMM_WORLD.sendRecvReplace(bufInt, 100, MPI.INT, dest, sendtag, src, recvtag);
    s2 = MPI.COMM_WORLD.sendRecvReplace(bufDouble, 100, MPI.DOUBLE, dest, sendtag, src, recvtag);
    s3 = MPI.COMM_WORLD.sendRecvReplace(bufFloat, 100, MPI.FLOAT, dest, sendtag, src, recvtag);
    s4 = MPI.COMM_WORLD.sendRecvReplace(bufLong, 100, MPI.LONG, dest, sendtag, src, recvtag);
    s5 = MPI.COMM_WORLD.sendRecvReplace(bufShort, 100, MPI.SHORT, dest, sendtag, src, recvtag);
    s6 = MPI.COMM_WORLD.sendRecvReplace(bufBoolean, 100, MPI.BOOLEAN, dest, sendtag, src, recvtag);
    s7 = MPI.COMM_WORLD.sendRecvReplace(bufChar, 100, MPI.CHAR, dest, sendtag, src, recvtag);
    s8 = MPI.COMM_WORLD.sendRecvReplace(bufByte, 100, MPI.BYTE, dest, sendtag, src, recvtag);

    for (i = 0; i < 100; i++) {
      int valInt = bufInt[i];
      double valDouble = bufDouble[i];
      long valLong = bufLong[i];
      float valFloat = bufFloat[i];
      short valShort = bufShort[i];
      byte valByte = bufByte[i];
      char valChar = bufChar[i];
      boolean valBoolean = bufBoolean[i];

      if (valInt != 1) {
        System.out.println("INT: ERROR in MPI.Sendrecv: incorrect data\n");
        break;
      }
      if (valDouble != (double)1) {
        System.out.println("DBL: ERROR in MPI.Sendrecv: incorrect data\n");
        break;
      }
      if (valFloat != (float)1) {
        System.out.println("FLOAT: ERROR in MPI.Sendrecv: incorrect data\n");
        break;
      }
      if (valLong != (long)1) {
        System.out.println("LONG: ERROR in MPI.Sendrecv: incorrect data\n");
        break;
      }
      if (valShort != (short)1) {
        System.out.println("SHORT: ERROR in MPI.Sendrecv: incorrect data\n");
        break;
      }
      if (valByte != (byte)1) {
        System.out.println("BYTE: ERROR in MPI.Sendrecv: incorrect data\n");
        break;
      }
      if (valChar != 'a') {
        System.out.println("CHAR: ERROR in MPI.Sendrecv: incorrect data\n");
        break;
      }
      if (valBoolean != true) {
        System.out.println("BOOL: ERROR in MPI.Sendrecv: incorrect data\n");
        break;
      }
    }

    if (s1.getSource() != src) {
        System.out.println("ERROR in MPI.Sendrecv: incorrect source\n");
    }
    if (s1.getTag() != recvtag) {
      System.out.println("ERROR in MPI.Sendrecv: incorrect tag (" + s1.getTag() + ")");
    }
    if (s2.getSource() != src) {
      System.out.println("ERROR in MPI.Sendrecv: incorrect source\n");
    }
    if (s2.getTag() != recvtag) {
      System.out.println("ERROR in MPI.Sendrecv: incorrect tag (" + s2.getTag() + ")");
    }
    if (s3.getSource() != src) {
      System.out.println("ERROR in MPI.Sendrecv: incorrect source\n");
    }
    if (s3.getTag() != recvtag) {
      System.out.println("ERROR in MPI.Sendrecv: incorrect tag (" + s3.getTag() + ")");
    }
    if (s4.getSource() != src) {
      System.out.println("ERROR in MPI.Sendrecv: incorrect source\n");
    }
    if (s4.getTag() != recvtag) {
      System.out.println("ERROR in MPI.Sendrecv: incorrect tag (" + s4.getTag() + ")");
    }
    if (s5.getSource() != src) {
      System.out.println("ERROR in MPI.Sendrecv: incorrect source\n");
    }
    if (s5.getTag() != recvtag) {
      System.out.println("ERROR in MPI.Sendrecv: incorrect tag (" + s5.getTag() + ")");
    }
    if (s6.getSource() != src) {
      System.out.println("ERROR in MPI.Sendrecv: incorrect source\n");
    }
    if (s6.getTag() != recvtag) {
      System.out.println("ERROR in MPI.Sendrecv: incorrect tag (" + s6.getTag() + ")");
    }
    if (s7.getSource() != src) {
      System.out.println("ERROR in MPI.Sendrecv: incorrect source\n");
    }
    if (s7.getTag() != recvtag) {
      System.out.println("ERROR in MPI.Sendrecv: incorrect tag (" + s7.getTag() + ")");
    }
    if (s8.getSource() != src) {
      System.out.println("ERROR in MPI.Sendrecv: incorrect source\n");
    }
    if (s8.getTag() != recvtag) {
      System.out.println("ERROR in MPI.Sendrecv: incorrect tag (" + s8.getTag() + ")");
    }
      
    s1.free();
    s2.free();
    s3.free();
    s4.free();
    s5.free();
    s6.free();
    s7.free();
    s8.free();

    MPI.COMM_WORLD.barrier();

    if (me == 1) System.out.println("Sendrecv_rep TEST COMPLETE");

    MPI.Finalize();
  }

  public static void main(String[] args) throws Exception {
    try {
      sendrecv_rep c = new sendrecv_rep(args);
    } catch (Exception e) {
    }
  }
}
