/* Copyright (c) 2001-2022, The Ohio State University. All rights
 * reserved.
 *
 * This file is part of the MVAPICH2-J software package developed by the
 * team members of The Ohio State University's Network-Based Computing
 * Laboratory (NBCL), headed by Professor Dhabaleswar K. (DK) Panda.
 *
 * For detailed copyright and licensing information, please refer to the
 * copyright file COPYRIGHT in the top level MVAPICH2-J directory.
 */

package mpi.pt2pt;

/****************************************************************************
 *
 * MESSAGE PASSING INTERFACE TEST CASE SUITE
 *
 * Copyright IBM Corp. 1995
 *
 * IBM Corp. hereby grants a non-exclusive license to use, copy, modify, and
 * distribute this software for any purpose and without fee provided that the
 * above copyright notice and the following paragraphs appear in all copies.
 *
 * IBM Corp. makes no representation that the test cases comprising this
 * suite are correct or are an accurate representation of any standard.
 *
 * In no event shall IBM be liable to any party for direct, indirect, special
 * incidental, or consequential damage arising out of the use of this software
 * even if IBM Corp. has been advised of the possibility of such damage.
 *
 * IBM CORP. SPECIFICALLY DISCLAIMS ANY WARRANTIES INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE.  THE SOFTWARE PROVIDED HEREUNDER IS ON AN "AS IS" BASIS AND IBM
 * CORP. HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
 * ENHANCEMENTS, OR MODIFICATIONS.
 ****************************************************************************
 *
 * These test cases reflect an interpretation of the MPI Standard.  They are
 * are, in most cases, unit tests of specific MPI behaviors.  If a user of any
 * test case from this set believes that the MPI Standard requires behavior
 * different than that implied by the test case we would appreciate feedback.
 *
 * Comments may be sent to:
 * Richard Treumann
 * treumann@kgn.ibm.com
 ****************************************************************************
 *
 * MPI-Java version :
 * Sung-Hoon Ko(shko@npac.syr.edu)
 * Northeast Parallel Architectures Center at Syracuse University
 * 03/22/98
 ****************************************************************************
 */
// FIXME: Aamir: indices

import mpi.*;

public class waitall {
  public waitall() {}

  public waitall(String[] args) throws Exception {

    int me, tasks, bytes, i;
    int mebuf[] = new int[1];

    MPI.Init(args);
    me = MPI.COMM_WORLD.getRank();
    tasks = MPI.COMM_WORLD.getSize();

    int data[] = new int[tasks];

    Request req[] = new Request[2 * tasks];
    Status stats[] = new Status[2 * tasks];

    mebuf[0] = me;

    for (i = 0; i < tasks; i++) {
      if (i != me) req[2 * i] = MPI.COMM_WORLD.iSend(mebuf, 1, MPI.INT, i, 1);
      else req[2 * i] = MPI.REQUEST_NULL;
      // Original IBM code used `Irsend' here. Clearly wrong? dbc.

      if (i != me) req[2 * i + 1] = MPI.COMM_WORLD.iRecv(data, 1, MPI.INT, i, 1);
      else req[2 * i + 1] = MPI.REQUEST_NULL;
    }

    stats = Request.waitAllStatus(req);

    if (me == 0) {
      for (i = 0; i < tasks; i++) {
        Status s1 = stats[2 * i + 1];

        int retCt = s1.getCount(MPI.INT);
        int retEls = s1.getElements(MPI.INT);
        int retSrc = s1.getSource();
        int retTag = s1.getTag();

        if (i == me) {
          if (retSrc != -2 || retTag != -1 || retEls != 0 || retCt != 0) {
            System.out.println(
                "Incorrect values for s1: "
                    + "s1.source = <"
                    + s1.getSource()
                    + ">, "
                    + "s1.tag    = <"
                    + s1.getTag()
                    + ">, "
                    + "s1.numEls = <"
                    + retEls
                    + ">, "
                    + "s1.count  = <"
                    + retCt
                    + ">, "
                    + "s1.numEls = <"
                    + s1.numEls
                    + ">, "
                    + "s1.count  = <"
                    + s1.count
                    + ">");
          }

        } else {
          if (retSrc != i || retTag != 1 || retEls != 1 || retCt != 1) {
            System.out.println(
                "Incorrect values for s1: "
                    + "s1.source = <"
                    + s1.getSource()
                    + ">, "
                    + "s1.tag    = <"
                    + s1.getTag()
                    + ">, "
                    + "s1.numEls = <"
                    + retEls
                    + ">, "
                    + "s1.count  = <"
                    + retCt
                    + ">, "
                    + "s1.numEls = <"
                    + s1.numEls
                    + ">, "
                    + "s1.count  = <"
                    + s1.count
                    + ">");
          }
        }

        s1.free();
      }
    }

    MPI.COMM_WORLD.barrier();

    if (me == 0) System.out.println("Waitall TEST COMPLETE");

    MPI.Finalize();
  }

  public static void main(String[] args) throws Exception {
    try {
      waitall c = new waitall(args);
    } catch (Exception e) {
    }
  }
}
