/* Copyright (c) 2001-2022, The Ohio State University. All rights
 * reserved.
 *
 * This file is part of the MVAPICH2-J software package developed by the
 * team members of The Ohio State University's Network-Based Computing
 * Laboratory (NBCL), headed by Professor Dhabaleswar K. (DK) Panda.
 *
 * For detailed copyright and licensing information, please refer to the
 * copyright file COPYRIGHT in the top level MVAPICH2-J directory.
 */

package mpi.pt2pt;

import mpi.*;

import java.util.Arrays;

public class waitall2 {
  public waitall2() {}

  public waitall2(String args[]) throws Exception {
    MPI.Init(args);
    int me = MPI.COMM_WORLD.getRank();

    mpi.Request r[] = new Request[10];
    mpi.Status s[] = new Status[10];
    r[0] = MPI.REQUEST_NULL;
    r[9] = MPI.REQUEST_NULL;

    int intArray[] = new int[100];
    float floatArray[] = new float[100];
    double doubleArray[] = new double[100];
    long longArray[] = new long[100];
    boolean booleanArray[] = new boolean[100];
    short shortArray[] = new short[100];
    char charArray[] = new char[100];
    byte byteArray[] = new byte[100];

    int intReadArray[] = new int[100];
    float floatReadArray[] = new float[100];
    double doubleReadArray[] = new double[100];
    long longReadArray[] = new long[100];
    boolean booleanReadArray[] = new boolean[100];
    short shortReadArray[] = new short[100];
    char charReadArray[] = new char[100];
    byte byteReadArray[] = new byte[100];

    for (int i = 0; i < intArray.length; i++) {
      intArray[i] = i + 1;
      floatArray[i] = i + 11;
      doubleArray[i] = i + 11.11;
      longArray[i] = i + 11;
      booleanArray[i] = true;
      shortArray[i] = 1;
      charArray[i] = 's';
      byteArray[i] = 's';

      intReadArray[i] = 3;
      floatReadArray[i] = i + 19;
      doubleReadArray[i] = i + 99.11;
      longReadArray[i] = i + 9;
      shortReadArray[i] = 2;
      booleanReadArray[i] = false;
      charReadArray[i] = 'x';
      byteReadArray[i] = 'x';
    }

    if (MPI.COMM_WORLD.getRank() == 0) {

      r[1] = MPI.COMM_WORLD.iSend(intArray, 100, MPI.INT, 1, 1);
      r[2] = MPI.COMM_WORLD.iSend(byteArray, 100, MPI.BYTE, 1, 2);
      s[1] = r[1].waitStatus();
      s[2] = r[2].waitStatus();
      s[1].free();
      s[2].free();

    } else if (MPI.COMM_WORLD.getRank() == 1) {
      r[1] = MPI.COMM_WORLD.iRecv(intReadArray, 100, MPI.INT, 0, 1);
      r[2] = MPI.COMM_WORLD.iRecv(byteReadArray, 100, MPI.BYTE, 0, 2);
      s[1] = r[1].waitStatus();
      s[2] = r[2].waitStatus();
      s[1].free();
      s[2].free();

      if (Arrays.equals(intArray, intReadArray)) {
        System.out.println("waitall2 TEST Completed");

        System.out.println("\n#################" + "\n <<<<PASSED>>>> " + "\n################");

      } else {
        System.out.println("\n#################" + "\n <<<<FAILED>>>> " + "\n################");
      }
    }

    MPI.COMM_WORLD.barrier();
    MPI.Finalize();
  }

  public static void main(String args[]) throws Exception {
    waitall2 test = new waitall2(args);
  }
}
