/* Copyright (c) 2001-2022, The Ohio State University. All rights
 * reserved.
 *
 * This file is part of the MVAPICH2-J software package developed by the
 * team members of The Ohio State University's Network-Based Computing
 * Laboratory (NBCL), headed by Professor Dhabaleswar K. (DK) Panda.
 *
 * For detailed copyright and licensing information, please refer to the
 * copyright file COPYRIGHT in the top level MVAPICH2-J directory.
 */

package mpi.signals;

import java.io.*;

/** Simple implementation of the class task lesseee */
public class Job implements Serializable {

  /** This value holds the input task. */
  private Task input;

  /** This values holds the result of the input task evaluation. */
  private Result output;

  /**
   * A Job is instantiated by specifying the input task, expected result and their fully qualified
   * class names.
   */
  public Job(Task task_input, Result result) {
    input = task_input;
    output = result;
  }

  /** returns the input task */
  public Task getInput() {
    return input;
  }

  /** returns the result of the evaluation. This method blocks if the evaluation is not complete. */
  public Object getOutput() {
    return output;
  }

  /** sets the result of the evaluation */
  public void setOutput(Result job_output) {
    output = job_output;
  }

  /** Returns a string representation of the job */
  public String toString() {

    return "Input is : " + input + "\n Output is : " + output + "\n";
  }

  /** For garbage collection */
  protected void finalize() throws Throwable {
    input = null;
    output = null;
    super.finalize();
  }
}
