/* Copyright (c) 2001-2022, The Ohio State University. All rights
 * reserved.
 *
 * This file is part of the MVAPICH2-J software package developed by the
 * team members of The Ohio State University's Network-Based Computing
 * Laboratory (NBCL), headed by Professor Dhabaleswar K. (DK) Panda.
 *
 * For detailed copyright and licensing information, please refer to the
 * copyright file COPYRIGHT in the top level MVAPICH2-J directory.
 */

package mpi.signals;

import java.util.*;

import mpi.*;

import java.io.*;

// Test case contributed by Sivakumar Venkata Pabolu.

/** Test program to emulate Pool of tasks.... need to get back to this soon. */
public class SendTest_Conv2 {

  public SendTest_Conv2() {}

  public SendTest_Conv2(String[] args) throws Exception {

    MPI.Init(args);

    int my_pe = MPI.COMM_WORLD.getRank();
    int npes = MPI.COMM_WORLD.getSize();

    if (npes != 2) {
      if (my_pe == 0) System.out.println("signals->SendTest_Conv2 runs with 2 tasks");

      MPI.COMM_WORLD.barrier();
      MPI.Finalize();
      return;
    }

    if (my_pe == 0) {
      String proc_name = "mpiJava"; // Modified, dbc
      SendTest_Conv2.bugTestRank0(proc_name, npes, 0);
    } else {
      SendTest_Conv2.bugTestRank12(0, my_pe);
    }

    MPI.COMM_WORLD.barrier();
    if (my_pe == 0) {
      System.out.println("############################");
      System.out.println("Starting next test");
      System.out.println("############################");
      String proc_name = "mpiJava"; // Modified, dbc
      SendTest_Conv2.bugTestRank0(proc_name, npes, 1);
    } else {
      SendTest_Conv2.bugTestRank12(1, my_pe);
    }

    MPI.Finalize();
  }

  public static void bugTestRank0(String proc_name, int npes, int flag) {

    Object[] t = new Object[2];

    for (int i = 0; i < 2; i++) {

      if (flag == 0) t[i] = new MyLinkList(1000, proc_name); // Modified, dbc.
      else t[i] = new MyLinkList1(1000, proc_name); // Modified, dbc.

      System.out.println("Created Task[" + i + "] : \n " + t[i]);
    }

    for (int i = 1; i < npes; i++) {
      System.out.println("Sending Task[" + (i - 1) + "]: \n " + t[i - 1]);

      SendTest_Conv.Send(t[i - 1], i, 0);
    }

    Object[] t2 = new Object[2];

    for (int i = 1; i < npes; i++) {
      System.out.println("waiting to hear from proc " + (i - 1));

      if (flag == 0) {
        t2[i - 1] = (MyLinkList) SendTest_Conv.Recv(i, 0);
        System.out.println("Recieved : " + (MyLinkList) t2[i - 1]);
      } else {
        t2[i - 1] = (MyLinkList1) SendTest_Conv.Recv(i, 0);
        System.out.println("Recieved : " + (MyLinkList1) t2[i - 1]);
      }
    }

    System.out.println("Recieved Items: ");

    for (int i = 1; i < npes; i++) System.out.println(t2[i - 1]);

    System.out.println("Exiting....");
  }

  public static void bugTestRank12(int flag, int my_pe) {

    Object[] inbuf = new Object[1];

    System.out.println("before Recv on proc no " + my_pe);

    try {

      inbuf[0] = SendTest_Conv.Recv(0, 0);

      if (flag == 0) System.out.println("Recieved : " + (MyLinkList) inbuf[0] + " on " + my_pe);
      else System.out.println("Recieved : " + (MyLinkList1) inbuf[0] + " on " + my_pe);
    } catch (Exception e) {
      System.err.println(e);
    }

    SendTest_Conv.Send(inbuf[0], 0, 0);
  }

  public static void main(String[] args) throws MPIException {}
}
