/* Copyright (c) 2001-2022, The Ohio State University. All rights
 * reserved.
 *
 * This file is part of the MVAPICH2-J software package developed by the
 * team members of The Ohio State University's Network-Based Computing
 * Laboratory (NBCL), headed by Professor Dhabaleswar K. (DK) Panda.
 *
 * For detailed copyright and licensing information, please refer to the
 * copyright file COPYRIGHT in the top level MVAPICH2-J directory.
 */

import mpi.Intercomm;
import mpi.MPI;
import mpi.MPIException;

/* mvapich2/test/mpi/spawn/spawn1.c java port without MTestFinalize */
public class Spawn1 {
  public static void main(String[] args) throws Exception {
    int i = 0;
    int errs = 0;

    int rank = 0;
    int size = 0;
    int rsize = 0;
    int np = 2;
    int[] errcodes = new int[2];

    // boolean canSpawn = false;
    boolean canSpawn = true;

    Intercomm parentComm;
    Intercomm interComm;

    MPI.Init(args);

    System.out.println("comm_world size: " + MPI.COMM_WORLD.getSize());

    String[] spawn_argv = {
      "-cp",
      "/home/alattar.2/mvapich2-j/lib/mvapich2-j.jar:.",
      "-Djava.library.path=/home/alattar.2/mvapich2-j/lib",
      Spawn1.class.getName()
    };

    if (canSpawn) {
      parentComm = Intercomm.getParent();

      if (parentComm.isNull()) {
        System.out.println("about to call spawn");
        // Create 2 more procs
        interComm = MPI.COMM_WORLD.spawn("java", spawn_argv, np, MPI.INFO_NULL, 0, errcodes);

        System.out.println("called spawn successfully");

      } else {
        interComm = parentComm;
      }

      // we now have a valid intercomm
      rsize = interComm.getRemoteSize();
      System.out.println("remote size: " + rsize);
      size = interComm.getSize();
      System.out.println("size: " + size);
      rank = interComm.getRank();
      System.out.println("rank: " + rank);

      if (parentComm.isNull()) {
        // Master
        if (rsize != np) {
          errs++;
          System.out.println("Did not create " + np + " processes (got " + rsize + ")");
        }

        if (rank == 0) {
          for (i = 0; i < rsize; i++) {
            int[] sarr = {i};
            interComm.send(sarr, 1, MPI.INT, i, 0);
          }
        }

      } else {
        // Child
        if (size != np) {
          errs++;
          System.out.println("(Child) did not create " + np + " processes (got " + size + ")");
        }

        int[] ri = new int[1];
        interComm.recv(ri, 1, MPI.INT, 0, 0);
        System.out.println("interComm.recv received: " + ri[0]);

        if (ri[0] != rank) {
          errs++;
          System.out.println("Unexpected rank on child " + rank + " (" + ri[0] + ")");
        }

        // send the errors back to the master proc
        int[] errsArr = {errs};
      }

      if (parentComm.isNull()) {
        int[] errsArr = {errs};
        int[] toterrs = new int[1];
        try {
          MPI.COMM_WORLD.reduce(errsArr, toterrs, 1, MPI.INT, MPI.SUM, 0);

          if (rank == 0) {
            if (toterrs[0] > 0) {
              System.out.println("Found " + toterrs[0] + " errors");
            } else {
              System.out.println("No Errors");
            }
          }

        } catch (MPIException mpe) {
          mpe.printStackTrace();
        }
      }
    } else {
      System.out.println("Cannot spawn procs");
      int[] errsArr = {errs};
      int[] toterrs = new int[1];
      try {
        MPI.COMM_WORLD.reduce(errsArr, toterrs, 1, MPI.INT, MPI.SUM, 0);

        if (rank == 0) {
          if (toterrs[0] > 0) {
            System.out.println("Found " + toterrs[0] + " errors");
          } else {
            System.out.println("No Errors");
          }
        }

      } catch (MPIException mpe) {
        mpe.printStackTrace();
      }
    }

    MPI.Finalize();
  }
}
