/* Copyright (c) 2001-2022, The Ohio State University. All rights
 * reserved.
 *
 * This file is part of the MVAPICH2-J software package developed by the
 * team members of The Ohio State University's Network-Based Computing
 * Laboratory (NBCL), headed by Professor Dhabaleswar K. (DK) Panda.
 *
 * For detailed copyright and licensing information, please refer to the
 * copyright file COPYRIGHT in the top level MVAPICH2-J directory.
 */

import mpi.Intercomm;
import mpi.MPI;
import mpi.Status;

public class SparkMPI {
  public static void main(String[] args) throws Exception {
    Intercomm interComm = null;
    int[] errcodes = new int[1];

    MPI.Init(args);

    int rank = MPI.COMM_WORLD.getRank();
    int size = MPI.COMM_WORLD.getSize();

    if (Intercomm.getParent().isNull()) {
      // Parent procs
      String[] spawn_argv = {
        "-cp",
        "/home/alattar.2/mvapich2-j/lib/mvapich2-j.jar:.",
        "-Djava.library.path=/home/alattar.2/mvapich2-j/lib",
        SparkMPI.class.getName()
      };

      interComm = MPI.COMM_WORLD.spawn("java", spawn_argv, 1, MPI.INFO_NULL, 0, errcodes);

      int[] ri = new int[1];
      boolean flag1 = true;
      boolean flag2 = true;
      while (flag2) {
        Status s = interComm.iProbe(MPI.ANY_SOURCE, MPI.ANY_TAG);

        if (s != null) {
          Status rs = interComm.recv(ri, s.getCount(MPI.BYTE) + 16, MPI.INT, s.source, s.tag);

          System.out.println(
              "recieved source: "
                  + rs.source
                  + " received tag: "
                  + rs.tag
                  + " s.getCount(MPI.BYTE): "
                  + rs.getCount(MPI.BYTE));
        }
      }
    } else {
      // Spawned proc
      interComm = Intercomm.getParent();

      System.out.println("interComm local group size: " + interComm.getSize());

      int rsize = interComm.getRemoteSize();

      System.out.println("interComm remote group size: " + rsize);

      // Send to each proc in remote comm
      for (int i = 0; i < rsize; i++) {
        int[] si = {i};
        interComm.send(si, 1, MPI.INT, i, 10 + i);
      }
    }

    MPI.Finalize();
  }
}
