/* Copyright (c) 2001-2022, The Ohio State University. All rights
 * reserved.
 *
 * This file is part of the MVAPICH2-J software package developed by the
 * team members of The Ohio State University's Network-Based Computing
 * Laboratory (NBCL), headed by Professor Dhabaleswar K. (DK) Panda.
 *
 * For detailed copyright and licensing information, please refer to the
 * copyright file COPYRIGHT in the top level MVAPICH2-J directory.
 */

package mpi.topo;

import mpi.*;

public class cart_topo {
  public cart_topo() {}

  public cart_topo(String[] args) throws Exception {

    MPI.Init(args);
    Group group = MPI.COMM_WORLD.getGroup();
    int processes = group.getSize();
    int rank = group.getRank();

    if (processes < 8) {
      if (rank == 0) System.out.println("topo->cart_topo: MUST RUN WITH 8 tasks");

      MPI.COMM_WORLD.barrier();
      MPI.Finalize();
      return;
    }

    int dims3[] = new int[3];
    dims3[0] = 2;
    dims3[1] = 2;
    dims3[2] = 2;

    boolean periods3[] = new boolean[3];
    periods3[0] = false;
    periods3[1] = false;
    periods3[2] = false;

    Cartcomm comm2 = MPI.COMM_WORLD.createCart(dims3, periods3, false);
    int[] c1 = new int[3];
    if (rank == 0) System.out.println("\ncoords to ranks (3D)\n");
    for (int i = 0; i < 2; i++) {
      for (int j = 0; j < 2; j++) {
        for (int k = 0; k < 2; k++) {

          c1[0] = i;
          c1[1] = j;
          c1[2] = k;

          if (rank == 0) {
            System.out.println(
                "rank =<" + comm2.getRank(c1) + ">,i=<" + c1[0] + ">=j<" + c1[1] + ">,k=<" + c1[2]);
          }
        }
      }
    }

    MPI.COMM_WORLD.barrier();
    MPI.Finalize();
  } // end main

  public static void main(String[] args) throws Exception {
    try {
      cart_topo c = new cart_topo(args);
    } catch (Exception e) {
    }
  }
}
