/* Copyright (c) 2001-2022, The Ohio State University. All rights
 * reserved.
 *
 * This file is part of the MVAPICH2-J software package developed by the
 * team members of The Ohio State University's Network-Based Computing
 * Laboratory (NBCL), headed by Professor Dhabaleswar K. (DK) Panda.
 *
 * For detailed copyright and licensing information, please refer to the
 * copyright file COPYRIGHT in the top level MVAPICH2-J directory.
 */

package mpi.topo;

import mpi.*;

public class sub2 {
  public sub2() {}

  public sub2(String[] args) throws Exception {

    int dims[] = new int[2];
    dims[0] = 2;
    dims[1] = 3;
    boolean periods[] = new boolean[2];
    int size, rank;
    MPI.Init(args);
    rank = MPI.COMM_WORLD.getRank();
    size = MPI.COMM_WORLD.getSize();

    if (size != 8) {
      if (rank == 0) System.out.println("topo->sub2: MUST RUN WITH 8 TASKS");

      MPI.COMM_WORLD.barrier();
      MPI.Finalize();
      return;
    }

    Cartcomm comm = MPI.COMM_WORLD.createCart(dims, periods, false);

    if (comm != null) {
      int[] dims2 = comm.getTopo().dims;
      boolean remain[] = new boolean[2];
      remain[0] = false;
      remain[1] = true;
      Cartcomm subcomm = comm.sub(remain);
      int nsize = subcomm.getSize();
      int nrank = subcomm.getRank();

      System.out.println(
          "rank <" + rank + ">,nrank<" + nrank + ">,size<" + size + ">,nsize <" + nsize);
    } else {
      System.out.println("rank <" + rank + ",size<" + size + ">");
    }

    MPI.COMM_WORLD.barrier();
    MPI.Finalize();
  }

  public static void main(String[] args) throws Exception {
    try {
      sub2 c = new sub2(args);
    } catch (Exception e) {
    }
  }
}
