#!/bin/sh 
#set -x

declare -a MPI_ENV_TESTS=('EnvTester' 'ErrStreamPrinter' 'abort' 'initialized' 'procname' 'wtime');

declare -a MPI_PT2PT_TESTS=('BreakANYSRC' 'BufferTest1' 'BufferTest3' 'getcount'
'iprobe' 'rsend' 'rsend2' 'sendrecv' 'sendrecv_rep' 'seq' 'ssend' 'test1'
'test2' 'test3' 'testall' 'testany' 'waitall' 'waitall2' 'waitany' 'waitnull'
'wildcard' 'SimpleNB' 'Send_rendez' 'Send_rendez_fairness' 'Send_eager_fairness'
'Ssend_test1' 'non_overtaking' 'intertwined' 'Isend_rendez'
'BlockingPt2ptWithArrays' 'BlockingPt2ptWithByteBuffer'
'NonBlockingPt2ptWithArrays' 'NonBlockingPt2ptWithArraysAndTest'
'NonBlockingPt2ptWithByteBuffer' 'NonBlockingPt2ptWithByteBufferAndTest'
'rSendWithArrays' 'irSendWithArrays' 'isSendWithArrays');

#Group Tests
declare -a MPI_GROUP_TESTS=('Group_com' 'Group_incl' 'Group_tranks' 'Group_diff' 'Group_intersect' 'Group_union' 'Group_excl' 'Group_self' 'group' 'range');

declare -a MPI_CCL_TESTS=('bcast' 'alltoall' 'gather' 'barrier' 'reduce' 'reduce2' 'ReduceWithByteBuffer' 'allgather' 'allreduce' 'allgatherv' 'alltoallv' 'gatherv' 'scatter' 'reduce_scatter' 'ReduceScatterWithByteBuffer' 'AllreduceWithByteBuffer' 'GatherWithByteBuffer' 'AllgatherWithByteBuffer' 'ScatterWithByteBuffer' 'AlltoAllWithByteBuffer' 'GathervWithByteBuffer' 'AllgathervWithByteBuffer' 'ScattervWithArrays' 'ScattervWithByteBuffer' 'AlltoallvWithByteBuffer');

#Runing test cases

cd ..

for testcase in ${MPI_ENV_TESTS[@]}; do
	echo "Running:   mpi.env.$testcase"
	LD_PRELOAD=${MPILIB}/lib/libmpi.so ${MPILIB}/bin/mpirun_rsh -export-all -np 2 -hostfile ${MV2J_HOME}/test/nativetest/hosts MV2_CPU_BINDING_LEVEL=SOCKET MV2_HOMOGENEOUS_CLUSTER=1 MV2_CPU_BINDING_POLICY=SCATTER java -cp ${MV2J_HOME}/lib/mvapich2-j.jar:. -Djava.library.path=${MV2J_HOME}/lib mpi.env.$testcase
	echo ""
done

for testcase in ${MPI_PT2PT_TESTS[@]}; do
	echo "Running:   mpi.pt2pt.$testcase"
	LD_PRELOAD=${MPILIB}/lib/libmpi.so ${MPILIB}/bin/mpirun_rsh -export-all -np 2 -hostfile ${MV2J_HOME}/test/nativetest/hosts MV2_CPU_BINDING_LEVEL=SOCKET MV2_HOMOGENEOUS_CLUSTER=1 MV2_CPU_BINDING_POLICY=SCATTER java -cp ${MV2J_HOME}/lib/mvapich2-j.jar:. -Djava.library.path=${MV2J_HOME}/lib mpi.pt2pt.$testcase
	echo ""
done

for testcase in ${MPI_CCL_TESTS[@]}; do
	echo "Running:   mpi.ccl.$testcase"
	LD_PRELOAD=${MPILIB}/lib/libmpi.so ${MPILIB}/bin/mpirun_rsh -export-all -np 4 -hostfile ${MV2J_HOME}/test/nativetest/hosts MV2_CPU_BINDING_LEVEL=SOCKET MV2_HOMOGENEOUS_CLUSTER=1 MV2_CPU_BINDING_POLICY=SCATTER java -cp ${MV2J_HOME}/lib/mvapich2-j.jar:. -Djava.library.path=${MV2J_HOME}/lib mpi.ccl.$testcase
	echo ""
done

for testcase in ${MPI_COMM_TESTS[@]}; do
	echo "Running:   mpi.comm.$testcase"
	LD_PRELOAD=${MPILIB}/lib/libmpi.so ${MPILIB}/bin/mpirun_rsh -export-all -np 8 -hostfile ${MV2J_HOME}/test/nativetest/hosts MV2_CPU_BINDING_LEVEL=SOCKET MV2_HOMOGENEOUS_CLUSTER=1 MV2_CPU_BINDING_POLICY=SCATTER java -cp ${MV2J_HOME}/lib/mvapich2-j.jar:. -Djava.library.path=${MV2J_HOME}/lib mpi.comm.$testcase
	echo ""
done

# split runs for 6 processors so writing it down here
echo "Running:   mpi.comm.split"
LD_PRELOAD=${MPILIB}/lib/libmpi.so ${MPILIB}/bin/mpirun_rsh -export-all -np 6 -hostfile ${MV2J_HOME}/test/nativetest/hosts MV2_CPU_BINDING_LEVEL=SOCKET MV2_HOMOGENEOUS_CLUSTER=1 MV2_CPU_BINDING_POLICY=SCATTER java -cp ${MV2J_HOME}/lib/mvapich2-j.jar:. -Djava.library.path=${MV2J_HOME}/lib mpi.comm.split

for testcase in ${MPI_GROUP_TESTS[@]}; do
	LD_PRELOAD=${MPILIB}/lib/libmpi.so ${MPILIB}/bin/mpirun_rsh -export-all -np 8 -hostfile ${MV2J_HOME}/test/nativetest/hosts MV2_CPU_BINDING_LEVEL=SOCKET MV2_HOMOGENEOUS_CLUSTER=1 MV2_CPU_BINDING_POLICY=SCATTER java -cp ${MV2J_HOME}/lib/mvapich2-j.jar:. -Djava.library.path=${MV2J_HOME}/lib mpi.group.$testcase
	echo ""
done
